@@ -22,23 +22,12 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  */
-#include "pfmon.h"
-
 #include <ctype.h>
 #include <regex.h>
 #include <math.h> /* for lroundf */
 #include <limits.h>
 
-extern pfmon_support_t pfmon_itanium;
-extern pfmon_support_t pfmon_itanium2;
-extern pfmon_support_t pfmon_montecito;
-extern pfmon_support_t pfmon_generic_ia64;
-extern pfmon_support_t pfmon_amd64;
-extern pfmon_support_t pfmon_i386_pm;
-extern pfmon_support_t pfmon_i386_p6;
-extern pfmon_support_t pfmon_pentium4;
-extern pfmon_support_t pfmon_gen_ia32;
-extern pfmon_support_t pfmon_coreduo;
+#include "pfmon_support.h"
 
 static pfmon_support_t *pfmon_cpus[]={
 #ifdef CONFIG_PFMON_ITANIUM
@@ -60,13 +49,21 @@ static pfmon_support_t *pfmon_cpus[]={
 	&pfmon_i386_p6,
 	&pfmon_i386_pm,
 #endif
-
 #ifdef CONFIG_PFMON_PENTIUM4
 	&pfmon_pentium4,
 #endif
+#ifdef CONFIG_PFMON_CORE
+	&pfmon_core,
+#endif
 #ifdef CONFIG_PFMON_GEN_IA32
 	&pfmon_coreduo,
 	&pfmon_gen_ia32,	/* must always be last of IA-32 choices */
+#endif
+#ifdef CONFIG_PFMON_MIPS64
+ 	&pfmon_mips64_20kc,
+ 	&pfmon_mips64_25kf,
+ 	&pfmon_mips64_ice9a,
+ 	&pfmon_mips64_ice9b,
 #endif
 	NULL
 };
@@ -293,11 +290,12 @@ prepare_pmd_registers(pfmon_event_set_t *set)
 int
 install_pmd_registers(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 {
+	int error;
 	/*
 	 * and the PMD registers
 	 */
-	if (pfmon_write_pmds(sdesc->ctxid, set, set->master_pd, set->event_count) == -1) {
-		warning( "cannot write PMDs: %s\n", strerror(errno));
+	if (pfmon_write_pmds(sdesc->ctxid, set, set->master_pd, set->event_count,&error) == -1) {
+		warning( "cannot write PMDs: %s\n", strerror(error));
 		return -1;
 	}
 	DPRINT(("sdesc->id=%u installed registers\n", sdesc->id));	
@@ -316,11 +314,12 @@ install_pmd_registers(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 int
 install_pmc_registers(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 {
+	int error;
 	/*
 	 * now program the PMC registers
 	 */
-	if (pfmon_write_pmcs(sdesc->ctxid, set, set->master_pc, set->pc_count) == -1) {
-		warning("cannot write PMCs: %s\n", strerror(errno));
+	if (pfmon_write_pmcs(sdesc->ctxid, set, set->master_pc, set->pc_count,&error) == -1) {
+		warning("cannot write PMCs: %s\n", strerror(error));
 		return -1;
 	}
 	/*
@@ -342,7 +341,7 @@ install_event_sets(pfmon_sdesc_t *sdesc)
 	pfmon_event_set_t *set;
 	pfarg_setdesc_t sd;
 	double offs;
-	int ret, ctxid;
+	int ret, ctxid, error;
 
 	ctxid = sdesc->ctxid;
 
@@ -351,32 +350,32 @@ install_event_sets(pfmon_sdesc_t *sdesc)
 			memset(&sd, 0, sizeof(sd));
 			sd.set_id      = set->id;
 			sd.set_flags   = set->set_flags;
-			sd.set_timeout = options.switch_timeout * 1000;
+			sd.set_timeout = options.switch_timeout * 1000000; /* msecs -> nsecs */
 
 			if (sdesc->nsets > 1)
 				sd.set_flags |= PFM_SETFL_TIME_SWITCH;
 
-			if (pfmon_create_evtsets(ctxid, &sd, 1) == -1) {
-				warning("cannot create event sets: %s\n", strerror(errno));
+			if (pfmon_create_evtsets(ctxid, &sd, 1, &error) == -1) {
+				warning("cannot create event sets: %s\n", strerror(error));
 				return -1;
 			}
 		}
-		options.eff_switch_timeout = sd.set_timeout / 1000;
+		options.eff_switch_timeout = sd.set_timeout / 1000000; /* nsecs -> msecs */
 
 		if (options.switch_timeout) {
 
 			offs  = (double)(options.eff_switch_timeout - options.switch_timeout)
 				*100.0/options.switch_timeout;
 
-			vbprintf("requested switch timeout %u msecs\n", options.switch_timeout);
-			vbprintf("effective switch timeout %u msecs (+%3.2f%%)\n", options.eff_switch_timeout, offs);
+			vbprintf("requested switch timeout %"PRIu64" msecs\n", options.switch_timeout);
+			vbprintf("effective switch timeout %"PRIu64" msecs (+%3.2f%%)\n", options.eff_switch_timeout, offs);
 			/*
 			 * warn user of excessive offset (2%)
 			 */
 			if (offs >= 2.0) {
 				printf("Warning: host kernel timer granularity only allows the following timeout:\n"
-					"\trequested switch timeout %u msecs\n"
-					"\teffective switch timeout %u msecs (+%3.2f%% offset)\n",
+					"\trequested switch timeout %"PRIu64" msecs\n"
+					"\teffective switch timeout %"PRIu64" msecs (+%3.2f%% offset)\n",
 					options.switch_timeout,
 					options.eff_switch_timeout, offs);
 			}
@@ -445,7 +444,7 @@ run_measurements(char **argv)
 	if (options.opt_check_evt_only) exit(0);
 
 	/*
-	 * final model-specific chance to setup for PMCs and PMDs (no actual perfmonctl() call).
+	 * final model-specific chance to setup for PMCs and PMDs.
 	 * we use the master copies.
 	 */
 	for (set = options.sets; set; set = set->next) {
@@ -486,7 +485,7 @@ run_measurements(char **argv)
 	 * initialize various context flags
 	 */
 	if (options.opt_syst_wide)
-		ctx.ctx_flags  = PFM_FL_SYSTEM_WIDE | PFM_FL_OVFL_NO_MSG; 
+		ctx.ctx_flags  = PFM_FL_SYSTEM_WIDE;
 
 	if (options.opt_block)
 		ctx.ctx_flags |= PFM_FL_NOTIFY_BLOCK;
@@ -532,38 +531,27 @@ run_measurements(char **argv)
 }
 
 static void
-show_detailed_event_name(unsigned int idx, const char *name)
+show_event_name(unsigned int idx, const char *name, int mode)
 {
-	pfmlib_regmask_t used_counters, impl_pmds;
-	unsigned int num_pmds;
-	int i;
-	int code;
+	size_t l;
+	unsigned int n, i;
+	char *mask_name;
 
-	memset(&used_counters, 0, sizeof(used_counters));
-	memset(&impl_pmds, 0, sizeof(impl_pmds));
+	pfm_get_max_event_name_len(&l);
+	mask_name = malloc(l+1);
+	if (!mask_name)
+		fatal_error("cannot allocate memory for mask name\n");
 
-	pfm_get_num_pmds(&num_pmds);
-
-	pfm_get_event_code(idx, &code);
-	pfm_get_event_counters(idx, &used_counters);
-
-	printf("%s code=0x%02x counters=[ ", name, code);
-
-	/*
-	 * cannot used pfm_get_num_counters() because we are interested
-	 * in the number of counters supported by this event
-	 */
-	for (i=0; i < num_pmds; i++) {
-		if (pfm_regmask_isset(&used_counters, i)) printf("%d ", i);
+	pfm_get_num_event_masks(idx, &n);
+	if (n == 0 || mode == 0) {
+		printf("%s\n", name);
+		return;
 	}
-
-	putchar(']');
-	putchar(' ');
-
-	if (pfmon_current->pfmon_detailed_event_name) {
-		(*pfmon_current->pfmon_detailed_event_name)(idx);
+	for (i = 0; n; n--, i++) {
+		pfm_get_event_mask_name(idx, i, mask_name, l);
+		printf("%s:%s\n", name, mask_name);
 	}
-	putchar('\n');
+	free(mask_name);
 }
 
 /*
@@ -575,39 +563,39 @@ pfmon_list_all_events(char *pattern, int mode)
 {
 	regex_t preg;
 	unsigned int i, count;
-	char name[PFMON_MAX_EVTNAME_LEN];
+	char *name;
+	size_t len;
 
+	pfm_get_max_event_name_len(&len);
 	pfm_get_num_events(&count);
 
+	name = malloc(len+1);
+	if (!name)
+		fatal_error("cannot allocate memory for event name\n");
+
 	if (pattern) {
 		int done = 0;
 
-		if (regcomp(&preg, pattern, REG_ICASE|REG_NOSUB)) {
+		if (regcomp(&preg, pattern, REG_ICASE|REG_NOSUB))
 			fatal_error("error in regular expression for event \"%s\"\n", pattern);
-		}
 
 		for(i=0; i < count; i++) {
-			pfm_get_event_name(i, name, PFMON_MAX_EVTNAME_LEN);
+			pfm_get_event_name(i, name, len);
 
 			if (regexec(&preg, name, 0, NULL, 0) == 0) {
-				if (mode == 0) 
-					printf("%s\n", name);
-				else {
-					show_detailed_event_name(i, name);
-				}
+				show_event_name(i, name, mode);
 				done = 1;
 			}
 		}
-		if (done == 0) fatal_error("event not supported\n");
+		if (done == 0)
+			fatal_error("event not found\n");
 	} else {
 		for(i=0; i < count; i++) {
-			pfm_get_event_name(i, name, PFMON_MAX_EVTNAME_LEN);
-			if (mode == 0) 
-				printf("%s\n", name);
-			else
-				show_detailed_event_name(i, name);
+			pfm_get_event_name(i, name, len);
+			show_event_name(i, name, mode);
 		}
 	}
+	free(name);
 }
 
 /*
@@ -648,9 +636,9 @@ static struct option pfmon_common_options[]={
 	{ "cpu-list", 1, 0, 30},
 	{ "trigger-data-start-addresses", 1, 0, 31},
 	{ "trigger-data-stop-addresses", 1, 0, 32},
-	//{ "interval", 2, 0, 33},
+	{ "print-interval", 1, 0, 33},
 	{ "switch-timeout", 1, 0, 35},
-	//{ "excl-idle", 0, 0, 36},
+	// 36 is free
 	{ "extra-smpl-pmds", 1, 0, 37},
 
 	{ "verbose", 0, &options.opt_verbose, 1 },
@@ -683,7 +671,6 @@ static struct option pfmon_common_options[]={
 	{ "exec-split-results", 0, &options.opt_split_exec, 1},
 	{ "resolve-addresses", 0, &options.opt_addr2sym, 1},
 	{ "saturate-smpl-buffer", 0, &options.opt_no_ovfl_notify, 1},
-	{ "print-interval", 0, &options.opt_print_interval, 1},
 	{ "dont-start", 0, &options.opt_dont_start, 1},
 	{ "pin-command", 0, &options.opt_pin_cmd, 1},
 	{ "print-syms", 0, &options.opt_print_syms, 1},
@@ -708,126 +695,125 @@ usage(char **argv)
 	printf(	"-h, --help\t\t\t\tDisplay this help and exit.\n"
 		"-V, --version\t\t\t\tOutput version information and exit.\n"
 		"-l[regex], --show-events[=regex]\tDisplay all or a matching subset of\n"
-		"\t\t\t\t\t the events.\n"
-		"--long-show-events[=regex]\t\tDisplay all or a matching subset of\n"
-		"\t\t\t\t\t the events with info.\n"
+		"\t\t\t\t\tthe events.\n"
+		"-L, --long-show-events[=regex]\t\tdisplay matching events names and "
+		"unit masks\n"
 		"-i event, --event-info=event\t\tDisplay information about an event\n"
-		"\t\t\t\t\t (numeric code or regex).\n"
+		"\t\t\t\t\t(numeric code or regex).\n"
 		"-u, -3 --user-level\t\t\tMonitor at the user level for all\n"
-		"\t\t\t\t\t events (default: on).\n"
+		"\t\t\t\t\tevents (default: on).\n"
 		"-k, -0 --kernel-level\t\t\tMonitor at the kernel level for all\n"
-		"\t\t\t\t\t events (default: off).\n"
+		"\t\t\t\t\tevents (default: off).\n"
 		"-1\t\t\t\t\tMonitor execution at privilege level 1\n"
-		"\t\t\t\t\t (default: off).\n"
+		"\t\t\t\t\t(default: off).\n"
 		"-2\t\t\t\t\tMonitor execution at privilege level 2\n"
-		"\t\t\t\t\t (default: off).\n"
+		"\t\t\t\t\t(default: off).\n"
 		"-e, --events=ev1,ev2,...\t\tSelect events to monitor.\n"
 		"-I,--info\t\t\t\tList supported PMU models and compiled\n"
-		"\t\t\t\t\t in sampling output formats.\n"
+		"\t\t\t\t\tin sampling output formats.\n"
 		"-t secs, --session-timeout=secs\t\tDuration of the system wide session in\n"
-		"\t\t\t\t\t seconds.\n"
+		"\t\t\t\t\tseconds.\n"
 		"-S format, --smpl-module-info=format\tDisplay information about a sampling\n"
-		"\t\t\t\t\t output format.\n"
+		"\t\t\t\t\toutput format.\n"
 		"--debug\t\t\t\t\tEnable debug prints.\n"
 		"--verbose\t\t\t\tPrint more information during execution.\n"
 		"--outfile=filename\t\t\tPrint results in a file.\n"
 		"--append\t\t\t\tAppend results to outfile.\n"
 		"--overflow-block\t\t\tBlock the task when sampling buffer is\n"
-		"\t\t\t\t\t full (default: off).\n"
+		"\t\t\t\t\tfull (default: off).\n"
 		"--system-wide\t\t\t\tCreate a system wide monitoring session\n"
-		"\t\t\t\t\t (default: per-task).\n"
+		"\t\t\t\t\t(default: per-task).\n"
 		"--smpl-outfile=filename\t\t\tFile to save the sampling results.\n"
 		"--long-smpl-periods=val1,val2,...\tSet sampling period after user\n"
-		"\t\t\t\t\t notification.\n"
+		"\t\t\t\t\tnotification.\n"
 		"--short-smpl-periods=val1,val2,...\tSet sampling period.\n"
 		"--smpl-entries=n\t\t\tNumber of entries in sampling buffer.\n"
 		"--with-header\t\t\t\tGenerate a header for results.\n"
 		"--cpu-list=num,num1-num2,...\t\tSpecify list, via numbers, of CPUs for\n"
 		"\t\t\t\t\t system-wide session (default: all).\n"
 		"--aggregate-results\t\t\tAggregate counts and sampling buffer\n"
-		"\t\t\t\t\t outputs for multi CPU monitoring\n"
-		"\t\t\t\t\t (default: off).\n"
+		"\t\t\t\t\toutputs for multi CPU monitoring\n"
+		"\t\t\t\t\t(default: off).\n"
 		"--trigger-code-start-address=addr\tStart monitoring only when code address\n"
-		"\t\t\t\t\t is executed.\n"
+		"\t\t\t\t\tis executed.\n"
 		"--trigger-code-stop-address=addr\tStop monitoring when code address is\n"
-		"\t\t\t\t\t executed.\n"
+		"\t\t\t\t\texecuted.\n"
 		"--trigger-data-start-address=addr\tStart monitoring only when data address\n"
-		"\t\t\t\t\t is accessed.\n"
+		"\t\t\t\t\tis accessed.\n"
 		"--trigger-data-stop-address=addr\tStop monitoring when data address code\n"
-		"\t\t\t\t\t is accessed.\n"
+		"\t\t\t\t\tis accessed.\n"
 		"--trigger-code-repeat\t\t\tStart/stop monitoring each time trigger\n"
-		"\t\t\t\t\t start/stop are executed.\n"
+		"\t\t\t\t\tstart/stop are executed.\n"
 		"--trigger-code-follow\t\t\tStart/stop code trigger applied to all\n"
-		"\t\t\t\t\t monitored task (default first only).\n"
+		"\t\t\t\t\tmonitored task (default first only).\n"
 		"--trigger-data-repeat\t\t\tStart/stop monitoring each time trigger\n"
-		"\t\t\t\t\t start/stop are accessed.\n"
+		"\t\t\t\t\tstart/stop are accessed.\n"
 		"--trigger-data-follow\t\t\tStart/stop data trigger applied to all\n"
-		"\t\t\t\t\t monitored task (default first only).\n"
+		"\t\t\t\t\tmonitored task (default first only).\n"
 		"--trigger-data-ro\t\t\tData trigger activated on read access\n"
-		"\t\t\t\t\t (default read-write).\n"
+		"\t\t\t\t\t(default read-write).\n"
 		"--trigger-data-wo\t\t\tData trigger activated on write access\n"
-		"\t\t\t\t\t (default read-write).\n"
+		"\t\t\t\t\t(default read-write).\n"
 		"--trigger-start-delay=secs\t\tNumber of seconds before activating\n"
-		"\t\t\t\t\t monitoring.\n"
+		"\t\t\t\t\tmonitoring.\n"
 		"--priv-levels=lvl1,lvl2,...\t\tSet privilege level per event (any\n"
-		"\t\t\t\t\t combination of [0123uk]).\n"
+		"\t\t\t\t\tcombination of [0123uk]).\n"
 		"--us-counter-format\t\t\tPrint counters using commas\n"
-		"\t\t\t\t\t (e.g., 1,024).\n"
+		"\t\t\t\t\t(e.g., 1,024).\n"
 		"--eu-counter-format\t\t\tPrint counters using points\n"
-		"\t\t\t\t\t (e.g., 1.024).\n"
+		"\t\t\t\t\t(e.g., 1.024).\n"
 		"--hex-counter-format\t\t\tPrint counters in hexadecimal\n"
-		"\t\t\t\t\t (e.g., 0x400).\n"
+		"\t\t\t\t\t(e.g., 0x400).\n"
 		"--smpl-module=name\t\t\tSelect sampling module, use -I to list\n"
-		"\t\t\t\t\t modules.\n"
+		"\t\t\t\t\tmodules.\n"
 		"--show-time\t\t\t\tShow real,user, and system time for the\n"
-		"\t\t\t\t\t command executed.\n"
+		"\t\t\t\t\tcommand executed.\n"
 		"--symbol-file=filename\t\t\tELF image containing a symbol table.\n"
 		"--sysmap-file=filename\t\t\tSystem.map-format file containing a\n"
-		"\t\t\t\t\t symbol table.\n"
+		"\t\t\t\t\tsymbol table.\n"
 		"--check-events-only\t\t\tVerify combination of events and exit\n"
-		"\t\t\t\t\t (no measurement).\n"
+		"\t\t\t\t\t(no measurement).\n"
 		"--smpl-periods-random=mask1:seed1,...\tApply randomization to long and short\n"
-		"\t\t\t\t\t periods.\n"
+		"\t\t\t\t\tperiods.\n"
 		"--smpl-print-counts\t\t\tPrint counters values when sampling\n"
-		"\t\t\t\t\t session ends (default: no).\n"
+		"\t\t\t\t\tsession ends (default: no).\n"
 		"--attach-task pid\t\t\tMonitor process identified by pid.\n"
 		"--reset-non-smpl-periods\t\tOn overflow reset counters not used as\n"
-		"\t\t\t\t\t sampling periods.\n"
+		"\t\t\t\t\tsampling periods.\n"
 		"--follow-fork\t\t\t\tMonitoring continues across fork.\n"
 		"--follow-vfork\t\t\t\tMonitoring continues across vfork.\n"
 		"--follow-pthread\t\t\tMonitoring continues across\n"
-		"\t\t\t\t\t pthread_create.\n"
+		"\t\t\t\t\tpthread_create.\n"
 		"--follow-exec[=pattern]\t\t\tFollow exec with optional command\n"
-		"\t\t\t\t\t pattern.\n"
+		"\t\t\t\t\tpattern.\n"
 		"--follow-exec-exclude=pattern\t\tFollow exec but exclude commands\n"
-		"\t\t\t\t\t matching the pattern.\n"
+		"\t\t\t\t\tmatching the pattern.\n"
 		"--follow-all\t\t\t\tFollow fork, vfork, exec, pthreads.\n"
 		"--no-cmd-output\t\t\t\tRedirect all output of executed commands\n"
-		"\t\t\t\t\t to /dev/null.\n"
+		"\t\t\t\t\tto /dev/null.\n"
 		"--exec-split-results\t\t\tGenerate separate results output on\n"
-		"\t\t\t\t\t execve().\n"
+		"\t\t\t\t\texecve().\n"
 		"--resolve-addresses\t\t\tTry to resolve code/data addresses to\n"
-		"\t\t\t\t\t symbols.\n"
+		"\t\t\t\t\tsymbols.\n"
 		"--extra-smpl-pmds=num,num1-num2,...\tSpecify a list of extra PMD register to\n"
-		"\t\t\t\t\t include in samples.\n"
+		"\t\t\t\t\tinclude in samples.\n"
 
 #ifdef CONFIG_PFMON_DEMANGLE
 		"--demangle-cpp\t\t\t\tC++ symbol demangling (default: off).\n"
 		"--demangle-java\t\t\t\tJava symbol demangling (default: off).\n"
 #endif
 		"--saturate-smpl-buffer\t\t\tOnly collect samples until buffer\n"
-		"\t\t\t\t\t becomes full.\n"
-		"--interval[=n]\t\t\t\tNumber of seconds to multiplex all event\n"
-		"\t\t\t\t\t sets (default 1s).\n"
+		"\t\t\t\t\tbecomes full.\n"
+		"--print-interval=n\t\t\t\tnumber of milliseconds between prints of count deltas\n"
 		"--pin-command\t\t\t\tPin executed command on --cpu-list\n"
 		"\t\t\t\t\t (system-wide only).\n"
-		"--switch-timeout=microseconds\t\tThe number of microseconds before\n"
+		"--switch-timeout=milliseconds\t\tThe number of milliseconds before\n"
 		"\t\t\t\t\t switching to the next event set.\n"
 		"--dont-start\t\t\t\tDo not activate monitoring in pfmon\n"
-		"\t\t\t\t\t (per-thread only).\n"
-		"\t\t\t\t\t measurement.\n"
+		"\t\t\t\t\t(per-thread only).\n"
+		"\t\t\t\t\tmeasurement.\n"
 		"--cpu-set-relative\t\t\tCPU identifications relative to cpu_set\n"
-		"\t\t\t\t\t affinity (default: off).\n"
+		"\t\t\t\t\taffinity (default: off).\n"
 	);
 
 	if (pfmon_current->pfmon_usage) pfmon_current->pfmon_usage();
@@ -1156,9 +1142,9 @@ static int
 pfmon_show_event_info(char *event)
 {
 	regex_t preg;
-	int found = 0;
+	int code, ret, found = 0;
 	unsigned int i, c;
-	int code, ret;
+	char *p;
 	unsigned long lcode;
 	char name[PFMON_MAX_EVTNAME_LEN];
 
@@ -1176,6 +1162,9 @@ pfmon_show_event_info(char *event)
 		__pfmon_show_event_info(c);
 		return 0;
 	}
+	p = strchr(event, ':');
+	if (p)
+		*p = '\0';
 
 	if (regcomp(&preg, event, REG_ICASE|REG_NOSUB))
 		fatal_error("error in regular expression for event \"%s\"\n", event);
@@ -1242,15 +1231,15 @@ pfmon_initialize(char **argv)
 	struct timespec ts;
 	char *str;
 	size_t len;
+	unsigned int version;
 	int ret;
-#define VERSION_22 (2U<<16|2U)
 
 	pfmon_get_version();
 
 	/*
 	 * check if kernel supports event sets
 	 */
-	if (options.pfm_version >= VERSION_22)
+	if (options.pfm_version >= PERFMON_VERSION_22)
 		options.opt_is22 = 1;
 
 	pfmon_collect_affinity();
@@ -1263,6 +1252,11 @@ pfmon_initialize(char **argv)
 	if (pfm_initialize() != PFMLIB_SUCCESS) 
 		fatal_error("cannot initialize library. Most likely host PMU is not supported.\n");
 
+	pfm_get_version(&version);
+	if (PFM_VERSION_MAJOR(version) < 3 || PFM_VERSION_MINOR(version) < 2)
+		fatal_error("linked with libpfm v%u.%u, needs at least v3.2\n",
+			PFM_VERSION_MAJOR(version),PFM_VERSION_MINOR(version));
+
 	ret = pfm_get_pmu_type(&options.pmu_type);
 	if (ret != PFMLIB_SUCCESS) {
 		fatal_error("cannot determine PMU type\n");
@@ -1284,6 +1278,9 @@ pfmon_initialize(char **argv)
 
 	pfmon_detect();
 
+	options.session_timeout = PFMON_NO_TIMEOUT;
+	options.interval = PFMON_NO_TIMEOUT;
+
 	/*
 	 * collect some system parameters
 	 */
@@ -1300,7 +1297,6 @@ pfmon_initialize(char **argv)
 	options.config_cpus = long_val;
 
 	clock_getres(CLOCK_REALTIME, &ts);
-
 	long_val = lroundf(1000000000.0 / (double)ts.tv_nsec);
 	if (long_val == -1) 
 		fatal_error("cannot figure out the clock tick\n");
@@ -1388,11 +1384,11 @@ setup_plm(pfmon_event_set_t *set)
 			while (*arg && *arg != ',') {
 				switch (*arg) {
 					case 'k':
-					case '0': val |= 1; break;
-					case '1': val |= 2; break;
-					case '2': val |= 4; break;
+					case '0': val |= PFM_PLM0; break;
+					case '1': val |= PFM_PLM1; break;
+					case '2': val |= PFM_PLM2; break;
 					case '3': 
-					case 'u': val |= 8; break;
+					case 'u': val |= PFM_PLM3; break;
 					default: goto error;
 				}
 				arg++;
@@ -1417,7 +1413,8 @@ too_many:
 static int
 pfmon_check_extra_options(int c, char *optarg)
 {
-	if (pfmon_current->pfmon_parse_options && pfmon_current->pfmon_parse_options(c, optarg) == 0) {
+	if (pfmon_current->pfmon_parse_options
+	    && pfmon_current->pfmon_parse_options(c, optarg) == 0) {
 		return 0;
 	}
 
@@ -1443,11 +1440,11 @@ populate_cpumask(char *cpu_list)
 			fatal_error("pfmon can only handle to %u CPUs\n", PFMON_MAX_CPUS);
 
 		for(i=0, j=0; i < options.online_cpus; i++) {
-			if (PFMON_CPUMASK_ISSET(options.phys_cpu_mask, i)) {
+			if (pfmon_bitmask_isset(options.phys_cpu_mask, i)) {
 				if (options.opt_vcpu) {
-					PFMON_CPUMASK_SET(options.virt_cpu_mask, j);
+					pfmon_bitmask_set(options.virt_cpu_mask, j);
 				} else {
-					PFMON_CPUMASK_SET(options.virt_cpu_mask, i);
+					pfmon_bitmask_set(options.virt_cpu_mask, i);
 
 				}
 				j++;
@@ -1487,10 +1484,10 @@ populate_cpumask(char *cpu_list)
 			if (phys_cpu == -1)
 				goto no_access;
 
-			if(!PFMON_CPUMASK_ISSET(options.phys_cpu_mask, phys_cpu))
+			if(!pfmon_bitmask_isset(options.phys_cpu_mask, phys_cpu))
 					goto no_access;
 
-			PFMON_CPUMASK_SET(options.virt_cpu_mask, start_cpu);
+			pfmon_bitmask_set(options.virt_cpu_mask, start_cpu);
 
 			count++;
 		}
@@ -1509,7 +1506,7 @@ end:
 
 		count = options.selected_cpus;
 		for(i=0; count;i++) {
-			if (PFMON_CPUMASK_ISSET(options.virt_cpu_mask, i) == 0) continue;
+			if (pfmon_bitmask_isset(options.virt_cpu_mask, i) == 0) continue;
 			vbprintf("CPU%lu ", i);
 			count--;
 		}
@@ -1571,7 +1568,7 @@ pfmon_verify_cmdline_options(int argc, char **argv)
 
 		populate_cpumask(options.cpu_list);
 
-		if (optind != argc && options.session_timeout > 0)
+		if (optind != argc && options.session_timeout != PFMON_NO_TIMEOUT)
 			fatal_error("you cannot use both a timeout and command in system-wide mode\n");
 
 		if (options.opt_block == 1) 
@@ -1595,6 +1592,8 @@ pfmon_verify_cmdline_options(int argc, char **argv)
 		if (options.opt_split_exec)
 			warning("--exec-split has not effect in system-wide mode\n");
 
+		if (options.opt_aggr && options.interval != PFMON_NO_TIMEOUT)
+			fatal_error("--print-interval is not supported with --aggregate-results\n");
 	} else {
 		/* wait4 use RUSAGE_BOTH */
 		if (options.opt_show_rusage && (options.opt_follow_fork || options.opt_follow_vfork || options.opt_follow_pthread))
@@ -1619,7 +1618,8 @@ pfmon_verify_cmdline_options(int argc, char **argv)
 
 		if (options.opt_follow_exec && has_code_data_triggers && (options.opt_code_trigger_follow ||options.opt_data_trigger_follow) )
 			fatal_error("cannot use code/data trigger follow with --follow-exec option\n");
-
+		if (options.interval != PFMON_NO_TIMEOUT)
+			fatal_error("--print-interval is not supported in per-thread mode\n");
 	}
 
 	/*
@@ -1659,7 +1659,7 @@ pfmon_verify_event_sets(void)
 }
 
 static void
-pfmon_setup_event_sets(unsigned int switch_timeout)
+pfmon_setup_event_sets(uint64_t switch_timeout)
 {
 	pfmon_event_set_t *set;
 
@@ -1685,7 +1685,7 @@ main(int argc, char **argv)
 	char 			*endptr = NULL;
 	pfmon_smpl_module_t	*smpl_mod = NULL;
 	unsigned long 		long_val;
-	unsigned int		switch_timeout = 0;
+	uint64_t		switch_timeout = 0;
 	int 			c, r, ret;
 	int			using_def_set = 1;
 
@@ -1693,7 +1693,7 @@ main(int argc, char **argv)
 
 	pfmon_initialize(argv);
 
-	while ((c=getopt_long(argc, argv,"+0123kuvhe:Il::i:Vt:S:p:", pfmon_cmd_options, 0)) != -1) {
+	while ((c=getopt_long(argc, argv,"+0123kuvhe:Il::L::i:Vt:S:p:", pfmon_cmd_options, 0)) != -1) {
 		switch(c) {
 			case   0: continue; /* fast path for options */
 
@@ -1765,13 +1765,14 @@ main(int argc, char **argv)
 			case 'V':
 			case   7:
 				printf("pfmon version " PFMON_VERSION " Date: " __DATE__ "\n"
-					"Copyright (C) 2001-2004 Hewlett-Packard Company\n");
+					"Copyright (C) 2001-2006 Hewlett-Packard Company\n");
 				exit(0);
 
 			case   8:
 				options.outfile = optarg;
 				break;
 
+			case 'L':
 			case   9:
 				pfmon_list_all_events(optarg, 1);
 				exit(0);
@@ -1812,8 +1813,11 @@ main(int argc, char **argv)
 
 			case 't':
 			case 16 :
-				if (options.session_timeout) fatal_error("too many session timeouts\n");
-				if (*optarg == '\0') fatal_error("--session-timeout needs an argument\n");
+				if (options.session_timeout != PFMON_NO_TIMEOUT)
+						fatal_error("too many session timeouts\n");
+				if (*optarg == '\0')
+					fatal_error("--session-timeout needs an argument\n");
+
 			  	long_val = strtoul(optarg,&endptr, 10);
 				if (*endptr != '\0') 
 					fatal_error("invalid number of seconds for session timeout: %s\n", optarg);
@@ -1821,8 +1825,7 @@ main(int argc, char **argv)
 				if (long_val >= UINT_MAX) 
 					fatal_error("timeout is too big, must be < %u\n", UINT_MAX);
 
-				options.session_timeout = (unsigned int)long_val;
-
+				options.session_timeout = long_val;
 				break;
 
 			case 17 :
@@ -1970,28 +1973,17 @@ main(int argc, char **argv)
 				break;
 
 			case 33 : 
-				if (*optarg == '\0') {
-					options.interval = PFMON_DFL_INTERVAL;
-				} else {
-					long_val = strtoul(optarg,&endptr, 10);
-					if (*endptr != '\0') 
-						fatal_error("invalid number of seconds for interval: %s\n", optarg);
-					if (long_val >= UINT_MAX) 
-						fatal_error("interval is too big, must be < %u\n", UINT_MAX);
-					options.interval = long_val;
-				}
+				long_val = strtoull(optarg,&endptr, 10);
+				if (*endptr != '\0') 
+					fatal_error("invalid number of milliseconds for print interval: %s\n", optarg);
+				options.interval = long_val;
 				break;
 			case 35 :
 				if (switch_timeout) fatal_error("too many switch timeouts\n");
-				if (*optarg == '\0') fatal_error("--switch-timeout needs an argument\n");
-			  	long_val = strtoul(optarg,&endptr, 10);
+			  	switch_timeout = strtoull(optarg,&endptr, 10);
 				if (*endptr != '\0') 
-					fatal_error("invalid number of microseconds for switch timeout: %s\n", optarg);
+					fatal_error("invalid number of milliseconds for switch timeout: %s\n", optarg);
 
-				if (long_val >= UINT_MAX || long_val == 0)
-					fatal_error("number of microseconds must be between [1-4294967296) for switch timeout\n");
-
-				switch_timeout = long_val;
 				break;
 			case 37 :
 				if (options.last_set == NULL)
