@@ -60,14 +60,25 @@ typedef pfarg_pmc_t	pfmon_pmc_t;
 typedef pfarg_setdesc_t	pfmon_setdesc_t;
 typedef pfarg_setinfo_t	pfmon_setinfo_t;
 
+#ifndef __ia64__
+/*
+ * definition for ia-64 perfmon v2.0 sampling format identification
+ * we forced definition on non ia-64 to avoid clobbering the code with
+ * #ifdef
+ */
+typedef unsigned char pfm_uuid_t[16];
+#endif
+
 /*
  * pfmon context description structure
  */
 typedef struct {
 	uint32_t	ctx_flags;
 	size_t		ctx_arg_size;
+	size_t		ctx_map_size;
 	void		*ctx_arg;
-	pfm_uuid_t	ctx_uuid;
+	pfm_uuid_t	ctx_uuid; /* legacy v2.0 */
+	char		*fmt_name;
 } pfmon_ctx_t;
 
 #define PFMON_VERSION		"3.2"
@@ -101,8 +112,18 @@ typedef enum {
  * max number of cpus (threads) supported
  */
 #define PFMON_MAX_CPUS		2048 /* MUST BE power of 2 */
-#define PFMON_CPUMASK_BITS	(sizeof(unsigned long)<<3)
-#define PFMON_CPUMASK_COUNT	(PFMON_MAX_CPUS/PFMON_CPUMASK_BITS)
+/*
+ * max number of PMU models supported
+ */
+#define PFMON_MAX_PMUS		256	/* MUST BE power of 2 */
+
+#define PFMON_BITMASK_BITS	(sizeof(unsigned long)<<3)
+
+#if PFMON_MAX_CPUS > PFMON_MAX_PMUS
+#define PFMON_BITMASK_COUNT	(PFMON_MAX_CPUS/PFMON_BITMASK_BITS)
+#else
+#define PFMON_BITMASK_COUNT	(PFMON_MAX_PMUS/PFMON_BITMASK_BITS)
+#endif
 
 #define PFMON_MAX_FILENAME_LEN	256	/* max for output/input files */
 #define PFMON_MAX_CMDLINE_LEN	1024	/* per-task mode max cmdline length /proc/cmdline */
@@ -122,8 +143,6 @@ typedef enum {
 
 #define PFMON_DFL_SMPL_ENTRIES	2048	/* default number of entries in sampling buffers */
 
-#define PFMON_ANY_PMU_MASK	(~0)	/* sampling module matches any PMU model */
-
 /*
  * sampling rates type definition
  */
@@ -174,12 +193,13 @@ typedef struct _pfmon_event_set_t {
 	uint32_t		set_flags;		/* flags for set creation */
 	void			*mod_args;		/* model specific extensions */
 	char			*events_str;		/* comma separated list of events for the set */
+	uint64_t		prev_nruns;		/* previous number of runs (incremental) */
 	uint64_t		nruns;			/* number of times the set was loaded on the PMU */
 	uint64_t		duration;		/* active duration in cycles */
 	unsigned int		id;			/* logical index */
 } pfmon_event_set_t;
 
-typedef unsigned long pfmon_cpumask_t[PFMON_CPUMASK_COUNT];
+typedef unsigned long pfmon_bitmask_t[PFMON_BITMASK_COUNT];
 
 /*
  * pfmon sampling descriptor. contains information about a
@@ -193,21 +213,15 @@ typedef struct {
 	uint64_t	 entry_count;	/* number of entries recorded for this buffer */
 	uint64_t	 last_ovfl;	/* last buffer overflow observed XXX: format specific */
 	size_t		last_count;	/* last number of entries in ovferflowed buffer */
+	size_t		 map_size;	/* remapped buffer size */
 	pid_t		 pid;		/* process which owns this context */
 	unsigned int	 cpu;		/* on which CPU does this apply to (system wide) */
 	int 		 processing;
 	void 		 *data;		/* sampling module-specific session data */
 } pfmon_smpl_desc_t;
 
-/*
- * sampling output format description
- */
-typedef uint64_t pfmon_pmu_mask_t;
-
 #define PFMON_CHECK_VERSION(h)	(PFM_VERSION_MAJOR((h)) != PFM_VERSION_MAJOR(PFMON_SMPL_VERSION))
 
-#define PFMON_PMU_MASK(t)	(((pfmon_pmu_mask_t)1)<<(t))
-
 /*
  * pfmon monitor definition
  */
@@ -290,7 +304,6 @@ typedef struct _pfmon_desc_t {
 	pid_t			pid;				/* process identification */
 	pid_t			ppid;				/* parent process identification */
 	pid_t			tid;				/* thread identification */
-	pid_t			ptid;				/* parent thread identification */
 
 	pfmon_ctxid_t		ctxid;				/* perfmon session identifier */
 	pfmon_smpl_desc_t	csmpl;				/* sampling descriptor */
@@ -317,6 +330,7 @@ typedef struct _pfmon_desc_t {
 	char			new_cmdline[PFMON_MAX_CMDLINE_LEN];	/* process command line from exec() */
 
 	FILE			*out_fp;				/* result file pointer */
+	int			done_header;				/* true if header printed */
 
 	pfmon_syms_list_t	primary_syms;			/* symbol lists for main binary (not-attach, not syswide) */
 } pfmon_sdesc_t;
@@ -355,6 +369,7 @@ typedef struct {
 	int			(*init_ctx_arg)(pfmon_ctx_t *ctx, unsigned int max_pmds_sample);
 	int			(*parse_options)(int code, char *optarg);
 	void			(*show_options)(void);
+	void			(*initialize_mask)(void);
 	int			(*initialize_module)(void);
 	int			(*initialize_session)(pfmon_smpl_desc_t *csmpl);
 	int			(*terminate_session)(pfmon_sdesc_t *sdesc);
@@ -362,13 +377,15 @@ typedef struct {
 	int			(*validate_events)(pfmon_event_set_t *set);
 	char			*name;		/* name of the format */
 	char			*description;	/* one line of text describing the format */
-	pfmon_pmu_mask_t	pmu_mask;	/* mask of which PMUs are supported */
-	pfm_uuid_t		uuid;		/* UUID for the kernel module used */
+	pfmon_bitmask_t		pmu_mask;	/* supported PMUs (valid after initialize_mask) */
+	pfm_uuid_t		uuid;		/* UUID for format (perfmon v2.0 leagcy) */
+	char			*fmt_name;	/* name of format */
 	int			flags;		/* module flags */
 } pfmon_smpl_module_t;
 
 #define PFMON_SMPL_MOD_FL_LEGACY	0x1	/* IA64: module support perfmon v2.x (x<2) only */
 #define PFMON_SMPL_MOD_FL_DEF_SYM	0x2	/* defer symbol hash table to terminate session */
+#define PFMON_SMPL_MOD_FL_PEBS		0x4	/* use Intel PEBS */
 
 typedef struct {
 	struct {
@@ -408,7 +425,6 @@ typedef struct {
 		int opt_split_exec;	  /* split result output on execve() */
 		int opt_support_gen;	  /* operating in a generic PMU mode is supported */
 		int opt_addr2sym;	  /* try converting addresses to symbol */
-		int opt_print_interval;	  /* print partial results at the end of multiplex interval */
 		int opt_pin_cmd;	  /* in system-wide, pin executed command to cpu-list processors only */
 		int opt_print_syms;	  /* print primary sysmbol table */
 		int opt_is22;	  	  /* host kernel implements at least perfmon v2.2 */
@@ -428,16 +444,17 @@ typedef struct {
 
 	pid_t	attach_pid;		  /* process to attach to */
 
-	pfmon_cpumask_t virt_cpu_mask;	  /* user specified CPUs for system-wide */
-	pfmon_cpumask_t phys_cpu_mask;	  /* actual CPUS affinity for pfmon upon start (cpu_set) */
+	pfmon_bitmask_t virt_cpu_mask;	  /* user specified CPUs for system-wide */
+	pfmon_bitmask_t phys_cpu_mask;	  /* actual CPUS affinity for pfmon upon start (cpu_set) */
 
 	unsigned int	trigger_delay; 	  /* number of seconds to wait before start a session */
 	unsigned int	session_timeout;  /* number of seconds to wait before stopping a session */
-	unsigned int	switch_timeout;	  /* requested switch timeout in ms */
-	unsigned int 	eff_switch_timeout;/* effective switch timeout in ms */
-	unsigned long	interval;	  /* number of seconds between two partial counts prints */
+
+	uint64_t	switch_timeout;	  /* requested switch timeout in ms */
+	uint64_t	eff_switch_timeout;/* effective switch timeout in ms */
+
+	uint64_t	interval;	  /* number of seconds between two partial counts prints */
 	unsigned long   smpl_entries;     /* number of entries in sampling buffer */
-	int		dfl_plm;	  /* default plm */
 
 	char		*code_trigger_start;	/* code trigger start cmdline */
 	char		*code_trigger_stop;	/* code trigger stop cmdline */
@@ -461,7 +478,8 @@ typedef struct {
 	unsigned int 	ndbrs;		/* number of data debug registers */
 	unsigned int 	max_counters;	/* maximum number of counters */
 	unsigned int	page_size;	/* page size used by host kernel */
-	unsigned int	clock_tick;	/* _SC_CLK_TCK */
+	unsigned int	clock_tick;	/* CLOCK_REALTIME */
+	int		dfl_plm;	/* default plm */
 	unsigned long	cpu_mhz;	/* CPU speed in MHz */
 	int		libpfm_generic;	/* generic PMU type */
 	int		pmu_type;	/* pfmlib type of host PMU */
@@ -475,16 +493,32 @@ typedef struct {
 	size_t			max_event_name_len;	/* max len for event name (pretty printing) */
 	char			*ev_name1;	/* can hold full event name (cannot share) */
 	char			*ev_name2;	/* can hold full event name (cannot share) */
-	pfmon_smpl_module_t	*smpl_mod;			/* which sampling module to use */
+	pfmon_smpl_module_t	*smpl_mod;	/* which sampling module to use */
 	size_t			ctx_arg_size;
-	void 			*model_options;			/* point to model specific pfmon options */
+	void 			*model_options;		/* point to model specific pfmon options */
 	pfmon_syms_list_t 	primary_syms;		/* opaque pointer to initial binary symbols (no syswide, no-attach) */
-	pfmon_pmu_mask_t	generic_pmu_type;	/* value for a generic PMU for the architecture */
+	int			generic_pmu_type;	/* value for a generic PMU for the architecture */
 } program_options_t;
 
-#define PFMON_CPUMASK_SET(m, g)		((m)[(g)/PFMON_CPUMASK_BITS] |=  (1UL << ((g) % PFMON_CPUMASK_BITS)))
-#define PFMON_CPUMASK_CLEAR(m, g)	((m)[(g)/PFMON_CPUMASK_BITS] &= ~(1UL << ((g) % PFMON_CPUMASK_BITS)))
-#define PFMON_CPUMASK_ISSET(m, g)	((m)[(g)/PFMON_CPUMASK_BITS] &   (1UL << ((g) % PFMON_CPUMASK_BITS)))
+static inline void pfmon_bitmask_set(pfmon_bitmask_t m, int i)
+{
+	m[(i)/PFMON_BITMASK_BITS] |=  (1UL << ((i) % PFMON_BITMASK_BITS));
+}
+
+static inline void pfmon_bitmask_clear(pfmon_bitmask_t m, int i)
+{
+	m[(i)/PFMON_BITMASK_BITS] &= ~(1UL << ((i) % PFMON_BITMASK_BITS));
+}
+
+static inline int pfmon_bitmask_isset(pfmon_bitmask_t m, int i)
+{
+	return (m[(i)/PFMON_BITMASK_BITS] & (1UL << ((i) % PFMON_BITMASK_BITS))) != 0;
+}
+
+static inline void pfmon_bitmask_setall(pfmon_bitmask_t m)
+{
+	memset(m, 0xff, sizeof(pfmon_bitmask_t));
+}
 
 #define opt_debug		program_opt_flags.opt_debug
 #define opt_verbose		program_opt_flags.opt_verbose
@@ -522,7 +556,6 @@ typedef struct {
 #define opt_support_gen		program_opt_flags.opt_support_gen
 #define opt_addr2sym		program_opt_flags.opt_addr2sym
 #define opt_no_ovfl_notify	program_opt_flags.opt_no_ovfl_notify
-#define opt_print_interval	program_opt_flags.opt_print_interval
 #define opt_pin_cmd		program_opt_flags.opt_pin_cmd
 #define opt_print_syms		program_opt_flags.opt_print_syms
 #define opt_is22		program_opt_flags.opt_is22
@@ -548,7 +581,7 @@ typedef struct {
 	void	(*pfmon_show_event_info)(unsigned int evt);
 	char	*name;			/* support module name */
 	int	pmu_type;		/* indicate the PMU type, must be one from pfmlib.h */
-	pfmon_pmu_mask_t generic_pmu_type;	/* what is the generic PMU type for the architecture */
+	int	generic_pmu_type;	/* what is the generic PMU type for the architecture */
 } pfmon_support_t;
 
 extern pfmon_support_t *pfmon_current;
@@ -584,6 +617,7 @@ extern int pfm_uuid2str(pfm_uuid_t uuid, size_t maxlen, char *str);
 extern int pfmon_extract_cmdline(pid_t pid, char *cmdline, int maxlen);
 extern void pfmon_backtrace(void);
 extern unsigned long pfmon_find_cpu_speed(void);
+extern int find_pid_attr(pid_t tid, char *attr);
 extern int pfmon_pin_self(unsigned int cpu);
 extern char * priv_level_str(unsigned int plm);
 extern pid_t gettid(void); 
@@ -591,14 +625,14 @@ extern int pfmon_print_address(FILE *fp, void *hash_desc, pfmon_syms_list_t *lis
 extern int perfmon_debug(int m);
 extern int extract_cache_size(unsigned int level, unsigned int type, unsigned long *size);
 extern void pfmon_clone_sets(pfmon_event_set_t *list, pfmon_sdesc_t *sdesc);
+extern void pfmon_free_sets(pfmon_sdesc_t *sdesc);
 extern void pfmon_dump_sets(void);
-extern int pfmon_validate_buf_size(size_t);
 extern int pfmon_check_kernel_pmu(void);
 extern void pfmon_get_version(void);
-extern int pfmon_set_affinity(pid_t pid, pfmon_cpumask_t mask); 
+extern int pfmon_set_affinity(pid_t pid, pfmon_bitmask_t mask); 
 extern int pfmon_cpu_virt_to_phys(int vcp);
 extern int pfmon_collect_affinity(void);
-extern int pfmon_compute_smpl_entries(size_t hdr_sz, size_t entry_sz);
+extern int pfmon_compute_smpl_entries(size_t hdr_sz, size_t entry_sz, size_t slack);
 extern int find_in_cpuinfo(char *entry, char **result);
 extern int pfmon_detect_unavail_pmcs(pfmlib_regmask_t *r_pmcs);
 
@@ -618,10 +652,9 @@ extern int  pfmon_validate_data_trigger_address(unsigned long addr);
 extern void pfmon_segv_handler_info(struct siginfo *info, void *sc);
 extern int  pfmon_get_breakpoint_addr(pid_t pid, unsigned long *addr, int *is_data);
 extern int  pfmon_get_return_pointer(pid_t pid, unsigned long *rp);
-extern int  pfmon_get_timestamp(uint64_t *t);
 extern int  pfmon_get_cache_info(int cpu, pfmon_cache_info_t *info);
-extern int  __pfmon_set_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask); 
-extern int  __pfmon_get_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask); 
+extern int  __pfmon_set_affinity(pid_t pid, size_t size, pfmon_bitmask_t mask); 
+extern int  __pfmon_get_affinity(pid_t pid, size_t size, pfmon_bitmask_t mask); 
 extern void pfmon_print_simple_cpuinfo(FILE *fp, const char *msg);
 extern void pfmon_print_cpuinfo(FILE *fp);
 
@@ -634,6 +667,7 @@ extern int install_event_sets(pfmon_sdesc_t *sdesc);
 extern int print_results(pfmon_sdesc_t *sdesc);
 extern void close_results(pfmon_sdesc_t *sdesc);
 extern int show_results(pfmon_sdesc_t *sdesc, int needs_order, pfmon_results_mode_t mode);
+extern int show_incr_results(pfmon_sdesc_t *sdesc, int needs_order, pfmon_results_mode_t mode);
 extern int open_results(pfmon_sdesc_t *sdesc);
 extern int read_results(pfmon_sdesc_t *sdesc);
 extern int read_incremental_results(pfmon_sdesc_t *sdesc);
@@ -751,21 +785,21 @@ extern void pfmon_hash_stats(void *hash_desc, FILE *fp);
 /*
  * from pfmon_os.c
  */
-extern int pfmon_create_context(pfmon_ctx_t *ctx, void **smpl_addr, int *fd);
-extern int pfmon_write_pmcs(int fd, pfmon_event_set_t *set, pfmon_pmc_t *pc, int count);
-extern int pfmon_write_pmds(int fd, pfmon_event_set_t *set, pfmon_pmd_t *pd, int count);
-extern int pfmon_read_pmds(int fd, pfmon_event_set_t *set, pfmon_pmd_t *pd, int count);
-extern int pfmon_create_evtsets(int fd, pfmon_setdesc_t *setf, int count);
-extern int pfmon_getinfo_evtsets(int fd, pfmon_setinfo_t *setf, int count);
-extern int pfmon_restart(int fd);
-extern int pfmon_start(int fd);
-extern int pfmon_stop(int fd);
-extern int pfmon_load_context(int fd, pid_t tid);
-extern int pfmon_unload_context(int fd);
+extern int pfmon_create_context(pfmon_ctx_t *ctx, void **smpl_addr, int *fd, int *err);
+extern int pfmon_write_pmcs(int fd, pfmon_event_set_t *set, pfmon_pmc_t *pc, int count, int *err);
+extern int pfmon_write_pmds(int fd, pfmon_event_set_t *set, pfmon_pmd_t *pd, int count, int *err);
+extern int pfmon_read_pmds(int fd, pfmon_event_set_t *set, pfmon_pmd_t *pd, int count, int *err);
+extern int pfmon_create_evtsets(int fd, pfmon_setdesc_t *setf, int count, int *err);
+extern int pfmon_getinfo_evtsets(int fd, pfmon_setinfo_t *setf, int count, int *err);
+extern int pfmon_restart(int fd, int *err);
+extern int pfmon_start(int fd, int *err);
+extern int pfmon_stop(int fd, int *err);
+extern int pfmon_load_context(int fd, pid_t tid, int *err);
+extern int pfmon_unload_context(int fd, int *err);
 extern int prepare_pmc_registers(pfmon_event_set_t *set);
 #ifdef __ia64__
-extern int pfmon_write_ibrs(int fd, pfmon_pmc_t *pc, int count);
-extern int pfmon_write_dbrs(int fd, pfmon_pmc_t *pc, int count);
+extern int pfmon_write_ibrs(int fd, pfmon_pmc_t *pc, int count, int *err);
+extern int pfmon_write_dbrs(int fd, pfmon_pmc_t *pc, int count, int *err);
 #endif
 
 /*
@@ -777,6 +811,8 @@ extern int dfl_smpl_init_ctx_arg(pfmon_ctx_t *ctx, unsigned int max_pmds_sample)
 extern int dfl_smpl_is_default(void);
 extern void dfl_smpl_compute_smpl_entries(void);
 
+#define PFMON_NO_TIMEOUT	(~0U)	/* timeout not set */
+
 #define PFMON_BPL	64 
 #define PFMON_LBPL	6  /* log2(PFMON_BPL) */
 
@@ -817,4 +853,7 @@ static inline unsigned int pfmon_bv_weight(uint64_t *d)
 		sum+= bit_weight(d[i]);
 	return sum;
 }
+#define PERFMON_VERSION_22 (2U<<16|2U)
+#define PERFMON_VERSION_24 (2U<<16|4U)
+
 #endif /*__PFMON_H__ */
