@@ -42,8 +42,8 @@ pfmon_i386_p6_setup_cnt_mask(pfmon_event_set_t *set)
 	pfmlib_i386_p6_input_param_t *param = set->mod_inp;
 	pfmon_i386_p6_args_t *args;
 	char *cnt_mask_str;
-	char *p;
-	unsigned int cnt_mask;
+	char *p, *endptr = NULL;
+	unsigned long l;
 	unsigned int cnt=0;
 	unsigned int i;
 
@@ -70,11 +70,11 @@ pfmon_i386_p6_setup_cnt_mask(pfmon_event_set_t *set)
 
 		if ( p ) *p++ = '\0';
 
-		cnt_mask = atoi(cnt_mask_str);
+		l = strtoul(cnt_mask_str, &endptr, 0);
+		if (endptr || l < 0 || l >255)
+			goto invalid;
 
-		if (cnt_mask < 0 || cnt_mask >255) goto invalid;
-
-		param->pfp_i386_p6_counters[cnt++].cnt_mask= cnt_mask;
+		param->pfp_i386_p6_counters[cnt++].cnt_mask= (unsigned int)l;
 
 		cnt_mask_str = p;
 	}
@@ -205,9 +205,9 @@ pfmon_i386_p6_install_pmd_registers(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set
 static void
 pfmon_i386_p6_usage(void)
 {
-	printf( "--counter-mask=msk1,msk2,...\t\tSet event counter mask (0,1,2,3).\n"
+	printf( "--counter-mask=msk1,msk2,...\t\tSet event counter mask (0,1,2,3)\n"
 		"--inv-mask=i1,i2,...\t\t\tSet event inverse counter mask\n"
-		"\t\t\t\t\t (y/n,0/1).\n"
+		"\t\t\t\t\t(y/n,0/1).\n"
 		"--edge-mask=e1,e2,...\t\t\tSet event edge detect (y/n,0/1).\n"
 	);
 }
@@ -366,6 +366,7 @@ pfmon_support_t pfmon_i386_p6={
 	.pfmon_setup_ctx_flags		= pfmon_i386_p6_setup_ctx_flags,
 	.pfmon_verify_cmdline		= pfmon_i386_p6_verify_cmdline,
 };
+
 /*
  * Intel Pentium M shares same PMU as i386_p6 but with a different event table
  */
