@@ -23,7 +23,6 @@
 
 #include <ctype.h>
 #include <limits.h>
-#include <perfmon/pfmlib_itanium.h>
 
 #include "pfmon_itanium.h"
 
@@ -101,7 +100,7 @@ install_irange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 	pfmlib_ita_output_param_t *param = set->mod_outp;
 	pfmon_pmc_t dbreg[PFMON_ITA_MAX_IBRS];
 	unsigned int i, used_dbr;
-	int  r;
+	int r, error;
 
 	memset(dbreg, 0, sizeof(dbreg));
 
@@ -114,8 +113,9 @@ install_irange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 		dbreg[i].reg_flags = 0UL;
 	}
 
-	r = pfmon_write_ibrs(sdesc->ctxid, dbreg, used_dbr);
-	if (r == -1) warning("cannot install code range restriction: %s\n", strerror(errno));
+	r = pfmon_write_ibrs(sdesc->ctxid, dbreg, used_dbr, &error);
+	if (r == -1)
+		warning("cannot install code range restriction: %s\n", strerror(error));
 	return r;
 }
 
@@ -125,7 +125,7 @@ install_drange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 	pfmlib_ita_output_param_t *param = set->mod_outp;
 	pfmon_pmc_t dbreg[PFMON_ITA_MAX_DBRS];
 	unsigned int i, used_dbr;
-	int r;
+	int r, error;
 
 	memset(dbreg, 0, sizeof(dbreg));
 
@@ -138,8 +138,9 @@ install_drange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 		dbreg[i].reg_flags = 0UL;
 	}
 
-	r = pfmon_write_dbrs(sdesc->ctxid, dbreg, used_dbr);
-	if (r == -1) warning("cannot install data range restriction: %s\n", strerror(errno));
+	r = pfmon_write_dbrs(sdesc->ctxid, dbreg, used_dbr, &error);
+	if (r == -1)
+		warning("cannot install data range restriction: %s\n", strerror(error));
 	return r;
 }
 
@@ -285,29 +286,31 @@ pfmon_ita_usage(void)
 		"--btb-tm-tk\t\t\t\tCapture taken IA-64 branches only.\n"
 		"--btb-tm-ntk\t\t\t\tCapture not taken IA-64 branches only.\n"
 		"--btb-ptm-correct\t\t\tCapture branch if target predicted\n"
-		"\t\t\t\t\t correctly.\n"
+		"\t\t\t\t\tcorrectly.\n"
 		"--btb-ptm-incorrect\t\t\tCapture branch if target is\n"
-		"\t\t\t\t\t mispredicted.\n"
+		"\t\t\t\t\tmispredicted.\n"
 		"--btb-ppm-correct\t\t\tCapture branch if path is predicted\n"
-		"\t\t\t\t\t correctly.\n"
+		"\t\t\t\t\tcorrectly.\n"
 		"--btb-ppm-incorrect\t\t\tCapture branch if path is mispredicted.\n"
 		"--btb-all-mispredicted\t\t\tCapture all mispredicted branches.\n"
 		"--irange=start-end\t\t\tSpecify an instruction address range\n"
-		"\t\t\t\t\t constraint.\n"
+		"\t\t\t\t\tconstraint.\n"
 		"--drange=start-end\t\t\tSpecify a data address range constraint.\n"
 		"--checkpoint-func=addr\t\t\tA bundle address to use as checkpoint.\n"
 		"--ia32\t\t\t\t\tMonitor IA-32 execution only.\n"
 		"--ia64\t\t\t\t\tMonitor IA-64 execution only.\n"
 		"--insn-sets=set1,set2,...\t\tSet per event instruction set\n"
-		"\t\t\t\t\t (setX=[ia32|ia64|both]).\n"
+		"\t\t\t\t\t(setX=[ia32|ia64|both]).\n"
 		"--no-qual-check\t\t\t\tDo not check qualifier constraints on\n"
-		"\t\t\t\t\t events.\n"
+		"\t\t\t\t\tevents.\n"
 		"--insecure\t\t\t\tAllow rum/sum in monitored task\n"
-		"\t\t\t\t\t (per-thread mode and root only).\n"
+		"\t\t\t\t\t(per-thread mode and root only).\n"
 		"--excl-intr\t\t\t\tExclude interrupt-triggered execution\n"
-		"\t\t\t\t\t from system-wide measurement.\n"
+		"\t\t\t\t\tfrom system-wide measurement.\n"
 		"--intr-only\t\t\t\tInclude only interrupt-triggered\n"
-		"\t\t\t\t\t execution from system-wide measurement.\n"
+		"\t\t\t\t\texecution from system-wide measurement.\n"
+		"--exclude-idle\t\t\t\tNot stop monitoring in the idle loop\n"
+		"\t\t\t\t\t(default: off)\n"
 	);
 }
 
@@ -459,11 +462,11 @@ static struct option cmd_ita_options[]={
 	{ "btb-ppm-incorrect", 0, 0, 416},
 	{ "ia32", 0, 0, 417},
 	{ "ia64", 0, 0, 418},
-	{ "excl-intr", 1, 0, 419},
-	{ "intr-only", 1, 0, 420},
+	{ "excl-intr", 0, 0, 419},
+	{ "intr-only", 0, 0, 420},
+	{ "exclude-idle", 0, 0, 421 },
 	{ "no-qual-check", 0, &pfmon_ita_opt.opt_no_qual_check, 0x1},
 	{ "insecure", 0, &pfmon_ita_opt.opt_insecure, 0x1},
-
 	{ 0, 0, 0, 0}
 };
 
@@ -589,6 +592,9 @@ pfmon_ita_parse_options(int code, char *optarg)
 				fatal_error("cannot combine --excl-intr with --intr-only\n");
 			set->set_flags |= PFM_ITA_SETFL_INTR_ONLY;
 			break;
+		case 421 :
+			set->set_flags |= PFM_ITA_SETFL_IDLE_EXCL;
+			break;
 		default:
 			return -1;
 	}
@@ -1053,30 +1059,42 @@ pfmon_ita_setup_ctx_flags(pfmon_ctx_t *ctx)
 static void
 pfmon_ita_verify_cmdline(int argc, char **argv)
 {
-	if (pfmon_ita_opt.opt_insecure) {
-
-		if (options.opt_is22 == 0)
-			fatal_error("the --insecure option requires at least perfmon v2.2\n");
-
-		if (getuid() != 0)
-			fatal_error("the --insecure option requires root privileges\n");
-	}
+	if (pfmon_ita_opt.opt_insecure && options.opt_is22 == 0)
+		fatal_error("the --insecure option requires at least perfmon v2.2\n");
 }
 
 static void
 pfmon_ita_verify_event_sets(void)
 {
 	pfmon_event_set_t *set;
+	int has = 0;
 
-	if (options.opt_syst_wide == 0) {
-		for(set = options.sets; set; set = set->next) {
-			if (set->set_flags & PFM_ITA_SETFL_EXCL_INTR) {
-				fatal_error("excl-intr is not available for per-task measurement\n");
-			}
-			if (set->set_flags & PFM_ITA_SETFL_INTR_ONLY) {
-				fatal_error("intr-only is not available for per-task measurement\n");
-			}
-		}
+	for(set = options.sets; set; set = set->next) {
+		if (set->set_flags & PFM_ITA_SETFL_EXCL_INTR)
+			has |= 1;
+		if (set->set_flags & PFM_ITA_SETFL_INTR_ONLY)
+			has |= 2;
+		if (set->set_flags & PFM_ITA_SETFL_IDLE_EXCL)
+			has |= 4;
+	}
+	if (options.opt_is22 == 0) { 
+		if (has & 1)
+			fatal_error("--excl-intr requires at least perfmon v2.2\n");
+		if (has & 2)
+			fatal_error("--intr-only requires at least perfmon v2.2\n");
+		if (has & 4)
+			fatal_error("--exclude-idle requires at least perfmon v2.4\n");
+	} else {
+		if (has & 4 && options.pfm_version < PERFMON_VERSION_24)
+			fatal_error("--exclude-idle requires at least perfmon v2.4\n");
+	}
+	if (has && options.opt_syst_wide == 0) {
+		if (has & 1)
+			fatal_error("--excl-intr option is not available for per-task measurement\n");
+		if (has & 2) 
+			fatal_error("--intr-only option is not available for per-task measurement\n");
+		if (has & 4)
+			fatal_error("--exclude-idle option is not available for per-task measurement\n");
 	}
 }
 
