@@ -23,7 +23,6 @@
 
 #include <ctype.h>
 #include <limits.h>
-#include <perfmon/pfmlib_montecito.h>
 
 #include "pfmon_montecito.h"
 
@@ -159,7 +158,7 @@ install_irange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 	pfmlib_mont_output_param_t *param = set->mod_outp;
 	pfarg_pmc_t dbreg[PFMON_MONT_MAX_IBRS];
 	unsigned int i, used_dbr;
-	int  r;
+	int r, error;
 
 	check_ibrp_events(set);
 
@@ -173,8 +172,9 @@ install_irange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 		dbreg[i].reg_value = param->pfp_mont_irange.rr_br[i].reg_value; 
 	}
 
-	r = pfmon_write_ibrs(sdesc->ctxid, dbreg, used_dbr);
-	if (r == -1) warning("cannot install code range restriction: %s\n", strerror(errno));
+	r = pfmon_write_ibrs(sdesc->ctxid, dbreg, used_dbr, &error);
+	if (r == -1)
+		warning("cannot install code range restriction: %s\n", strerror(error));
 	return r;
 }
 
@@ -184,7 +184,7 @@ install_drange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 	pfmlib_mont_output_param_t *param = set->mod_outp;
 	pfarg_pmc_t dbreg[PFMON_MONT_MAX_DBRS];
 	unsigned int i, used_dbr;
-	int r;
+	int r, error;
 
 	memset(dbreg, 0, sizeof(dbreg));
 
@@ -196,8 +196,9 @@ install_drange(pfmon_sdesc_t *sdesc, pfmon_event_set_t *set)
 		dbreg[i].reg_value = param->pfp_mont_drange.rr_br[i].reg_value; 
 	}
 
-	r = pfmon_write_dbrs(sdesc->ctxid, dbreg, used_dbr);
-	if (r == -1) warning("cannot install data range restriction: %s\n", strerror(errno));
+	r = pfmon_write_dbrs(sdesc->ctxid, dbreg, used_dbr, &error);
+	if (r == -1)
+		warning("cannot install data range restriction: %s\n", strerror(error));
 
 	return r;
 }
@@ -373,34 +374,36 @@ pfmon_mont_usage(void)
 		"--etb-tm-tk\t\t\t\tCapture taken IA-64 branches only.\n"
 		"--etb-tm-ntk\t\t\t\tCapture not taken IA-64 branches only.\n"
 		"--etb-ptm-correct\t\t\tCapture branch if target predicted\n"
-		"\t\t\t\t\t correctly.\n"
+		"\t\t\t\t\tcorrectly.\n"
 		"--etb-ptm-incorrect\t\t\tCapture branch if target is\n"
-		"\t\t\t\t\t mispredicted.\n"
+		"\t\t\t\t\tmispredicted.\n"
 		"--etb-ppm-correct\t\t\tCapture branch if path is predicted\n"
-		"\t\t\t\t\t correctly.\n"
+		"\t\t\t\t\tcorrectly.\n"
 		"--etb-ppm-incorrect\t\t\tCapture branch if path is mispredicted.\n"
 		"--etb-brt-iprel\t\t\t\tCapture IP-relative branches only.\n"
 		"--etb-brt-ret\t\t\t\tCapture return branches only.\n"
 		"--etb-brt-ind\t\t\t\tCapture non-return indirect branches\n"
-		"\t\t\t\t\t only.\n"
+		"\t\t\t\t\tonly.\n"
 		"--irange=start-end\t\t\tSpecify an instruction address range\n"
-		"\t\t\t\t\t constraint.\n"
+		"\t\t\t\t\tconstraint.\n"
 		"--drange=start-end\t\t\tSpecify a data address range constraint.\n"
 		"--checkpoint-func=addr\t\t\tA bundle address to use as checkpoint.\n"
 		"--inverse-irange\t\t\tInverse instruction range restriction.\n"
 		"--no-qual-check\t\t\t\tDo not check qualifier constraints on\n"
-		"\t\t\t\t\t events.\n"
+		"\t\t\t\t\tevents.\n"
 		"--insecure\t\t\t\tAllow rum/sum in monitored task\n"
-		"\t\t\t\t\t (per-thread mode only).\n"
+		"\t\t\t\t\t(per-thread mode only).\n"
+		"--exclude-idle\t\t\t\tDo not stop monitoring in the idle loop\n"
+		"\t\t\t\t\t(default: off)\n"
 		"--excl-intr\t\t\t\tExclude interrupt-triggered execution\n"
-		"\t\t\t\t\t from system-wide measurement.\n"
+		"\t\t\t\t\tfrom system-wide measurement.\n"
 		"--intr-only\t\t\t\tInclude only interrupt-triggered\n"
-		"\t\t\t\t\t execution from system-wide measurement.\n"
+		"\t\t\t\t\texecution from system-wide measurement.\n"
 		"--irange-demand-fetch\t\t\tLimit irange to demand fetched cache\n"
-		"\t\t\t\t\t lines for specific prefetch events.\n"
+		"\t\t\t\t\tlines for specific prefetch events.\n"
 		"--irange-prefetch-match\t\t\tLimit irange to explicitly prefetched\n"
-		"\t\t\t\t\t cache lines for specific prefetch\n"
-		"\t\t\t\t\t events.\n"
+		"\t\t\t\t\tcache lines for specific prefetch\n"
+		"\t\t\t\t\tevents.\n"
 	);
 }
 
@@ -553,8 +556,9 @@ static struct option cmd_mont_options[]={
 	{ "etb-brt-iprel", 0,  0, 417}, 
 	{ "etb-brt-ret", 0, 0, 418}, 
 	{ "etb-brt-ind", 0, 0, 419},
-	{ "excl-intr", 1, 0, 420},
-	{ "intr-only", 1, 0, 421},
+	{ "excl-intr", 0, 0, 420},
+	{ "intr-only", 0, 0, 421},
+	{ "exclude-idle", 0, 0, 422 },
 	{ "inverse-irange", 0, &pfmon_mont_opt.opt_inv_rr, 1},
 	{ "no-qual-check", 0, &pfmon_mont_opt.opt_no_qual_check, 0x1},
 	{ "insecure", 0, &pfmon_mont_opt.opt_insecure, 0x1},
@@ -666,7 +670,9 @@ pfmon_mont_parse_options(int code, char *optarg)
 				fatal_error("cannot combine --excl-intr with --intr-only\n");
 			set->set_flags |= PFM_ITA_SETFL_INTR_ONLY;
 			break;
-
+		case 422 :
+			set->set_flags |= PFM_ITA_SETFL_IDLE_EXCL;
+			break;
 		default:
 			return -1;
 	}
@@ -854,14 +860,13 @@ pfmon_mont_setup_rr(pfmon_event_set_t *set)
 
 /*
  * It is not possible to measure more than one of the
- * L2_OZQ_CANCELS0, L2_OZQ_CANCELS1, L2_OZQ_CANCELS2 at the
- * same time.
+ * L2D_OZQ_CANCELS0_* OR L2D_OZQ_CANCELS1_* events
+ * at the same time.
  */
 static char *cancel_events[]=
 {
-	"L2_OZQ_CANCELS0_ANY",
-	"L2_OZQ_CANCELS1_REL",
-	"L2_OZQ_CANCELS2_ACQ"
+	"L2D_OZQ_CANCELS0_ACQ",
+	"L2D_OZQ_CANCELS1_ANY"
 };
 #define NCANCEL_EVENTS	sizeof(cancel_events)/sizeof(char *)
 
@@ -869,7 +874,7 @@ static void
 check_cancel_events(pfmon_event_set_t *set)
 {
 	unsigned int i, j, tmp;
-	int code, seen_first = 0;
+	int code, seen_first = 0, ret;
 	int cancel_codes[NCANCEL_EVENTS];
 	unsigned int idx = 0;
 	char *name1, *name2;
@@ -878,7 +883,9 @@ check_cancel_events(pfmon_event_set_t *set)
 	name2 = options.ev_name2;
 
 	for(i=0; i < NCANCEL_EVENTS; i++) {
-		pfm_find_event(cancel_events[i], &tmp);
+		ret = pfm_find_event(cancel_events[i], &tmp);
+		if (ret != PFMLIB_SUCCESS)
+			fatal_error("pfmon bug: cannot find CANCEL events to check\n");
 		pfm_get_event_code(tmp, &code);
 		cancel_codes[i] = code;
 	}
@@ -1164,8 +1171,13 @@ pfmon_mont_detailed_event_name(unsigned int evt)
 static int
 pfmon_mont_setup_ctx_flags(pfmon_ctx_t *ctx)
 {
-	if (pfmon_mont_opt.opt_insecure)
+	if (pfmon_mont_opt.opt_insecure) {
+		if (options.opt_syst_wide) {
+			warning("option --insecure does not support system-wide mode\n");
+			return -1;
+		}
 		ctx->ctx_flags |= PFM_ITA_FL_INSECURE;
+	}
 	return 0;
 }
 
@@ -1173,28 +1185,42 @@ static void
 pfmon_mont_verify_event_sets(void)
 {
 	pfmon_event_set_t *set;
+	int has = 0;
 
-	if (options.opt_syst_wide == 0) {
-		for(set = options.sets; set; set = set->next) {
-			if (set->set_flags & PFM_ITA_SETFL_EXCL_INTR) {
-				fatal_error("excl-intr is not available for per-task measurement\n");
-			}
-			if (set->set_flags & PFM_ITA_SETFL_INTR_ONLY) {
-				fatal_error("intr-only is not available for per-task measurement\n");
-			}
-		}
+	for(set = options.sets; set; set = set->next) {
+		if (set->set_flags & PFM_ITA_SETFL_EXCL_INTR)
+			has |= 1;
+		if (set->set_flags & PFM_ITA_SETFL_INTR_ONLY)
+			has |= 2;
+		if (set->set_flags & PFM_ITA_SETFL_IDLE_EXCL)
+			has |= 4;
+	}
+	if (options.opt_is22 == 0) { 
+		if (has & 1)
+			fatal_error("--excl-intr requires at least perfmon v2.2\n");
+		if (has & 2)
+			fatal_error("--intr-only requires at least perfmon v2.2\n");
+		if (has & 4)
+			fatal_error("--exclude-idle requires at least perfmon v2.4\n");
+	} else {
+		if (has & 4 && options.pfm_version < PERFMON_VERSION_24)
+			fatal_error("--exclude-idle requires at least perfmon v2.4\n");
+	}
+	if (has && options.opt_syst_wide == 0) {
+		if (has & 1)
+			fatal_error("--excl-intr option is not available for per-task measurement\n");
+		if (has & 2) 
+			fatal_error("--intr-only option is not available for per-task measurement\n");
+		if (has & 4)
+			fatal_error("--exclude-idle option is not available for per-task measurement\n");
 	}
 }
 
 static void
 pfmon_mont_verify_cmdline(int argc, char **argv)
 {
-	
-	if (pfmon_mont_opt.opt_insecure) {
-
-		if (options.opt_is22 == 0)
-			fatal_error("the --insecure option requires at least perfmon v2.2\n");
-	}
+	if (pfmon_mont_opt.opt_insecure && options.opt_is22 == 0)
+		fatal_error("the --insecure option requires at least perfmon v2.2\n");
 }
 
 static void
