@@ -30,16 +30,14 @@ static void
 print_sdesc_header(FILE *fp, pfmon_sdesc_t *sdesc)
 {
 	fprintf(fp, "# command line     : %s\n#\n"
-		    "# process id       : %d\n"
-		    "# thread  id       : %d\n"
+		    "# process id(tgid) : %d\n"
+		    "# thread  id(pid)  : %d\n"
 		    "# parent process id: %d\n"
-		    "# parent thread  id: %d\n"
 		    "# exec count       : %u\n#\n",
 		    sdesc->cmdline,
 		    sdesc->pid,
 		    sdesc->tid,
 		    sdesc->ppid,
-		    sdesc->ptid,
 		    sdesc->exec_count);
 }
 
@@ -108,15 +106,15 @@ print_standard_header(FILE *fp, pfmon_sdesc_t *sdesc)
 		for(i=0; i < set->event_count; i++) {
 			pfm_get_full_event_name(&set->inp.pfp_events[i], name, l);
 			fprintf(fp, "#\tPMD%u: %s = %s\n",
-					set->outp.pfp_pmcs[i].reg_num,
+					set->outp.pfp_pmds[i].reg_num,
 					name,
 					priv_level_str(set->inp.pfp_events[i].plm));
 		}
 	}
 	fprintf(fp, "#\n");
 	if (options.nsets > 1) {
-		fprintf(fp, "# sets requested switch timeout: %u usecs\n", options.switch_timeout);
-		fprintf(fp, "# sets effective switch timeout: %u usecs\n", options.eff_switch_timeout);
+		fprintf(fp, "# sets requested switch timeout: %"PRIu64" msecs\n", options.switch_timeout);
+		fprintf(fp, "# sets effective switch timeout: %"PRIu64" msecs\n", options.eff_switch_timeout);
 	}
 	/*
 	 * invoke CPU model specific routine to print any additional information
@@ -138,13 +136,12 @@ print_standard_header(FILE *fp, pfmon_sdesc_t *sdesc)
 
 	if (options.opt_syst_wide) {
 		unsigned int i;
-
 		fprintf(fp, "# results captured on physical CPUs: ");
-		if (options.opt_aggr == 0) {
+		if (options.opt_aggr == 0 && (options.interval == PFMON_NO_TIMEOUT || fp != stdout)) {
 			fprintf(fp, "CPU%d", pfmon_cpu_virt_to_phys(sdesc->cpu));
 		} else {
 			for(i=0; i < options.online_cpus; i++) {
-				if (PFMON_CPUMASK_ISSET(options.virt_cpu_mask, i))
+				if (pfmon_bitmask_isset(options.virt_cpu_mask, i))
 					fprintf(fp, "CPU%d ", pfmon_cpu_virt_to_phys(i));
 			}
 		}
@@ -234,6 +231,7 @@ show_generic_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)
 	pid_t pid = 0, tid = 0, ppid = 0;
 	int is_syswide;
 	unsigned int i, cpu, event_count;
+	int need_cr = 0;
 	unsigned long max_len, the_len, max;
 	char *name;
 	char format_base[32];
@@ -242,8 +240,8 @@ show_generic_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)
 	uint64_t count;
 	size_t l;
 
-
-	if (options.opt_use_smpl && options.opt_smpl_print_counts == 0) return 0;
+	if (options.opt_use_smpl && options.opt_smpl_print_counts == 0)
+		return 0;
 
 	l = 1 + options.max_event_name_len;
 	name = malloc(l);
@@ -254,22 +252,33 @@ show_generic_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)
 	cpu        = sdesc->cpu;
 	fp         = sdesc->out_fp;
 
-	if (options.opt_with_header) {
-		if (fp == stdout) pthread_mutex_lock(&results_lock);
-		print_standard_header(fp, sdesc);
-		if (fp == stdout) pthread_mutex_unlock(&results_lock);
-	}
-
-
-	suffix     = "";
+	suffix = "";
 	format_str = "%*s %-*s\n";
 
+	if (options.opt_with_header && !sdesc->done_header) {
+		if (options.interval == PFMON_NO_TIMEOUT || sdesc->id == 0 || fp != stdout)
+			print_standard_header(fp, sdesc);
+		sdesc->done_header = 1;
+	}
+
 	if (is_syswide) {
-		if (fp == stdout && options.opt_aggr == 0) {
-			sprintf(format_base, "CPU%-4u %%*s %%-*s\n", cpu);
+			if (options.interval != PFMON_NO_TIMEOUT) {
+				if (sdesc->id == 0 && fp == stdout) {
+					if (mode == PFMON_RESULTS_INTER)
+						fprintf(sdesc->out_fp, "#\n");
+					else
+						fprintf(sdesc->out_fp, "# final totals\n");
+				}
+				fprintf(fp,"CPU%-4u ", cpu);
+				sprintf(format_base, "%%*s ");
+				use_max_len = 1;
+				need_cr = 1;
+			} else
+				sprintf(format_base, "CPU%-4u %%*s %%-*s\n", cpu);
+
 			format_str = format_base;
-			if (options.selected_cpus > 1) use_max_len = 1;
-		} 
+			if (options.selected_cpus > 1)
+				use_max_len = 1;
 	} else if (options.opt_aggr == 0 && options.opt_follows && options.outfile == NULL) {
 		/* isolate command */
 		p = strchr(sdesc->cmdline, ' ');
@@ -286,7 +295,6 @@ show_generic_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)
 	} 
 	max_len = options.max_event_name_len;
 
-
 	if (use_max_len == 0) {
 		/*
 		 * find longest count
@@ -331,6 +339,8 @@ show_generic_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)
 					tid, ppid);
 		}
 	}
+	if (need_cr)
+		fputc('\n', fp);
 	/* restore space character, if needed */
 	if (p) *p = ' ';
 	free(name);
@@ -338,7 +348,8 @@ show_generic_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)
 }
 
 static int
-pfmon_ordered_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode, int (*res_func)(pfmon_sdesc_t *, pfmon_results_mode_t))
+pfmon_ordered_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode,
+			int (*res_func)(pfmon_sdesc_t *, pfmon_results_mode_t))
 {
 	int ret;
 
@@ -349,7 +360,6 @@ pfmon_ordered_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode, int (*res
 	}
 
 	ret = (*res_func)(sdesc, mode);
-	DPRINT(("who_must_print=%u sdesc->id=%u done\n", who_must_print, sdesc->id));
 
 	who_must_print++;
 
@@ -365,6 +375,51 @@ pfmon_ordered_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode, int (*res
 	return ret;
 }
 
+static int
+do_show_incr_results(pfmon_sdesc_t *sdesc, pfmon_results_mode_t mode)
+{
+	FILE *fp = NULL;
+	pfmon_event_set_t *set;
+	unsigned int i;
+	char counter_str[32];
+
+	fp = sdesc->out_fp;
+
+	if (options.opt_with_header && !sdesc->done_header) {
+		if (options.interval == PFMON_NO_TIMEOUT || sdesc->id == 0 || fp != stdout)
+			print_standard_header(fp, sdesc);
+		sdesc->done_header = 1;
+	}
+
+	if (sdesc->id == 0 && fp == stdout) {
+		if (mode == PFMON_RESULTS_INTER)
+			fprintf(sdesc->out_fp, "#\n");
+		else
+			fprintf(sdesc->out_fp, "# final totals\n");
+	}
+
+	fprintf(fp,"CPU%-4u ", sdesc->cpu);
+	for(set = sdesc->sets; set ; set = set->next) {
+		for (i=0; i < set->event_count; i++) {
+			counter2str(set->master_pd[i].reg_value, counter_str);
+			fprintf(fp, "%26s ", counter_str);
+		}
+	}
+	fputc('\n', fp);
+	return 0;
+}
+
+int
+show_incr_results(pfmon_sdesc_t *sdesc, int needs_order, pfmon_results_mode_t mode)
+{
+	int ret;
+
+	if (needs_order)
+		ret = pfmon_ordered_results(sdesc, mode, do_show_incr_results);
+	else
+		ret = do_show_incr_results(sdesc, mode); 
+	return ret;
+}
 
 int
 show_results(pfmon_sdesc_t *sdesc, int needs_order, pfmon_results_mode_t mode)
@@ -374,8 +429,7 @@ show_results(pfmon_sdesc_t *sdesc, int needs_order, pfmon_results_mode_t mode)
 	if (needs_order)
 		ret = pfmon_ordered_results(sdesc, mode, show_generic_results);
 	else
-		ret = show_generic_results(sdesc, mode);
-
+		ret = show_generic_results(sdesc, mode); 
 	return ret;
 }
 
@@ -405,28 +459,59 @@ int
 read_incremental_results(pfmon_sdesc_t *sdesc)
 {
 	pfmon_event_set_t *set;
-	unsigned int j;
-	int ctxid;
+	pfmon_setinfo_t	setf;
+	uint64_t total_runs = 0, d;
+	unsigned int j, i;
+	int ctxid, error;
+
+	if (sdesc->nsets > 1)
+		memset(&setf, 0, sizeof(setf));
 
 	ctxid = sdesc->ctxid;
 
 	for (set = sdesc->sets; set ; set = set->next) {
-		if (pfmon_read_pmds(ctxid, set, set->tmp_pd, set->event_count) == -1) {
-			warning("pfmon_read_pmds error %s\n", strerror(errno));
+		if (pfmon_read_pmds(ctxid, set, set->tmp_pd, set->event_count, &error) == -1) {
+			warning("pfmon_read_pmds error %s\n", strerror(error));
 			return -1;
 		}
 		/*
-		 * calculate delta: ugly but works
+		 * calculate delta
 		 */
 		for(j=0; j < set->event_count; j++) {
 			set->master_pd[j].reg_value = set->tmp_pd[j].reg_value - set->prev_pd[j].reg_value;
 			set->prev_pd[j].reg_value = set->tmp_pd[j].reg_value;
 		}
+		/*
+		 * calculate run delta
+		 */
+		if (sdesc->nsets > 1) {
+			setf.set_id = set->id;
+			if (pfmon_getinfo_evtsets(ctxid, &setf, 1, &error) == -1) {
+				warning("pfmon_getinfo_evt_sets error %s\n", strerror(error));
+				return -1;
+			}
+			set->prev_nruns = set->nruns;
+			set->nruns  = setf.set_runs;
+			total_runs += setf.set_runs - set->prev_nruns;
+		}
+	}
+	/*
+	 * scale results based on the number of runs in the delta
+	 */
+	if (sdesc->nsets > 1) {
+		for (set = sdesc->sets; set ; set = set->next) {
+			for(i=0; i < set->event_count; i++) {
+				d = set->nruns - set->prev_nruns;
+				if (d)
+					set->master_pd[i].reg_value = (set->master_pd[i].reg_value*total_runs)/d; 
+				else
+					set->master_pd[i].reg_value = 0;
+			}
+		}
 	}
 	return 0;
 }
 
-
 int
 read_results(pfmon_sdesc_t *sdesc)
 {
@@ -434,7 +519,7 @@ read_results(pfmon_sdesc_t *sdesc)
 	pfmon_setinfo_t	setf;
 	uint64_t total_runs = 0;
 	unsigned int i;
-	int ctxid;
+	int ctxid, error;
 
 	memset(&setf, 0, sizeof(setf));
 
@@ -442,15 +527,15 @@ read_results(pfmon_sdesc_t *sdesc)
 
 	for (set = sdesc->sets; set ; set = set->next) {
 
-		if (pfmon_read_pmds(ctxid, set, set->master_pd, set->event_count) == -1) {
-			warning("pfmon_read_pmds error %s\n", strerror(errno));
+		if (pfmon_read_pmds(ctxid, set, set->master_pd, set->event_count, &error) == -1) {
+			warning("pfmon_read_pmds error %s\n", strerror(error));
 			return -1;
 		}
 
-		if (sdesc->nsets > 1 || options.opt_is22) {
+		if (options.opt_is22) {
 			setf.set_id = set->id;
-			if (pfmon_getinfo_evtsets(ctxid,&setf, 1) == -1) {
-				warning("pfmon_getinfo_evt_sets error %s\n", strerror(errno));
+			if (pfmon_getinfo_evtsets(ctxid, &setf, 1, &error) == -1) {
+				warning("pfmon_getinfo_evt_sets error %s\n", strerror(error));
 				return -1;
 			}
 			if (setf.set_runs == 0)
