@@ -19,7 +19,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  */
-
 #include "pfmon.h"
 
 #include <ctype.h>
@@ -27,54 +26,37 @@
 #include <time.h>
 #include <dirent.h>
 
-extern pfmon_smpl_module_t detailed_ia64_smpl_module;
-extern pfmon_smpl_module_t dear_hist_ia64_smpl_module;
-
-extern pfmon_smpl_module_t inst_hist_ia64_old_smpl_module; /* backward compatibility */
-extern pfmon_smpl_module_t detailed_ia64_old_smpl_module;
-extern pfmon_smpl_module_t compact_ia64_old_smpl_module;   /* backward compatibility */
-extern pfmon_smpl_module_t dear_hist_ia64_old_smpl_module;
-extern pfmon_smpl_module_t raw_ia64_old_smpl_module;
-
-extern pfmon_smpl_module_t detailed_amd64_smpl_module;
-
-extern pfmon_smpl_module_t detailed_i386_p6_smpl_module;
-extern pfmon_smpl_module_t detailed_gen_ia32_smpl_module;
-extern pfmon_smpl_module_t inst_hist_smpl_module;
-extern pfmon_smpl_module_t compact_smpl_module;
-extern pfmon_smpl_module_t raw_smpl_module;
+#include "pfmon_smpl.h"
 
 static pfmon_smpl_module_t *smpl_modules[]={
-#ifdef CONFIG_PFMON_IA64
-	&detailed_ia64_smpl_module,	/* must be first for Itanium */
-	&dear_hist_ia64_smpl_module,
-
-	&detailed_ia64_old_smpl_module,		/* for perfmon v2.0 (must be first for old modules) */
-	&inst_hist_ia64_old_smpl_module,	/* for perfmon v2.0 */
-
-	&compact_ia64_old_smpl_module,	/* for perfmon v2.0 */
-	&raw_ia64_old_smpl_module,	/* for perfmon v2.0 */
-	&dear_hist_ia64_old_smpl_module,/* for perfmon v2.0 */
-#endif
-#ifdef CONFIG_PFMON_AMD64
-	&detailed_amd64_smpl_module, /* must be first for X86-64 */
-#endif
-#ifdef CONFIG_PFMON_I386_P6
-	&detailed_i386_p6_smpl_module, /* must be first for P6 */
-#endif
-#ifdef CONFIG_PFMON_GEN_IA32
-	&detailed_gen_ia32_smpl_module, /* must be last for I386 */
-#endif
 	&inst_hist_smpl_module,		/* works for any PMU model */
+	&detailed_smpl_module,		/* works for any PMU model */
 	&compact_smpl_module,		/* works for any PMU model */
 	&raw_smpl_module,		/* works for any PMU model */
+#ifdef CONFIG_PFMON_CORE
+	&pebs_smpl_module,
+#endif
+
+#ifdef CONFIG_PFMON_IA64
+	&dear_hist_ia64_smpl_module,
+
+	&detailed_old_smpl_module,	/* for perfmon v2.0 (must be first for old modules) */
+	&inst_hist_old_smpl_module,	/* for perfmon v2.0 */
+	&dear_hist_ia64_old_smpl_module,/* for perfmon v2.0 */
+	&compact_old_smpl_module,	/* for perfmon v2.0 */
+	&raw_old_smpl_module,		/* for perfmon v2.0 */
+#endif
 	NULL
 };
 
 void
 pfmon_smpl_initialize(void)
 {
-	pfmon_smpl_module_t *mod;
+	pfmon_smpl_module_t *mod, **p;
+
+	for(p= smpl_modules; *p; p++)
+		(*p)->initialize_mask();
+
 	/*
 	 * pick a default sampling output format
 	 */
@@ -435,71 +417,40 @@ pfmon_check_smpl_module_legacy(pfm_uuid_t uuid)
 }
 
 static int
-pfmon_check_smpl_module(pfm_uuid_t uuid)
+pfmon_check_smpl_module(pfmon_smpl_module_t *smpl_mod)
 {
 #define SYS_KERNEL_PERFMON_FORMATS "/sys/kernel/perfmon/formats"
 	DIR *dir;
 	struct dirent *de;
-	char *str = NULL;
-	int found = 0;
-	FILE *fp;
-	size_t len, pref_len;
-	char uuid_str[64];
-	char buffer[256];
+	int ret = -1;
 
 	if (options.opt_is22 == 0)
-		return pfmon_check_smpl_module_legacy(uuid);
-
-	pfm_uuid2str(uuid, sizeof(uuid_str), uuid_str);
-
-	len = strlen(uuid_str);
-
-	pref_len = strlen(SYS_KERNEL_PERFMON_FORMATS);
+		return pfmon_check_smpl_module_legacy(smpl_mod->uuid);
 
 	dir = opendir(SYS_KERNEL_PERFMON_FORMATS);
 	if (dir == NULL) {
-		warning("unable to open /sys/kernel/perfmon/formats\n");
+		warning("unable to open %s\n", SYS_KERNEL_PERFMON_FORMATS);
 		return -1; /* will fail later on if module not present */
 	}
 
-	while(found == 0 && (de = readdir(dir))) {
+	while(ret == -1 && (de = readdir(dir))) {
 		/*
 		 * skip . and ..
 		 */
 		if (strcmp(de->d_name, ".") == 0 || strcmp(de->d_name, "..") == 0)
 			continue;
 
-		str = realloc(str, pref_len + strlen(de->d_name) + 1 + 1 + 4 + 1);
-		if (str == NULL) {
-			warning("cannot allocate format filename buffer\n");
-			return -1;
+		if (!strcmp(de->d_name, smpl_mod->fmt_name)) {
+			DPRINT(("found format %s\n", smpl_mod->fmt_name));
+			ret = 0;
 		}
-
-		strcpy(str, SYS_KERNEL_PERFMON_FORMATS);
-		strcat(str, "/");
-		strcat(str, de->d_name);
-		strcat(str, "/uuid");
-
-		fp = fopen(str, "r");
-		if (fp == NULL)
-			continue;
-
-		fgets(buffer, sizeof(buffer)-1, fp);
-
-		if (!strncmp(buffer, uuid_str, len)) {
-			DPRINT(("found uuid_str:%s:\nuuid_p:%s", uuid_str, buffer));
-			found = 1;
-		}
-		fclose(fp);
 	}
 	closedir(dir);
 
-	free(str);
+	if (ret == -1)
+		warning("cannot find requested kernel sampling format : %s\n", smpl_mod->fmt_name);
 
-	if (found == 0)
-		warning("cannot find requested kernel sampling format : %s\n", uuid_str);
-
-	return found ? 0 : -1;
+	return ret;
 }
 
 static void
@@ -547,7 +498,7 @@ pfmon_setup_set_sampling_rates(pfmon_event_set_t *set)
 		if (cnt) options.opt_use_smpl = 1;
 	}
 
-	if (options.opt_use_smpl && options.opt_print_interval) 
+	if (options.opt_use_smpl && options.interval != PFMON_NO_TIMEOUT) 
 		fatal_error("cannot use --print-interval when sampling\n");
 
 	if (set->random_smpl_args) {
@@ -621,7 +572,7 @@ pfmon_setup_sampling_rates(void)
 	if (options.smpl_mod == NULL) 
 		fatal_error("error, no sampling module selected\n");
 
-	if (pfmon_check_smpl_module(options.smpl_mod->uuid)) 
+	if (pfmon_check_smpl_module(options.smpl_mod))
 		fatal_error("cannot proceed");
 
 	if (options.smpl_mod->process_samples == NULL)
@@ -640,25 +591,18 @@ int
 pfmon_find_smpl_module(char *name, pfmon_smpl_module_t **mod, int ignore_cpu)
 {
 	pfmon_smpl_module_t **p;
-	pfmon_pmu_mask_t mask, gen_mask = 0;
-	int is22;
+	int is22, m;
 
 	is22 = options.opt_is22;
-
-	mask = PFMON_PMU_MASK(options.pmu_type);
-
-	if (options.generic_pmu_type != PFMLIB_NO_PMU)
-		gen_mask = PFMON_PMU_MASK(options.generic_pmu_type);
+	m = options.pmu_type;
 
 	for(p = smpl_modules; *p ; p++) {
-
 		if (name == NULL || !strcmp(name, (*p)->name)) {
-
 			if (   (is22      && ((*p)->flags & PFMON_SMPL_MOD_FL_LEGACY))
 			    || (is22 == 0 && ((*p)->flags & PFMON_SMPL_MOD_FL_LEGACY) == 0)) continue;
 
 			if (   ignore_cpu == 0
-			    && (*p)->pmu_mask != gen_mask && ((*p)->pmu_mask & mask) == 0) {
+			    && pfmon_bitmask_isset((*p)->pmu_mask, m) == 0) {
 				continue;
 			}
 			*mod = *p;
@@ -672,27 +616,23 @@ void
 pfmon_list_smpl_modules(void)
 {
 	pfmon_smpl_module_t **p = smpl_modules;
-	pfmon_pmu_mask_t mask, gen_mask = 0;
 	int type, is22;
 
 	pfm_get_pmu_type(&type);
-	mask = PFMON_PMU_MASK(type);
 
 	is22 = options.opt_is22;
 
-	/*
-	 * gen_mask covers a complete processor family (e.g., IA-64_
-	 */
-	if (options.generic_pmu_type != PFMLIB_NO_PMU)
-		gen_mask = PFMON_PMU_MASK(options.generic_pmu_type);
-
 	printf("supported sampling modules: ");
 	while (*p) {	
-		
-		if (   ((is22 == 0 && ((*p)->flags & PFMON_SMPL_MOD_FL_LEGACY)) ||( is22 && ((*p)->flags & PFMON_SMPL_MOD_FL_LEGACY) == 0))
-		    && ((*p)->pmu_mask == PFMON_ANY_PMU_MASK
-		    || (*p)->pmu_mask == gen_mask
-		    || ((*p)->pmu_mask & mask))) printf("[%s] ", (*p)->name);
+
+		if (   (is22      && ((*p)->flags & PFMON_SMPL_MOD_FL_LEGACY))
+		    || (is22 == 0 && ((*p)->flags & PFMON_SMPL_MOD_FL_LEGACY) == 0)
+		    || (pfmon_bitmask_isset((*p)->pmu_mask, type) == 0)) {
+			p++;
+			continue;
+		}
+
+		printf("[%s] ", (*p)->name);
 		p++;
 	}
 	printf("\n");
@@ -701,8 +641,7 @@ pfmon_list_smpl_modules(void)
 void
 pfmon_smpl_module_info(pfmon_smpl_module_t *fmt)
 {
-	unsigned long m;
-	int i;
+	int i, ret;
 	char name[PFMON_MAX_EVTNAME_LEN];
 
 	printf("name        : %s\n"
@@ -710,14 +649,12 @@ pfmon_smpl_module_info(pfmon_smpl_module_t *fmt)
 		fmt->name,
 		fmt->description);
 
-	m = fmt->pmu_mask;
 	printf("PMU models  : ");
-	if (m == PFMON_ANY_PMU_MASK) 
-		printf("[all] ");
-	else for(i=0; m; i++, m >>=1) {
-		if (m & 0x1) {
-			pfm_get_pmu_name_bytype(i, name,PFMON_MAX_EVTNAME_LEN);
-			printf("[%s] ", name);
+	for(i=0; i < PFMON_MAX_PMUS; i++) {
+		if (pfmon_bitmask_isset(fmt->pmu_mask, i)) {
+			ret = pfm_get_pmu_name_bytype(i, name,PFMON_MAX_EVTNAME_LEN);
+			if (ret == PFMLIB_SUCCESS)
+				printf("[%s] ", name);
 		}
 	}
 	printf("\n");
@@ -730,7 +667,7 @@ pfmon_smpl_module_info(pfmon_smpl_module_t *fmt)
 int
 pfmon_reset_sampling(pfmon_sdesc_t *sdesc)
 {
-	int ret;
+	int ret, error;
 
 	/* for debug purposes */
 	if (options.opt_block_restart) {
@@ -747,7 +684,7 @@ pfmon_reset_sampling(pfmon_sdesc_t *sdesc)
 	sdesc->csmpl.last_ovfl   = ~0;
 	sdesc->csmpl.last_count  = ~0;
 
-	ret = pfmon_restart(sdesc->ctxid);
+	ret = pfmon_restart(sdesc->ctxid, &error);
 	UNLOCK_SDESC(sdesc);
  	return ret;
 }
@@ -755,7 +692,7 @@ pfmon_reset_sampling(pfmon_sdesc_t *sdesc)
 int
 pfmon_process_smpl_buf(pfmon_sdesc_t *sdesc, int is_final)
 {
-	int ret;
+	int ret, error;
 
 	LOCK_SDESC(sdesc);
 
@@ -769,7 +706,7 @@ pfmon_process_smpl_buf(pfmon_sdesc_t *sdesc, int is_final)
 			printf("<press a key to resume monitoring>\n");
 			getchar();
 		}
-		ret = pfmon_restart(sdesc->ctxid);
+		ret = pfmon_restart(sdesc->ctxid, &error);
 	}
 	sdesc->csmpl.processing = 0;
 	UNLOCK_SDESC(sdesc);
@@ -781,24 +718,17 @@ void
 pfmon_smpl_mod_usage(void)
 {
 	pfmon_smpl_module_t **mod;
-	pfmon_pmu_mask_t mask, gen_mask = 0;
 	int type, ret;
 	int is22;
 
 	is22 = options.opt_is22;
 
 	ret = pfm_get_pmu_type(&type);
-	if (ret != PFMLIB_SUCCESS) {
+	if (ret != PFMLIB_SUCCESS)
 		fatal_error("library not initialized\n");
-	}
-	mask = PFMON_PMU_MASK(type);
-
-	if (options.generic_pmu_type != PFMLIB_NO_PMU)
-		gen_mask = PFMON_PMU_MASK(options.generic_pmu_type);
 
 	for(mod = smpl_modules; *mod; mod++) {
-
-		if ((*mod)->pmu_mask != gen_mask && ((*mod)->pmu_mask & mask) == 0) continue;
+		if (pfmon_bitmask_isset((*mod)->pmu_mask, type) == 0) continue;
 
 		if (   (is22      && ((*mod)->flags & PFMON_SMPL_MOD_FL_LEGACY))
 		    || (is22 == 0 && ((*mod)->flags & PFMON_SMPL_MOD_FL_LEGACY) == 0)) continue;
@@ -832,20 +762,21 @@ pfmon_smpl_init_ctx_arg(pfmon_ctx_t *ctx, unsigned int max_pmds_sample)
 	int ret = 0;
 
 	/*
-	 * copy the module format's UUID
+	 * legacy: copy the module format's UUID
 	 */
-	memcpy(ctx->ctx_uuid, options.smpl_mod->uuid, sizeof(pfm_uuid_t));
+	if (options.opt_is22)
+		ctx->fmt_name = options.smpl_mod->fmt_name;
+	else
+		memcpy(ctx->ctx_uuid, options.smpl_mod->uuid, sizeof(pfm_uuid_t));
 
-	if (dfl_smpl_is_default()) {
+	if (dfl_smpl_is_default())
 		return dfl_smpl_init_ctx_arg(ctx, max_pmds_sample);
-	}
-
 
 	if (options.smpl_mod->init_ctx_arg)
 		ret = (*options.smpl_mod->init_ctx_arg)(ctx, max_pmds_sample);
 
 	if (ret == 0 && options.smpl_entries == 0)
-		warning("number of entries in samping buffer is zero\n");
+		warning("number of entries in sampling buffer is zero\n");
 
 	return ret;
 }
