@@ -69,6 +69,8 @@ extern char *cplus_demangle(char *, int);
 #define for_each_sym_module(m)	
 struct sym_hash_table_t;
 
+#define PFMON_MAX_SECTIONS	2
+
 typedef struct _sym_hash_data {
 	unsigned long		addr;
 	unsigned long		eaddr;
@@ -99,7 +101,6 @@ symcmp(const void *a, const void *b)
 static int
 read_sym64(Elf *elf, const char *filename, module_symbols_t *mod_sym, pfmon_sym_type_t sym_type, map_desc_t *map)
 {
-#define PFMON_MAX_SECTIONS	2
 	Elf_Data *data;
 	Elf64_Shdr *hdr;
 	Elf64_Phdr *phdrs, *p;
@@ -176,7 +177,7 @@ read_sym64(Elf *elf, const char *filename, module_symbols_t *mod_sym, pfmon_sym_
 		 */
 		data = elf_rawdata(section, NULL); 
 		if (data == NULL) {
-			warning("can't extract raw elf data for symbol table\n");
+			warning("cannot extract raw elf data for symbol table\n");
 			return -1;
 		}
 		table_size  += hdr->sh_size/hdr->sh_entsize;
@@ -206,7 +207,7 @@ read_sym64(Elf *elf, const char *filename, module_symbols_t *mod_sym, pfmon_sym_
 		 */
 		data = elf_rawdata(section, NULL); 
 		if (data == NULL) {
-			warning("can't extract raw elf data for symbol table\n");
+			warning("cannot extract raw elf data for symbol table\n");
 			return -1;
 		}
 		symtab_data = (Elf64_Sym *)data->d_buf;
@@ -225,7 +226,7 @@ read_sym64(Elf *elf, const char *filename, module_symbols_t *mod_sym, pfmon_sym_
 		 */
 		data = elf_rawdata(strsym_section, NULL); 
 		if (data == NULL) {
-			vbprintf("can't extract raw elf data for string section\n");
+			vbprintf("cannot extract raw elf data for string section\n");
 			return -1;
 		}
 		d_buf = data->d_buf;
@@ -269,11 +270,10 @@ read_sym64(Elf *elf, const char *filename, module_symbols_t *mod_sym, pfmon_sym_
 					end   = p->p_vaddr + p->p_memsz;
 					if (value >= start && value < end) break;
 				}
+				/*
+				 * that happens for dynamic symbols
+				 */
 				if (k == phnums) {
-					/*
-					 * that happens for dynamic symbols
-					 */
-					DPRINT(("cannot find phdrs for %s\n", symbol_tab[j].name));
 					/*
 					 * required for shortcut
 					 */
@@ -283,11 +283,8 @@ read_sym64(Elf *elf, const char *filename, module_symbols_t *mod_sym, pfmon_sym_
 skip_phdrs:
 				fpos = p->p_offset + value - start;
 
-				if (fpos < map->map_offset || (fpos-map->map_offset) >= map->map_size) {
-					DPRINT(("%20.20s rejected fpos=0x%lx map_start=0x%lx map_size=0x%lx map_offset=0x%lx\n", 
-								symbol_tab[j].name, fpos, map->map_start, map->map_size, map->map_offset));
+				if (fpos < map->map_offset || (fpos-map->map_offset) >= map->map_size)
 					continue;
-				}
 
 				value = map->map_start + (fpos-map->map_offset);
 				//printf("%20.20s @0x%lx num_scan=%lu\n", symbol_tab[j].name, value, num_scans);
@@ -331,89 +328,225 @@ static int
 read_sym32(Elf *elf, const char *filename, module_symbols_t *mod_sym, pfmon_sym_type_t sym_type, map_desc_t *map)
 {
 	Elf_Data *data;
-	Elf32_Shdr *symtab_hdr = NULL;
-	Elf_Scn *symtab_section;
+	Elf32_Shdr *hdr;
+	Elf32_Phdr *phdrs, *p;
+	Elf32_Ehdr *ehdr;
+	Elf32_Addr start = 0, end = 0;
+	Elf_Scn *sections[PFMON_MAX_SECTIONS], *section;
 	Elf_Scn *strsym_section;
 	Elf32_Sym *symtab_data;
 	symbol_t *symbol_tab;
-	char *name_space;
+	void *d_buf;
+	char *str, *s_tmp;
+	Elf32_Half phnums, k;
+	unsigned long table_size, fpos, value, num_scans = 0;
 	size_t strsym_index;
-	int i, j, table_size;
-	int use_text = 0, use_data = 0;
+	unsigned int num_sections, use_text = 0, use_data = 0 ;
+	unsigned int s, i, j;
 	int type;
 
-    
-	/* first find the symbol table table */
-	symtab_section = NULL;
-	while ((symtab_section = elf_nextscn(elf, symtab_section)) != 0) {
+	ehdr = elf32_getehdr(elf);
+	if (ehdr == NULL) {
+		warning("no elf header\n");
+		return -1;
+	}
+	phnums = ehdr->e_phnum;
+	phdrs  = elf32_getphdr(elf);
 
-		symtab_hdr = elf32_getshdr(symtab_section);
-		if (symtab_hdr == NULL) {
-			vbprintf("cannot get section header\n");
+	/* first find the symbol table and the dynamic symbol table */
+	sections[0] = sections[1] = section = NULL;
+	num_sections = 0;
+
+	if (sym_type == PFMON_TEXT_SYMBOL) 
+		use_text = 1;
+	else if (sym_type == PFMON_DATA_SYMBOL) 
+		use_data = 1;
+	else {
+		warning("invalid sym_type %d\n", sym_type);
+		return -1;
+	}
+	while ((section = elf_nextscn(elf, section)) != 0 && num_sections < PFMON_MAX_SECTIONS) {
+
+		hdr = elf32_getshdr(section);
+		if (hdr == NULL) {
+			warning("cannot get section header\n");
 			return -1;
 		}
-			
-		/* is this the symbol table? no DYNSYMTAB? */
-		if (symtab_hdr->sh_type == SHT_SYMTAB) goto found;
+
+		/* is this the symbol table? no DYNAMIC? */
+		if (hdr->sh_type == SHT_SYMTAB || hdr->sh_type == SHT_DYNSYM) {
+			sections[num_sections++] = section;
+		} 
 	}
-	vbprintf("no symbol table found for %s\n", filename);
-	return -1;
-found:
-	/* use elf_rawdata since there is no memory image of this data */
-	data = elf_rawdata(symtab_section, NULL); 
-	if (data == NULL) {
-		vbprintf("can't extract raw elf data for symbol table\n");
+
+	if (num_sections == 0) {
+		vbprintf("no symbol table found for %s\n", filename);
 		return -1;
 	}
-	symtab_data = (Elf32_Sym *)data->d_buf;
-	table_size  = symtab_hdr->sh_size/symtab_hdr->sh_entsize;
-  
-	/* get the string table */
-	strsym_index   = symtab_hdr->sh_link;
-	strsym_section = elf_getscn(elf, strsym_index);
-  
-	/* use elf_rawdata since there is no memory image of this data */
-	data = elf_rawdata(strsym_section, NULL); 
-	if (data == NULL) {
-		vbprintf("can't extract raw elf data for string section\n");
+
+	if (num_sections > PFMON_MAX_SECTIONS) {
+		vbprintf("too many symbol tables for %s\n", filename);
 		return -1;
 	}
-  
-	/* allocate space and copy content */
-	mod_sym->name_space = name_space = malloc(data->d_size);
-	if (name_space  == NULL) {
-		vbprintf("can't allocate space for string table\n"); 
-		return -1;
+	/*
+	 * phase 1: figure out max size
+	 */
+	table_size = 0;
+	for(s = 0; s < num_sections; s++) {
+
+		section = sections[s];
+		hdr     = elf32_getshdr(section);
+
+
+		/* 
+		 * use elf_rawdata since there is no memory image of this data
+		 */
+		data = elf_rawdata(section, NULL); 
+		if (data == NULL) {
+			warning("cannot extract raw elf data for symbol table\n");
+			return -1;
+		}
+		table_size  += hdr->sh_size/hdr->sh_entsize;
 	}
-  	memcpy(name_space, data->d_buf, data->d_size);
-  
-  	/* allocate space for the table and set it up */
-	mod_sym->symbol_tab = symbol_tab = malloc(table_size * sizeof(symbol_t));
+	/*
+	 * phase 2: allocate symbol_tab
+	 *
+	 * XXX: we allocate more than we need here because fo the split text/data
+	 */
+	symbol_tab = malloc(table_size * sizeof(symbol_t));
 	if (symbol_tab == NULL) {
-    		vbprintf("cannot allocate space for symbol table\n");
+		vbprintf("cannot allocate space for symbol table\n");
 		return -1;
 	}
 
-	if (sym_type == PFMON_TEXT_SYMBOL) use_text = 1;
-	if (sym_type == PFMON_DATA_SYMBOL) use_data = 1;
+	/*
+	 * phase 3: actual parsing of the tables
+	 */
+	j = 0;
+	for(s = 0; s < num_sections; s++) {
 
-  	for (i = 0, j= 0; i < table_size; i++) {
-    		symbol_tab[j].name  = name_space + symtab_data[i].st_name;
+		section = sections[s];
+		hdr     = elf32_getshdr(section);
 
-		if (symbol_tab[j].name == NULL || symbol_tab[j].name[0] == '\0') continue;
+		/* 
+		 * use elf_rawdata since there is no memory image of this data
+		 */
+		data = elf_rawdata(section, NULL); 
+		if (data == NULL) {
+			warning("cannot extract raw elf data for symbol table\n");
+			return -1;
+		}
+		symtab_data = (Elf32_Sym *)data->d_buf;
 
-		type = ELF_ST_TYPE(symtab_data[i].st_info);
-		if (type != STT_FUNC &&  type != STT_OBJECT && type != STT_COMMON) continue;
+		/*
+		 * only this table size now!
+		 */
+		table_size  = hdr->sh_size/hdr->sh_entsize;
 
-		if ((type == STT_FUNC && use_text) || (type == STT_OBJECT && use_data)) {
-    			symbol_tab[j].value = (unsigned long)symtab_data[i].st_value;
+		/* get the string table */
+		strsym_index   = hdr->sh_link;
+		strsym_section = elf_getscn(elf, strsym_index);
+
+		/* 
+		 * use elf_rawdata since there is no memory image of this data
+		 */
+		data = elf_rawdata(strsym_section, NULL); 
+		if (data == NULL) {
+			vbprintf("cannot extract raw elf data for string section\n");
+			return -1;
+		}
+		d_buf = data->d_buf;
+	
+		p = NULL;
+
+		for (i = 0; i < table_size; i++) {
+
+			type  = ELF_ST_TYPE(symtab_data[i].st_info);
+			str   = d_buf + symtab_data[i].st_name;
+			value = symtab_data[i].st_value;
+
+			/*
+			 * dicard if wrong type
+			 */
+			if ((type != STT_FUNC && use_text) || (type != STT_OBJECT && type != STT_COMMON && use_data)) 
+				continue;
+			/*
+			 * discard undefined symbols
+			 */
+			if (symtab_data[i].st_shndx == SHN_UNDEF) {
+				//printf("undef %s\n", str);
+				continue;
+			}
+
+			/*
+			 * discard no name symbols
+			 */
+			if (str == d_buf|| *str == '\0') continue;
+
+			if (map) {
+				/*
+				 * shortcut: try reuse previous phdrs
+				 */
+				if (p && value >= start && value < end) goto skip_phdrs;
+
+				num_scans++;
+				for(k=0, p = phdrs; k < phnums; k++, p++) {
+					if ((p->p_type & PT_LOAD) == 0) continue;
+					start = p->p_vaddr;
+					end   = p->p_vaddr + p->p_memsz;
+					if (value >= start && value < end) break;
+				}
+				/*
+				 * that happens for dynamic symbols
+				 */
+				if (k == phnums) {
+					/*
+					 * required for shortcut
+					 */
+					p = NULL;
+					continue;
+				}
+skip_phdrs:
+				fpos = p->p_offset + value - start;
+
+				if (fpos < map->map_offset || (fpos-map->map_offset) >= map->map_size)
+					continue;
+
+				value = map->map_start + (fpos-map->map_offset);
+				//printf("%20.20s @0x%lx num_scan=%lu\n", symbol_tab[j].name, value, num_scans);
+			}
+			s_tmp = NULL;
+#ifdef CONFIG_PFMON_DEMANGLE
+			if (options.opt_dem_type) {
+				int flg;
+				flg = options.opt_dem_type == 2 ?  DMGL_JAVA : 0;
+				s_tmp = cplus_demangle(data->d_buf + symtab_data[i].st_name, DMGL_TYPES|DMGL_ANSI|flg);
+			} 
+#endif
+			if (s_tmp == NULL)
+				s_tmp  = strdup(data->d_buf + symtab_data[i].st_name);
+			symbol_tab[j].name  = s_tmp;
+			symbol_tab[j].value = value;
 			symbol_tab[j].size  = symtab_data[i].st_size;
-			symbol_tab[j].type  = type == STT_FUNC ? PFMON_TEXT_SYMBOL : PFMON_DATA_SYMBOL; 
+			symbol_tab[j].type  = sym_type;
 			j++;
 		}
 	}
-  	mod_sym->nsyms = j;
-	qsort(symbol_tab, j, sizeof(symbol_t), symcmp);
+
+	if (j) {
+		/*
+		 * sort symbols by address, necessary to speed up search
+		 */
+		qsort(symbol_tab, j, sizeof(symbol_t), symcmp);
+
+		mod_sym->symbol_tab = symbol_tab;
+		mod_sym->min_addr   = symbol_tab[0].value;
+		mod_sym->nsyms      = j;
+	} else {
+		free(symbol_tab);
+		mod_sym->nsyms = 0;
+	}
+	vbprintf("table_size=%lu effective=%lu min_addr=0x%lx\n", table_size, j, mod_sym->min_addr);
 	return 0;
 }
 
@@ -471,19 +604,22 @@ load_elf_symbols(const char *filename, module_symbols_t *mod_sym, pfmon_sym_type
   	/* initial call to set internal version value */
 	if (elf_version(EV_CURRENT) == EV_NONE) {
 		DPRINT(("ELF library out of date"));
+		close(fd);
 		return -1;
 	}
 
   	/* prepare to read the entire file */
 	elf = elf_begin(fd, ELF_C_READ, NULL);
 	if (elf == NULL) {
-		DPRINT(("can't read %s\n", filename));
+		DPRINT(("cannot read %s\n", filename));
+		close(fd);
 		return -1;
 	}
 
 	/* error checking */
 	if (elf_kind(elf) != ELF_K_ELF) {
 		DPRINT(("%s is not an ELF file\n", filename));
+		close(fd);
 		return -1;
 	}
   
@@ -497,22 +633,25 @@ load_elf_symbols(const char *filename, module_symbols_t *mod_sym, pfmon_sym_type
 
 	switch (eident[EI_CLASS]) {
   		case ELFCLASS32:
-			if (read_sym32(elf, filename, mod_sym, sym_type, NULL)) {
+			if (read_sym32(elf, filename, mod_sym, sym_type, map)) {
 				DPRINT(("cannot extract symbols from %s\n", filename));
+				close(fd);
 				return -1;
 			}
 			break;
 		case ELFCLASS64:
 			if (read_sym64(elf, filename, mod_sym, sym_type, map)) {
 				DPRINT(("cannot extract symbols from %s\n", filename));
+				close(fd);
 				return -1;
 			}
     			break;
     		default:
     			DPRINT(("unsupported ELF class for %s\n", filename));
+			close(fd);
 			return -1;
 	}
-
+	close(fd);
 	return 0;
 }
 
@@ -1233,7 +1372,7 @@ found:
 	 * size = 0 when we could not figure it out
 	 */
 	if(size && (symbol_tab[l].value+size) < addr) {
-		printf("hole for 0x%lx\n", addr);
+		DPRINT(("hole for 0x%lx\n", addr));
 		return -1;
 	}
 	/*
@@ -1255,7 +1394,6 @@ found:
 		if (p && q && strcmp(q+1, p+1)) {
 			return -1;
 		}
-
 		/*
 		 * we may still be getting the wrong answer if there is
 		 * a hole with size = 0
@@ -1348,7 +1486,6 @@ pfmon_syms_hash_alloc(unsigned long hash_log_size, unsigned long max_entries, vo
 void 
 pfmon_syms_hash_free(void *hash_desc)
 {
-	if (options.opt_verbose) pfmon_hash_stats(hash_desc, stdout);
 	pfmon_hash_free(hash_desc);
 }
 
