@@ -50,7 +50,6 @@ typedef struct {
 	thread_state_t	   thread_state;
 } pfmon_thread_desc_t;
 
-
 typedef enum {
 	SESSION_INIT,
 	SESSION_RUN,
@@ -82,12 +81,8 @@ static pthread_cond_t  		results_cond;
 static pthread_key_t   		param_key;
 
 static pfmon_thread_desc_t	thread_desc[PFMON_MAX_CPUS];
-static sem_t 			ovfl_sem[PFMON_MAX_CPUS];
-
 static uint64_t			ovfl_cnts[PFMON_MAX_CPUS];
-
 static pfmon_sdesc_t		sdesc_sys_aggr;
-
 static pthread_t		master_thread_id;
 static pid_t			master_tid;
 
@@ -182,89 +177,12 @@ syswide_aggregate_results(pfmon_sdesc_t *sdesc)
 	}
 }
 
-/*
- * tid is the value returned by gettid()
- */
-static int
-setup_fasync(unsigned int mycpu, pfmon_ctxid_t id, pid_t tid)
-{
-	int r;
-
-	/*
-	 * setup asynchronous overflow notifications
-	 */
-	r = fcntl(id, F_SETFL, fcntl(id, F_GETFL, 0) | O_ASYNC);
-	if (r == -1) {
-		warning("CPU%u cannot set ASYNC: %s\n", mycpu, strerror(errno));
-		return -1;
-	}
-
-	/*
-	 * That's the tricky part here, especially because of threads.
-	 *
-	 * By default, SIGIO (asynchronous signal) is delivered to any one thread in 
-	 * the process, most likely the first one that does not have it blocked.
-	 *
-	 * For our purpose, we want the signal to be delivered to the thread that handles
-	 * the session for each CPU. For instance, the SIGIO for the session on 
-	 * CPU0 must be delivered to the thread running on CPU0, otherwise it is still 
-	 * doable using some mapping table, but more complicated and less efficient.
-	 *
-	 * NPTL : gettid() = kernel task pid
-	 *        getpid() = kernel tgid. Master thread tgid=pid
-	 *
-	 * LinuxThread:
-	 * 	  gettid() = does not really exist at the user level
-	 * 	  getpid() = kernel pid
-	 */
-
-	/*
-	 * There is a way to overrride the default SIGIO behavior by using a Linux 
-	 * specific extension (available via _GNU_SOURCE): the F_SETSIG command of
-	 * fcntl(). Using this command, you can specify the signal to be delivered
-	 * when there is an event for a file descriptor which is set in asynchronous
-	 * mode (O_ASYNC). If F_SETSIG is specified, then the signal handler
-	 * also receives the file descriptor number in si_fd. For what we do here 
-	 * we can live with SIGIO as the signal and we do not need the file descriptor. 
-	 * We use the call because of its nice side-effect on which thread gets the signal.
-	 * If a signal is explicitely requested, via F_SETSIG, then ONLY the task that 
-	 * is declared as the owner of the resource, via F_SETOWN, will receive the signal.
-	 * Therefore, We use this command to ensure that the right thread gets the signal 
-	 * independently of which thread library is used: LinuxThreads or NPTL.
-	 */
-
-	r = fcntl(id, F_SETSIG, SIGIO);
-	if (r == -1) {
-		warning("CPU%u cannot setsig: %s\n", mycpu, strerror(errno));
-		return -1;
-	}
-
-	/*
-	 * NPTL: 
-	 * 	If we specify getpid() here, the first worker thread on CPU0, 
-	 * 	receives the signal because the master thread has SIGIO blocked.
-	 * 	This is not quite what we want. That forces us to use the actual 
-	 * 	gettid() (kernel system call) value to identify precisely the 
-	 * 	thread we want. 
-	 *
-	 * LinuxThread:
-	 * 	getpid() identifies each thread uniquely, i.e., different in every 
-	 * 	thread,	therefore it works. The tid passed as argument is really
-	 * 	equivalent to getpid().
-	 */
-	r = fcntl(id, F_SETOWN, tid);
-	if (r == -1) {
-		warning("CPU%u cannot setown: %s\n", mycpu, strerror(errno));
-		return -1;
-	}
-	return 0;
-}
-
 static int 
 pfmon_sys_setup_context(pfmon_sdesc_t *sdesc, unsigned int cpu, pfmon_ctx_t *ctx)
 {
 	pfmon_smpl_desc_t *csmpl = &sdesc->csmpl;
 	pfmon_ctxid_t id;
+	int error;
 
 	/*
 	 * XXX: cache lines sharing for master list
@@ -274,12 +192,8 @@ pfmon_sys_setup_context(pfmon_sdesc_t *sdesc, unsigned int cpu, pfmon_ctx_t *ctx
 	memset(csmpl, 0, sizeof(pfmon_smpl_desc_t));
 
 	csmpl->cpu = cpu;
-
-	if (pfmon_create_context(ctx, &csmpl->smpl_hdr, &sdesc->ctxid) == -1 ) {
-		if (errno == EBUSY) {
-			warning("concurrent conflicting monitoring session is present in your system\n");
-		} else
-			warning("can't create perfmon context: %s\n", strerror(errno));
+	if (pfmon_create_context(ctx, &csmpl->smpl_hdr, &sdesc->ctxid, &error) == -1) {
+		warning("cannot create perfmon context: %s\n", strerror(error));
 		return -1;
 	}
 
@@ -302,11 +216,18 @@ pfmon_sys_setup_context(pfmon_sdesc_t *sdesc, unsigned int cpu, pfmon_ctx_t *ctx
 	if (options.opt_use_smpl) {
 		if (pfmon_setup_sampling_output(sdesc, &sdesc_sys_aggr.csmpl) == -1) return -1;
 		DPRINT(("-->sampling buffer at %p aggr_count=%p data=%p\n", csmpl->smpl_hdr, csmpl->aggr_count, csmpl->data));
+		sdesc->csmpl.map_size = ctx->ctx_map_size;
 	}
 
 	install_event_sets(sdesc);
 
-	if (pfmon_load_context(sdesc->ctxid, cpu) == -1) return -1;
+	if (pfmon_load_context(sdesc->ctxid, cpu, &error) == -1) {
+		if (error == EBUSY)
+			warning("CPU%u error conflicting monitoring session\n", cpu);
+		else
+			warning("CPU%u cannot load context: %s\n", cpu, strerror(error));
+		return -1;
+	}
 
 	return 0;
 }
@@ -328,11 +249,11 @@ setup_worker_signals(void)
 	
 	/*
 	 * POSIX: blocked signal mask is inherited from 
-	 * parent thread. The master thread has SIGIO blocked
+	 * parent thread. The master thread has SIGUSR1 blocked
 	 * therefore we must reenable it.
 	 */
 	sigemptyset(&my_set);
-        sigaddset(&my_set, SIGIO);
+        sigaddset(&my_set, SIGUSR1);
         pthread_sigmask(SIG_UNBLOCK, &my_set, NULL);
 
 }
@@ -343,12 +264,11 @@ do_measure_one_cpu(void *data)
 	pfmon_thread_desc_t *arg = (pfmon_thread_desc_t *)data;
 	pfmon_sdesc_t sdesc_var; /* local pfmon task descriptor */
 	pfmon_sdesc_t *sdesc = &sdesc_var; 
-	sem_t *my_ovfl_sem;
 	pfmon_ctxid_t ctxid = -1;
 	pid_t mytid = gettid();
 	unsigned int mycpu;
 	int aggr, needs_order;
-	int r;
+	int r, error;
 
 	/*
 	 * POSIX threads: 
@@ -362,7 +282,6 @@ do_measure_one_cpu(void *data)
 	setup_worker_signals();
 
 	mycpu       = arg->cpu;
-	my_ovfl_sem = ovfl_sem+arg->id;
 	aggr        = options.opt_aggr;
 
 	/*
@@ -403,23 +322,21 @@ do_measure_one_cpu(void *data)
 	}
 
 	r = pfmon_sys_setup_context(sdesc, arg->cpu, arg->ctx);
-	if (r) goto error;
+	if (r)
+		goto error;
 
 	ctxid       = sdesc->ctxid;
 	needs_order = aggr || sdesc->out_fp == stdout;
 
 	DPRINT(("sdesc->id=%u needs_order=%d\n", sdesc->id, needs_order));
 
-	if (options.opt_use_smpl) {
-		r = setup_fasync(mycpu, ctxid, mytid);
-		if (r) goto error;
-	}
-
 	/*
 	 * indicate we have reach the starting point
 	 */
 	arg->thread_state = THREAD_RUN;
-	barrier_wait(&barrier);
+
+	if (session_state == SESSION_ABORTED)
+		goto error;
 
 	/*
 	 * wait for the start signal
@@ -431,63 +348,83 @@ do_measure_one_cpu(void *data)
 	if (session_state == SESSION_ABORTED) goto error;
 
 	if (options.opt_dont_start == 0) {
-		if (pfmon_start(ctxid) == -1) goto error;
+		if (pfmon_start(ctxid, &error) == -1)
+			goto error;
 		vbprintf("CPU%u started monitoring\n", mycpu);
 	} else {
 		vbprintf("CPU%u pfmon does not start session\n", mycpu);
 	}
 
 	/*
-	 * print_interval is not possible when sampling
+	 * interval is not possible when sampling
 	 */
-	if (options.opt_print_interval) {
+	if (options.interval != PFMON_NO_TIMEOUT) {
+		struct timespec tm;
 
-		vbprintf("nsets=%u interval=%lu\n", sdesc->nsets, options.interval);
+		tm.tv_sec = options.interval / 1000;
+		tm.tv_nsec = (options.interval % 1000) * 1000000;
 
-		for(;session_state == SESSION_RUN;) {
+		for(;session_state == SESSION_RUN; ) {
 
-			sleep(options.interval);
+			nanosleep(&tm, NULL);
 
-			pfmon_stop(ctxid);
-
-			pthread_testcancel();
+			/*
+			 * we only check on stop to avoid printing too many messages
+			 */
+			if (pfmon_stop(ctxid, &error) == -1)
+				warning("CPU%u could not stop monitoring, CPU may be offline, check results\n", mycpu);
 
 			read_incremental_results(sdesc);
+			show_incr_results(sdesc, needs_order, PFMON_RESULTS_INTER);
 
-			show_results(sdesc, needs_order, PFMON_RESULTS_INTER);
-
-			pfmon_start(ctxid);
-
+			pfmon_start(ctxid, &error);
 		}
-
 		pthread_testcancel();
 	} else {
-		/*
-		 * we are sampling or we do not print at regular interval and wait
-		 * for the end of the session
-		 */
-		for(;;) {
-			sem_wait(my_ovfl_sem);
+		if (options.opt_use_smpl) {
+			for(;session_state == SESSION_RUN;) {
+				pfm_msg_t msg;
 
-			if (session_state != SESSION_RUN) break;
+				r = read(sdesc->ctxid, &msg, sizeof(msg));
+				if (r ==-1) {
+					/*
+					 * we have been interrupted by signal (likely),
+					 * go check session_state
+					 */
+					continue;
+				}
 
-			pthread_testcancel();
+				ovfl_cnts[mycpu]++;
 
-			if (aggr) pthread_mutex_lock(&pfmon_sys_aggr_lock);
+				pthread_testcancel();
 
-			pfmon_process_smpl_buf(sdesc, 0);
+				if (aggr) pthread_mutex_lock(&pfmon_sys_aggr_lock);
 
-			if (aggr) pthread_mutex_unlock(&pfmon_sys_aggr_lock);
+				r = pfmon_process_smpl_buf(sdesc, 0);
+				if (r)
+					vbprintf("CPU%-3u error processing buffer\n", mycpu);
 
-			pthread_testcancel();
+				if (aggr) pthread_mutex_unlock(&pfmon_sys_aggr_lock);
+
+				pthread_testcancel();
+			}
+		} else {
+			sigset_t myset;
+			int sig;
+
+        		sigemptyset(&myset);
+        		sigaddset(&myset, SIGUSR1);
+
+			for(;session_state == SESSION_RUN;) {
+				sigwait(&myset, &sig);
+			}
 		}
 	}
 
-	if (pfmon_stop(ctxid) == -1)
+	if (pfmon_stop(ctxid, &error) == -1)
 		warning("CPU%u could not stop monitoring, CPU may be offline, check results\n", mycpu);
 
 	vbprintf("CPU%-3u stopped monitoring\n", mycpu);
-
 	/*
 	 * read the final counts
 	 */
@@ -507,22 +444,25 @@ do_measure_one_cpu(void *data)
 
 		syswide_aggregate_results(sdesc);
 
-		if (options.opt_use_smpl) pfmon_process_smpl_buf(sdesc, 1);
+		if (options.opt_use_smpl)
+			pfmon_process_smpl_buf(sdesc, 1);
 
 		pthread_mutex_unlock(&pfmon_sys_aggr_lock);
 
 	} else {
-		if (options.opt_use_smpl) pfmon_process_smpl_buf(sdesc, 1);
+		if (options.opt_use_smpl)
+			pfmon_process_smpl_buf(sdesc, 1);
 
 		show_results(sdesc, needs_order, PFMON_RESULTS_FINAL);
 
 		close_results(sdesc);
 	}
 
-	if (options.opt_use_smpl && options.opt_aggr == 0) {
-		pfmon_close_sampling_output(sdesc, &sdesc->csmpl, -1, mycpu);
+	if (options.opt_use_smpl) {
+		if (options.opt_aggr == 0)
+			pfmon_close_sampling_output(sdesc, &sdesc->csmpl, -1, mycpu);
+		munmap(sdesc->csmpl.smpl_hdr, sdesc->csmpl.map_size);
 	}
-
 	close(sdesc->ctxid);
 
 	arg->thread_state = THREAD_DONE;
@@ -535,19 +475,15 @@ error:
 	if (sdesc->ctxid > -1)
 		close(sdesc->ctxid);
 
-	vbprintf("CPU%-3u session aborted\n", mycpu);
-
-	if (options.opt_use_smpl && options.opt_aggr == 0) {
-		pfmon_close_sampling_output(sdesc, &sdesc->csmpl, -1, mycpu);
-	}
-
 	arg->thread_state = THREAD_ERROR;
 
-	/*
-	 * indicate we have reach the starting point BUT 
-	 * with failures.
-	 */
-	barrier_wait(&barrier);
+	vbprintf("CPU%-3u session aborted\n", mycpu);
+
+	if (options.opt_use_smpl) {
+		if (options.opt_aggr == 0)
+			pfmon_close_sampling_output(sdesc, &sdesc->csmpl, -1, mycpu);
+		munmap(sdesc->csmpl.smpl_hdr, sdesc->csmpl.map_size);
+	}
 
 	pthread_exit((void *)(~0UL));
 	/* NO RETURN */
@@ -601,36 +537,25 @@ syswide_sigchild_handler(int n, siginfo_t *info, void *sc)
 	 * if we have a session timeout+child, then we are not using sleep
 	 * therefore it is safe to clear the alarm.
 	 */
-	if (options.session_timeout || options.trigger_delay) alarm(0);
+	if (options.session_timeout != PFMON_NO_TIMEOUT || options.trigger_delay)
+		alarm(0);
 
 	if (syswide_sigreason == REASON_NONE) syswide_sigreason = REASON_CHILD;
 }
 
 /*
  * must be executed by worker on each CPU and never by master thread
- *
- * assume signal is delivered to the thread on the right CPU.
+ * do not know which worker thread will execute the handler but this is
+ * fine because all we care about is that the thread is interrupted 
+ * from any blocking system calls it might have been into so that it
+ * can go back to mainloop and check session_state
  */
 static void
-syswide_sigio_handler(int n, siginfo_t *info, void *sc)
+syswide_sigusr1_handler(int n, siginfo_t *info, void *sc)
 {
-	pfmon_thread_desc_t *arg = (pfmon_thread_desc_t *)pthread_getspecific(param_key);
-
-	if (pthread_equal(pthread_self(), master_thread_id)) {
-		warning("error: SIGIO received by master thread\n");
-		return;
-	}
-
-	/* keep some statistics */
-	ovfl_cnts[arg->cpu]++;
-
-	/* 
-	 * force processing of the sampling buffer upon return from the handler
-	 */
-	sem_post(ovfl_sem+arg->id);
+	/* nothing to do */
 }
 
-
 static void
 setup_global_signals(void)
 {
@@ -647,7 +572,7 @@ setup_global_signals(void)
 	 * the master thread. Therefore all the per-CPU worker thread
 	 * MUST have those signals blocked.
 	 *
-	 * Conversly, SIGIO must be delivered to the worker threads.
+	 * Conversly, SIGUSR1 must be delivered to the worker threads.
 	 * We cannot control which of the worker thread will get the
 	 * signal. 
 	 */
@@ -698,25 +623,23 @@ setup_global_signals(void)
 	sigaction (SIGINT, &act, 0);
 
 	/*
-	 * install global SIGIO handler
+	 * install global SIGUSR1 handler (termination signal)
 	 * used by worker thread only,
 	 * no need to have other signals 
-	 * mask during handler execution because
+	 * masked during handler execution because
 	 * they are completely masked for the thread.
 	 */
 	memset(&act,0,sizeof(act));
 
-	act.sa_handler = (__sighandler_t)syswide_sigio_handler;
-	act.sa_flags   = SA_SIGINFO;
-
-	sigaction (SIGIO, &act, 0);
+	act.sa_handler = (__sighandler_t)syswide_sigusr1_handler;
+	sigaction (SIGUSR1, &act, 0);
 
 	/*
-	 * master thread does not handle SIGIO
+	 * master thread does not handle SIGUSR1
 	 * (inherited in sub threads)
 	 */
         sigemptyset(&my_set);
-        sigaddset(&my_set, SIGIO);
+        sigaddset(&my_set, SIGUSR1);
 	pthread_sigmask(SIG_BLOCK, &my_set, NULL);
 }
 
@@ -778,7 +701,7 @@ delimit_session(char **argv)
 
 		the_time = 0;
 
-		if (options.session_timeout) {
+		if (options.session_timeout != PFMON_NO_TIMEOUT) {
 			printf("<session to end in %u seconds>\n", options.session_timeout);
 
 			left_over = sleep(options.session_timeout);
@@ -812,7 +735,6 @@ delimit_session(char **argv)
 	}
 
 	if (pid == 0) {		 
-		printf("child fd =%d getfd=0x%x\n", 5, fcntl(5, F_GETFD,0));
 		pid = getpid();
 
 		if (options.opt_verbose) {
@@ -935,7 +857,7 @@ measure_system_wide(pfmon_ctx_t *ctx, char **argv)
 
 	for(i=0, j=0; num_cpus; i++) {
 		
-		if (PFMON_CPUMASK_ISSET(options.virt_cpu_mask, i) == 0) continue;
+		if (pfmon_bitmask_isset(options.virt_cpu_mask, i) == 0) continue;
 
 		thread_desc[j].id    = j;
 		thread_desc[j].cpu   = i;
@@ -943,8 +865,6 @@ measure_system_wide(pfmon_ctx_t *ctx, char **argv)
 		thread_desc[j].thread_state = THREAD_STARTED;
 		thread_desc[j].ctx   = ctx;
 
-		sem_init(ovfl_sem+j, 0, 0);
-
 		ret = pthread_create(&thread_desc[j].thread_id, 
 				     NULL, 
 				     (void *(*)(void *))do_measure_one_cpu, 
@@ -961,21 +881,20 @@ measure_system_wide(pfmon_ctx_t *ctx, char **argv)
 	/* reload number of cpus */
 	num_cpus = options.selected_cpus;
 
-	/*
-	 * wait for all worker thread to have reach the starting point
-	 */
-	barrier_wait(&barrier);
-
-	/*
-	 * check if some threads got problems
-	 */
-	for(i=0; i < num_cpus ; i++) {
-		if (thread_desc[i].thread_state == THREAD_ERROR) {
-			DPRINT(("CPU%-4u thread aborted\n", i));
-			goto abort;
+	for (j=0; j < num_cpus;) {
+		DPRINT(("nthread_ok=%d\n", j));
+		/*
+		 * poll thread state for errors
+		 */
+		for(i=0; i < num_cpus; i++) {
+			if (thread_desc[i].thread_state == THREAD_ERROR) {
+				vbprintf("CPU%-4u thread aborted\n", i);
+				goto abort;
+			}
+			if (thread_desc[i].thread_state == THREAD_RUN)
+				j++;
 		}
 	}
-
 	session_state = SESSION_RUN;
 
 	if (delimit_session(argv) == -1) goto abort;
@@ -988,7 +907,10 @@ measure_system_wide(pfmon_ctx_t *ctx, char **argv)
 	/*
 	 * get worker thread out of their mainloop
 	 */
-	for (i=0; i < num_cpus; i++) sem_post(ovfl_sem+i);
+	for (i=0; i < num_cpus; i++) {
+		if (thread_desc[i].thread_state != THREAD_ERROR)
+			pthread_kill(thread_desc[i].thread_id, SIGUSR1);
+	}
 
 	DPRINT(("main thread after session stop\n"));
 
@@ -1000,9 +922,8 @@ measure_system_wide(pfmon_ctx_t *ctx, char **argv)
 
 	if (options.opt_aggr) {
 		print_results(&sdesc_sys_aggr); /* mask taken from options.virt_cpu_mask */
-		if (options.opt_use_smpl) {
+		if (options.opt_use_smpl)
 			pfmon_close_aggr_sampling_output(&sdesc_sys_aggr, &sdesc_sys_aggr.csmpl);
-		}
 	}
 
 	pthread_key_delete(param_key);
@@ -1012,7 +933,7 @@ measure_system_wide(pfmon_ctx_t *ctx, char **argv)
 	if (options.opt_verbose && options.opt_use_smpl) {
 		num_cpus = options.selected_cpus;
 		for(i=0; num_cpus; i++) { 
-			if (PFMON_CPUMASK_ISSET(options.virt_cpu_mask, i)) {
+			if (pfmon_bitmask_isset(options.virt_cpu_mask, i)) {
 				vbprintf("CPU%-4u %"PRIu64" sampling buffer overflows\n", i, ovfl_cnts[i]);
 				num_cpus--;
 			}
@@ -1021,11 +942,13 @@ measure_system_wide(pfmon_ctx_t *ctx, char **argv)
 
 	return 0;
 abort:
-	if (session_state == SESSION_RUN) 
-		for (i=0; i < num_cpus; i++) sem_post(ovfl_sem+i);
-
 	session_state = SESSION_ABORTED;
 
+	if (session_state == SESSION_RUN) 
+		for (i=0; i < num_cpus; i++)
+			if (thread_desc[i].thread_id)
+				pthread_kill(thread_desc[i].thread_id, SIGUSR1);
+
 	num_cpus = options.selected_cpus;
 	vbprintf("aborting %lu threads\n", num_cpus);
 
@@ -1033,6 +956,7 @@ abort:
 		DPRINT(("cancelling on CPU%lu\n", i));
 		pthread_cancel(thread_desc[i].thread_id);
 	}
+
 	for(i=0; i < num_cpus; i++) {
 		ret = pthread_join(thread_desc[i].thread_id, &retval);
 		if (ret != 0) warning("cannot join thread %i\n", i);
