@@ -64,6 +64,10 @@
 #define PFMON_SDESC_FD_HASH_SIZE	256
 #define PFMON_SDESC_FD_HASH(x)		((x) & (PFMON_SDESC_FD_HASH_SIZE-1))
 
+#ifndef __WNOTHREAD
+#define __WNOTHREAD     0x20000000
+#endif
+
 /*
  * better is cache line size aligned
  */
@@ -386,20 +390,16 @@ task_setup_pfm_context(pfmon_sdesc_t *sdesc, pfmon_ctx_t *ctx)
 	pfmon_smpl_desc_t *csmpl = &sdesc->csmpl;
 	pfmon_ctxid_t id;
 	int activate_brkpoints = 0;
-	int ret;
+	int ret, error;
 
 	pfmon_clone_sets(options.sets, sdesc);
 
 	memset(csmpl, 0, sizeof(pfmon_smpl_desc_t));
 
-	if (pfmon_create_context(ctx, &csmpl->smpl_hdr, &sdesc->ctxid) == -1 ) {
-		if (errno == EBUSY) {
-			warning("concurrent conflicting monitoring session is present in your system\n");
-		} else
-			warning("can't create perfmon context: %s(%d)\n", strerror(errno), errno);
+	if (pfmon_create_context(ctx, &csmpl->smpl_hdr, &sdesc->ctxid, &error) == -1 ) {
+		warning("can't create perfmon context: %s\n", strerror(error));
 		return -1;
 	}
-
 	id = sdesc->ctxid;
 
 	/*
@@ -416,11 +416,20 @@ task_setup_pfm_context(pfmon_sdesc_t *sdesc, pfmon_ctx_t *ctx)
 	if (options.opt_use_smpl) {
 		if (pfmon_setup_sampling_output(sdesc, &sdesc_task_aggr.csmpl) == -1)
 			return -1;
+		sdesc->csmpl.map_size = ctx->ctx_map_size;
 	}
 
-	if (install_event_sets(sdesc) == -1) return -1;
+	if (install_event_sets(sdesc) == -1)
+		return -1;
 
-	if (pfmon_load_context(sdesc->ctxid, sdesc->tid) == -1) return -1;
+	if (pfmon_load_context(sdesc->ctxid, sdesc->tid, & error) == -1) {
+		if (error == EBUSY)
+			warning("error concurrent system-wide monitoring session exists\n");
+		else
+			warning("cannot attach context to %d: %s\n", sdesc->tid, strerror(error));
+
+		return -1;
+	}
 
 	if (sdesc->num_code_triggers) {
 		ret = install_code_triggers(sdesc);
@@ -445,9 +454,9 @@ task_reset_pfm_context(pfmon_sdesc_t *sdesc)
 	pfmon_event_set_t *set;
 	pfmon_pmd_t *pd;
 	unsigned int i, count;
-	int ret = -1;
+	int ret = -1, error;
 
-	vbprintf("[%d] resetting perfmon state\n", sdesc->tid);
+	vbprintf("[%d] resetting perfmon state hdr=%p\n", sdesc->tid, sdesc->csmpl.smpl_hdr);
 
 	for (set = sdesc->sets; set; set = set->next) {
 		pd = set->master_pd;
@@ -458,14 +467,16 @@ task_reset_pfm_context(pfmon_sdesc_t *sdesc)
 	}
 
 	/*
-	 * tas is stopped but we need to unload because we reprogram
+	 * task is stopped but we need to unload because we reprogram
 	 * the event sets
 	 */
-	if (pfmon_unload_context(sdesc->ctxid) == -1) return -1;
+	if (pfmon_unload_context(sdesc->ctxid, &error) == -1)
+		return -1;
 
 	install_event_sets(sdesc);
 
-	if (pfmon_load_context(sdesc->ctxid, sdesc->tid) == -1) return -1;
+	if (pfmon_load_context(sdesc->ctxid, sdesc->tid, &error) == -1)
+		return -1;
 
 	/* monitoring is always stopped on reload */
 
@@ -480,7 +491,7 @@ error:
 }
 
 static int
-task_collect_results(pfmon_sdesc_t *sdesc)
+task_collect_results(pfmon_sdesc_t *sdesc, int is_final)
 {
 	/*
 	 * no more context attached, there is nothing we can do here
@@ -513,13 +524,19 @@ task_collect_results(pfmon_sdesc_t *sdesc)
 
 		close_results(sdesc);
 	}
-	if (options.opt_use_smpl && options.opt_aggr == 0) 
-		pfmon_close_sampling_output(sdesc, &sdesc->csmpl, sdesc->tid, 0);
-
+	if (options.opt_use_smpl) {
+		if (options.opt_aggr == 0)
+			pfmon_close_sampling_output(sdesc, &sdesc->csmpl, sdesc->tid, 0);
+		/*
+		 * this function is collect on exec-split but we gather partial results,
+		 * thus we do not want to unmap the buffer just yet
+		 */
+		if (is_final)
+			munmap(sdesc->csmpl.smpl_hdr, sdesc->csmpl.map_size);
+	}
 	return 0;
 }
 
-
 /*
  * allocates sdesc with accompanying ctx_arg area
  */
@@ -528,11 +545,9 @@ pfmon_sdesc_alloc(void)
 {
 	pfmon_sdesc_t *tmp;
 
-	tmp = malloc(sizeof(pfmon_sdesc_t) + options.ctx_arg_size);
-	if (tmp == NULL) fatal_error("cannot allocate sdesc\n");
-
-	/* don't need to initialize ctx area, will be done via copy */
-	memset(tmp, 0, sizeof(pfmon_sdesc_t));
+	tmp = calloc(1, sizeof(pfmon_sdesc_t) + options.ctx_arg_size);
+	if (tmp == NULL)
+		fatal_error("cannot allocate sdesc\n");
 
 	pthread_mutex_init(&tmp->lock, PTHREAD_MUTEX_TIMED_NP);
 
@@ -542,6 +557,7 @@ pfmon_sdesc_alloc(void)
 static void
 pfmon_sdesc_free(pfmon_sdesc_t *t)
 {
+	pfmon_free_sets(t);
 	free(t);
 }
 
@@ -696,24 +712,28 @@ static const char *sdesc_type_str[]= {
 	"clone"
 };
 
+static inline void
+pfmon_sdesc_set_pid(pfmon_sdesc_t *sdesc, pid_t new_tid)
+{
+	sdesc->pid = find_pid_attr(new_tid, "Tgid");
+	sdesc->ppid = find_pid_attr(new_tid, "PPid");
+	sdesc->tid = new_tid;
+	pfmon_sdesc_pid_hash_add(sdesc_pid_hash, sdesc);
+}
+
 static pfmon_sdesc_t *
-pfmon_sdesc_new(int type, pfmon_sdesc_t *parent, pid_t new_pid)
+pfmon_sdesc_new(int type, pfmon_sdesc_t *parent, pid_t new_tid)
 {
 	pfmon_sdesc_t *sdesc;
 	unsigned int n;
 
 	sdesc = pfmon_sdesc_alloc();
 
-	sdesc->type = type;
-	sdesc->tid  = new_pid;
 
-	if (parent) strcpy(sdesc->cmdline, parent->cmdline);
+	pfmon_sdesc_set_pid(sdesc, new_tid);
 
-	/*
-	 * for a pure clone, process id is the same, the new_pid refers
-	 * to the new tid
-	 */
-	sdesc->pid = type == PFMON_SDESC_CLONE && parent ? parent->pid : new_pid;
+	if (parent)
+		strcpy(sdesc->cmdline, parent->cmdline);
 
 	/*
 	 * the following rules apply for flags inheritance:
@@ -724,23 +744,16 @@ pfmon_sdesc_new(int type, pfmon_sdesc_t *parent, pid_t new_pid)
 	 * fl_attached		: inherited
 	 */
 	if (parent) {
-		sdesc->ppid  = parent->pid;
-		sdesc->ptid  = parent->tid;
-
-		if (parent->fl_attached) sdesc->fl_attached = 1;
-		if (parent->fl_monitoring) sdesc->fl_monitoring = 1;
-
-	} else {
-		sdesc->ppid  = -1;
-		sdesc->ptid  = -1;
+		if (parent->fl_attached)
+			sdesc->fl_attached = 1;
+		if (parent->fl_monitoring)
+			sdesc->fl_monitoring = 1;
 	}
 
-	if (type == PFMON_SDESC_ATTACH) sdesc->fl_attached = 1;
-
-	if (new_pid) pfmon_sdesc_pid_hash_add(sdesc_pid_hash, sdesc);
+	if (type == PFMON_SDESC_ATTACH)
+		sdesc->fl_attached = 1;
 
 	sdesc->ctxid  = -1; /* not associated with a context */
-
 	sdesc->refcnt = 1;
 
 	/*
@@ -783,19 +796,6 @@ pfmon_sdesc_new(int type, pfmon_sdesc_t *parent, pid_t new_pid)
 	return sdesc;
 }
 
-static void
-pfmon_sdesc_set_pid(pfmon_sdesc_t *sdesc, pid_t new_pid)
-{
-	sdesc->tid  = new_pid;
-
-	/*
-	 * for a pure clone, process id is the same, the new_pid refers
-	 * to the new tid
-	 */
-	sdesc->pid = sdesc->type == PFMON_SDESC_CLONE ? sdesc->ppid : new_pid;
-	pfmon_sdesc_pid_hash_add(sdesc_pid_hash, sdesc);
-}
-
 /*
  * return:
  * 	0 : not interested
@@ -822,9 +822,10 @@ pfmon_sdesc_exec(pfmon_sdesc_t *sdesc)
 	/*
 	 * deactivate symbol hash table after first exec
 	 */
-	if ((sdesc->ppid != -1 || sdesc->exec_count) && options.opt_addr2sym ) {
+	if (sdesc->exec_count && options.opt_addr2sym) {
 		options.opt_addr2sym = 0;
 		vbprintf("[%d] deactivated symbol resolution because of multiple exec()\n", sdesc->tid);
+		for(;;);
 	}
 }
 
@@ -853,12 +854,8 @@ task_create(char **argv)
 	pid_t pid = 0;
 	int status, ret;
 
-	sdesc = pfmon_sdesc_new(PFMON_SDESC_VFORK, NULL, 0);
-	if (sdesc == NULL) return NULL;
-
 	if ((pid=vfork()) == -1) {
 		warning("cannot vfork process\n");
-		pfmon_sdesc_free(sdesc);
 		return NULL;
 	}
 
@@ -904,18 +901,19 @@ task_create(char **argv)
 	 */
 	if (WIFEXITED(status)) {
 		warning("error cannot monitor task %s(%d): exit status %d\n", argv[0], pid, WEXITSTATUS(status));
-		pfmon_sdesc_free(sdesc);
 		return NULL;
 	}
 
-	if (pfmon_setup_ptrace(pid)) {
+	sdesc = pfmon_sdesc_new(PFMON_SDESC_VFORK, NULL, pid);
+
+	if (sdesc == NULL || pfmon_setup_ptrace(pid)) {
 		/* get rid of the task, we cannot proceed */
 		status = ptrace(PTRACE_KILL, pid, NULL, NULL);
 		if (status != 0) warning("cannot kill task %d: %s\n", pid, strerror(errno));
-		pfmon_sdesc_free(sdesc);
+		if (sdesc)
+			pfmon_sdesc_free(sdesc);
 		return NULL;
 	}
-	pfmon_sdesc_set_pid(sdesc, pid);
 	pfmon_sdesc_exec(sdesc);
 
 	return sdesc;
@@ -926,7 +924,7 @@ task_attach(char **argv)
 {
 	pfmon_sdesc_t *sdesc;
 	pid_t pid = 0;
-	int status;
+	int status, ret;
 
 	sdesc = pfmon_sdesc_new(PFMON_SDESC_ATTACH, NULL, 0);
 	if (sdesc == NULL) return NULL;
@@ -940,7 +938,13 @@ task_attach(char **argv)
 		return NULL;
 	}
 
-	waitpid(pid, &status, WUNTRACED);
+	ret = waitpid(pid, &status, WUNTRACED|__WALL);
+	if (ret < 0) {
+		warning("error attaching to %d : %s\n", pid, strerror(errno));
+		ptrace(PTRACE_DETACH, pid, NULL, NULL);
+		pfmon_sdesc_free(sdesc);
+		return NULL;
+	}
 
 	/*
 	 * process is stopped at this point
@@ -954,7 +958,8 @@ task_attach(char **argv)
 	if (pfmon_setup_ptrace(pid)) {
 		/* cannot proceed, just detach */
 		status = ptrace(PTRACE_DETACH, pid, NULL, NULL);
-		if (status != 0) warning("cannot detach task %d: %s\n", pid, strerror(errno));
+		if (status != 0)
+			warning("cannot detach task %d: %s\n", pid, strerror(errno));
 		pfmon_sdesc_free(sdesc);
 		return NULL;
 	}
@@ -966,8 +971,6 @@ task_attach(char **argv)
 	return sdesc;
 }
 
-
-
 static void
 task_dispatch_sdesc(pfmon_sdesc_t *sdesc)
 {
@@ -1005,7 +1008,7 @@ task_pfm_init(pfmon_sdesc_t *sdesc, int from_exec, pfmon_ctx_t *ctx)
 	task_worker_msg_t msg;
 	pid_t tid;
 	int has_ctxid, was_monitoring;
-	int ret;
+	int ret, error;
 
 	tid = sdesc->tid;
 
@@ -1036,7 +1039,8 @@ task_pfm_init(pfmon_sdesc_t *sdesc, int from_exec, pfmon_ctx_t *ctx)
 	 * we always monitor the first process until it exec's.
 	 */
 	if (options.opt_follow_exec == 0) {
-		ret = sdesc->ppid != -1 || sdesc->exec_count ? 0 : 1;
+		//ret = sdesc->ppid != -1 || sdesc->exec_count ? 0 : 1;
+		ret = sdesc->exec_count ? 0 : 1;
 	} else {
 		ret = pfmon_sdesc_interesting(sdesc);
 	}
@@ -1055,7 +1059,7 @@ task_pfm_init(pfmon_sdesc_t *sdesc, int from_exec, pfmon_ctx_t *ctx)
 			if (options.opt_split_exec) {
 				if (sdesc->fl_monitoring) {
 					vbprintf("[%d] collecting results at exec\n", tid);
-					task_collect_results(sdesc);
+					task_collect_results(sdesc, 0);
 				}
 
 				if (sdesc->fl_dispatched) {
@@ -1075,7 +1079,7 @@ task_pfm_init(pfmon_sdesc_t *sdesc, int from_exec, pfmon_ctx_t *ctx)
 				 * code/data triggers are automatically cleared 
 				 * by the kernel on exec()
 				 */
-				pfmon_stop(sdesc->ctxid);
+				pfmon_stop(sdesc->ctxid, &error);
 			}
 			/*
 			 * monitoring is deactivated
@@ -1095,7 +1099,7 @@ task_pfm_init(pfmon_sdesc_t *sdesc, int from_exec, pfmon_ctx_t *ctx)
 	}
 	if (options.opt_split_exec && sdesc->ctxid != -1 && sdesc->fl_monitoring) {
 		vbprintf("[%d] collecting results at exec\n", tid);
-		task_collect_results(sdesc);
+		task_collect_results(sdesc, 0);
 	}
 
 	strcpy(sdesc->cmdline, sdesc->new_cmdline);
@@ -1124,7 +1128,18 @@ init_pfm:
 	 * we do the whole setup procedure. 
 	 */
 	if (has_ctxid == 0) {
-
+		/*
+		 * on some architectures (i386,x86-64), debug registers
+		 * are systematically inherited by children. We undo did
+		 * now to avoid getting spurious breakpoints. Should inheritance
+		 * be necessary then use the --trigger-code-follow and
+		 * --trigger-data-follow options
+		 */
+		if (options.num_code_triggers || options.num_data_triggers) {
+			ret = pfmon_disable_all_breakpoints(sdesc->pid);	
+			if (ret)
+				warning("error: could not disable all breakpoints for %d\n", sdesc->pid);
+		}
 		DPRINT(("setup perfmon ctx for [%d] monitoring=%d refcnt=%d: %s\n", 
 			tid, sdesc->fl_monitoring, sdesc->refcnt, sdesc->cmdline));
 
@@ -1135,7 +1150,7 @@ init_pfm:
 		 * we may defer actual activation until later
 		 */
 		if (options.opt_dont_start == 0) {
-			pfmon_start(sdesc->ctxid);
+			pfmon_start(sdesc->ctxid, &error);
 			vbprintf("[%d] activating monitoring\n", tid);
 		} else {
 			vbprintf("[%d] monitoring not activated\n", tid);
@@ -1166,7 +1181,7 @@ init_pfm:
 		 * need to restart now
 		 */
 		if (was_monitoring == 0 && options.opt_dont_start == 0) {
-			pfmon_start(sdesc->ctxid);
+			pfmon_start(sdesc->ctxid, &error);
 			vbprintf("[%d] restarting monitoring\n", tid);
 		}
 	}
@@ -1200,7 +1215,8 @@ task_pfm_exit(pfmon_sdesc_t *sdesc)
 	/*
 	 * a task descriptor not associated with a perfmon context, simply destroy
 	 */
-	if (sdesc->ctxid != -1) task_collect_results(sdesc);
+	if (sdesc->ctxid != -1)
+		task_collect_results(sdesc, 1);
 
 	pfmon_sdesc_exit(sdesc);
 }
@@ -1212,7 +1228,7 @@ task_handle_trigger(pfmon_sdesc_t *sdesc)
 	unsigned long addr, rp;
 	int is_start, is_repeat, is_data = 0;
 	pid_t tid;
-	int ret;
+	int ret, error;
 	
 	tid = sdesc->tid;
 
@@ -1237,6 +1253,8 @@ task_handle_trigger(pfmon_sdesc_t *sdesc)
 		is_start ? "start" : "stop",
 		is_data ? "data" : "code",
 		addr);
+printf("press a key\n");getchar();
+	load_pid_syms(tid, &options.primary_syms);
 
 	/*
 	 * dynamically insert stop trigger with start trigger is of type "func"
@@ -1259,10 +1277,10 @@ task_handle_trigger(pfmon_sdesc_t *sdesc)
 	}
 skip_stop:
 	if (is_start) {
-		pfmon_start(sdesc->ctxid);
+		pfmon_start(sdesc->ctxid, &error);
 		vbprintf("[%d] activating monitoring at trigger start\n", tid);
 	} else {
-		pfmon_stop(sdesc->ctxid);
+		pfmon_stop(sdesc->ctxid, &error);
 		vbprintf("[%d] stopping monitoring at trigger stop\n", tid);
 	}
 
@@ -1322,8 +1340,9 @@ task_detach(pfmon_sdesc_t *sdesc)
 			task_worker_send_msg(sdesc->cpu, &msg, 1);
 			was_dispatched = 1;
 		}
-		task_collect_results(sdesc);
-		if (was_dispatched) pfmon_sdesc_exit(sdesc);
+		task_collect_results(sdesc, 1);
+		if (was_dispatched)
+			pfmon_sdesc_exit(sdesc);
 	}
 
 	vbprintf("detached from [%d]\n", pid);
@@ -1335,12 +1354,18 @@ task_detach(pfmon_sdesc_t *sdesc)
 	return 0;
 }
 
+int tgkill(pid_t tgid, pid_t pid, int sig)
+{
+	return syscall(__NR_tgkill, tgid, pid, sig);
+}
+
 static void
 task_force_exit(void)
 {
 	pfmon_sdesc_t *t;
 	unsigned int i;
-	int sig;
+	long sig;
+	int ret;
 
 	for(i=0; i < PFMON_SDESC_PID_HASH_SIZE; i++) {
 		t = sdesc_pid_hash[i];
@@ -1352,9 +1377,9 @@ task_force_exit(void)
 				sig = SIGKILL;
 			}
 
-			vbprintf("sending signal %d to [%d]\n", sig, t->tid);
-			kill(t->tid, sig);
-
+			ret = tgkill(t->pid, t->tid, sig);
+			vbprintf("sending signal %d to [%d]\n",
+				sig, t->tid, ret);
 			t = t->next;
 		}
 	}
@@ -1390,7 +1415,7 @@ task_mainloop(pfmon_ctx_t *ctx, char **argv)
 	/* actually start the task */
 	pfmon_continue(sdesc->tid, 0);
 
-	if (options.session_timeout) {
+	if (options.session_timeout != PFMON_NO_TIMEOUT) {
 		alarm(options.session_timeout);
 		vbprintf("arming session alarm to %u seconds\n", options.session_timeout);
 	}
@@ -1399,7 +1424,6 @@ task_mainloop(pfmon_ctx_t *ctx, char **argv)
 
 	work_todo = 1;
 
-	//while (task_info.num_active_sdesc) {
 	for(;work_todo;) {
 
 		unmask_global_signals();
@@ -1407,7 +1431,7 @@ task_mainloop(pfmon_ctx_t *ctx, char **argv)
 		sem_wait(&master_work_sem);
 
 		mask_global_signals();
-
+retry:
 		while (work_todo && (tid = wait4(-1, &status, wait_type, &rusage)) > 0) {
 
 			if (needs_time) gettimeofday(&tv, NULL);
@@ -1541,7 +1565,6 @@ task_mainloop(pfmon_ctx_t *ctx, char **argv)
 					task_detach(sdesc);
 					continue;
 				}
-				//if (sdesc->fl_seen_stopsig == 0 && (sdesc->fl_active)) {
 				if (sdesc->fl_seen_stopsig == 0 && (sdesc->fl_monitoring)) {
 					sdesc->fl_seen_stopsig = 1;
 					r = task_pfm_init(sdesc, 0, ctx);
@@ -1575,8 +1598,9 @@ task_mainloop(pfmon_ctx_t *ctx, char **argv)
 					printf("session interrupted for unknown reason!\n");
 			}
 			task_force_exit();
-
 			cleaning_up  = 1;
+			wait_type |= __WNOTHREAD|__WALL;
+			goto retry;
 		}
 	}
 
@@ -1584,6 +1608,7 @@ task_mainloop(pfmon_ctx_t *ctx, char **argv)
 		print_results(&sdesc_task_aggr);
 		if (options.opt_use_smpl)
 			pfmon_close_aggr_sampling_output(&sdesc_task_aggr, &sdesc_task_aggr.csmpl);
+		munmap(sdesc->csmpl.smpl_hdr, sdesc->csmpl.map_size);
 	}
 
 	vbprintf("created tasks        : %lu\n"
@@ -1804,7 +1829,9 @@ task_worker_mainloop(void *data)
 					/*
 					 * ack the removal
 					 */
-					write(workers[mycpu].from_worker[1], &msg, sizeof(msg));
+					ret = write(workers[mycpu].from_worker[1], &msg, sizeof(msg));
+					if (ret != sizeof(msg))
+						warning("cannot ack remove task message\n");
 					break;
 
 				case PFMON_TASK_MSG_QUIT:
@@ -1829,7 +1856,7 @@ task_create_workers(void)
 	 * compute number of accessible CPUs
 	 */
 	for(i=0, j=0; i < options.online_cpus; i++) {
-		if (PFMON_CPUMASK_ISSET(options.phys_cpu_mask, i)) ncpus++;
+		if (pfmon_bitmask_isset(options.phys_cpu_mask, i)) ncpus++;
 	}
 
 	nfiles = sysconf(_SC_OPEN_MAX);
@@ -1852,7 +1879,7 @@ task_create_workers(void)
 
 	for (i=0, j=0; i < options.online_cpus; i++) {
 
-		if (PFMON_CPUMASK_ISSET(options.phys_cpu_mask, i) == 0) continue;
+		if (pfmon_bitmask_isset(options.phys_cpu_mask, i) == 0) continue;
 
 		workers[j].cpu_id = i;
 
