@@ -204,7 +204,7 @@ setup_event_set(pfmon_event_set_t *set)
 {
 	char *event_name, *next_event_name;
 	size_t l;
-	unsigned int cnt;
+	unsigned int i, cnt, idx;
 	int ret;
 
 	for (event_name = set->events_str, cnt = 0;
@@ -232,7 +232,22 @@ setup_event_set(pfmon_event_set_t *set)
 	return;
 
 error:
-	fatal_error("event %s : %s\n", event_name, pfm_strerror(ret));
+	if (ret != PFMLIB_ERR_UMASK || pfm_find_event(event_name, &idx) != PFMLIB_SUCCESS)
+		fatal_error("event %s : %s\n", event_name, pfm_strerror(ret));
+
+	pfm_get_max_event_name_len(&l);
+	next_event_name = malloc(l+1);
+	if (!next_event_name)
+		fatal_error("event %s : %s\n", event_name, pfm_strerror(ret));
+
+	pfm_get_num_event_masks(idx, &cnt);
+	warning("event %s needs at least a unit mask among: ", event_name);
+	for (i = 0; i < cnt ; i++) {
+		pfm_get_event_mask_name(idx, i, next_event_name, l);
+		warning("[%s] ", next_event_name);
+	}
+	free(next_event_name);
+	fatal_error("\n");
 too_many:
 	fatal_error("too many events specified, max=%d\n", options.max_counters);
 }
@@ -429,6 +444,7 @@ error:
 	return -1;
 }
 
+
 /*
  * we abuse libpfm's return values here
  */
@@ -660,12 +676,26 @@ pfmon_extract_cmdline(pid_t pid, char *cmdline, int maxlen)
 		return -1;
 	}
 
+	/*
+	 * extract full cmd pathname
+	 */
+	sprintf(fn, "/proc/%d/exe", pid);
+	m = readlink(fn, cmdline, maxlen-1);
+	if (m < 0) {
+		warning("cannot read symlink %s\n", fn);
+		return -1;
+	}
+	cmdline[m] = ' ';
+	maxlen -= m;
+	n = m;
+
 	sprintf(fn, "/proc/%d/cmdline", pid);
 	fd = open(fn, O_RDONLY);
 	if (fd<0) {
 		warning("cannot open  %s: %s\n", fn, strerror(errno));
 		return -1;
 	}
+
 	m = read(fd, cmdargs, sizeof(cmdargs)-1);
 	if (m<0) {
 		warning("cannot read %s : %s\n", fn, strerror(errno));
@@ -676,7 +706,12 @@ pfmon_extract_cmdline(pid_t pid, char *cmdline, int maxlen)
 
 	cmdargs[m]='\0';
 
-	for(n=0,i=0; i<m;i++) {
+	/*
+	 * skip command, we got it from readlink
+	 */
+	for(i=0; cmdargs[i]; i++);
+
+	for(; maxlen && i<m; i++, maxlen--) {
 		c = cmdargs[i];
 
 		if (c == '\0')
@@ -684,8 +719,7 @@ pfmon_extract_cmdline(pid_t pid, char *cmdline, int maxlen)
 		else 
 			cmdline[n++] = c;
 	}
-	cmdline[n] = '\0';
-
+	cmdline[n-1] = '\0';
 	return 0;
 }
 
@@ -739,25 +773,23 @@ pfmon_backtrace(void)
 #endif /* CONFIG_PFMON_LIBUNWIND */
 
 int
-find_in_cpuinfo(char *entry, char **result)
+find_in_proc(FILE *fp1, char *entry, char **result)
 {
-	FILE *fp1;
 	char *buffer, *p, *value;
 	size_t n, len;
 
 	len = strlen(entry);
 	*result = buffer = NULL;
 
-	fp1 = fopen("/proc/cpuinfo", "r");
-	if (fp1 == NULL) return -1;
-
 	while(getline(&buffer, &n, fp1) != -1) {
 
 		/* skip  blank lines */
-		if (*buffer == '\n') continue;
+		if (*buffer == '\n')
+			continue;
 
 		p = strchr(buffer, ':');
-		if (p == NULL) goto end_it;	
+		if (p == NULL)
+			goto end_it;	
 
 		/* 
 		 * p+2: +1 = space, +2= first character
@@ -774,23 +806,71 @@ find_in_cpuinfo(char *entry, char **result)
 		}
 	}
 end_it:
-	if (buffer) free(buffer);
-	fclose(fp1);
+	if (buffer)
+		free(buffer);
 	return *result == NULL ? -1 : 0;
 }
 
+int
+find_in_cpuinfo(char *entry, char **result)
+{
+	FILE *fp1;
+	int ret;
+
+	fp1 = fopen("/proc/cpuinfo", "r");
+	if (fp1 == NULL)
+		return -1;
+
+	ret = find_in_proc(fp1, entry, result);
+
+	fclose(fp1);
+
+	return ret;
+}
+
 unsigned long
 pfmon_find_cpu_speed(void)
 {
 	float fl;
 	char *mhz;
 
-	if (find_in_cpuinfo("cpu MHz", &mhz) == -1) return 0;
+	if (find_in_cpuinfo("cpu MHz", &mhz) == -1) 
+	  if (find_in_cpuinfo("BogoMIPS", &mhz) == -1) 
+	    return 0;
 	sscanf(mhz, "%f", &fl);
 	free(mhz);
 	return lroundf(fl);
 }
 
+/*
+ * find thread groud id (tgid) using the thread id (tid)
+ */
+pid_t find_pid_attr(pid_t tid, char *attr)
+{
+	FILE *fp1;
+	char path[32];
+	char *result = NULL;
+	int ret;
+
+	snprintf(path, sizeof(path), "/proc/%d/status", tid);
+
+	fp1 = fopen(path, "r");
+	if (fp1 == NULL)
+		return -1;
+
+	ret = find_in_proc(fp1, attr, &result);
+
+	fclose(fp1);
+
+	if (ret == 0) {
+		ret = atoi(result);
+	}
+	if (result)
+		free(result);
+
+	return ret;
+}
+
 /*
  * There are several ways to implement this depending on:
  * 	- NPTL vs. LinuxThreads (pthread_setaffinity_np())
@@ -808,39 +888,39 @@ pfmon_find_cpu_speed(void)
  */
 
 int
-pfmon_set_affinity(pid_t pid, pfmon_cpumask_t mask)
+pfmon_set_affinity(pid_t pid, pfmon_bitmask_t mask)
 {
-	pfmon_cpumask_t phys_mask;
+	pfmon_bitmask_t phys_mask;
 	int phys_cpu;
 	int i;
 
 	memset(phys_mask, 0, sizeof(phys_mask));
 
-	for (i=0; i < PFMON_MAX_CPUS; i++) {
-		if (PFMON_CPUMASK_ISSET(mask, i)) {
+	for (i=0; i < options.online_cpus; i++) {
+		if (pfmon_bitmask_isset(mask, i)) {
 			phys_cpu = pfmon_cpu_virt_to_phys(i);
 			if (phys_cpu == -1) {
 				warning("logical CPU%d does not exist in current CPU set\n", i);
 				return -1;
 			}	
 			vbprintf("vCPU%d -> pCPU%d\n", i, phys_cpu);
-			PFMON_CPUMASK_SET(phys_mask, phys_cpu);
+			pfmon_bitmask_set(phys_mask, phys_cpu);
 		}
 	}
 	/*
 	 * actual kernel affinity call, only taskes physical processor number at this point
 	 */
-	return __pfmon_set_affinity(pid, sizeof(pfmon_cpumask_t), phys_mask);
+	return __pfmon_set_affinity(pid, sizeof(pfmon_bitmask_t), phys_mask);
 }
 
 int
 pfmon_pin_self(unsigned int cpu)
 {	
-	pfmon_cpumask_t mask; /* size of the mask limited by PFMON_MAX_CPUS */
+	pfmon_bitmask_t mask; /* size of the mask limited by PFMON_MAX_CPUS */
 
-	memset(mask, 0, sizeof(pfmon_cpumask_t));
+	memset(mask, 0, sizeof(pfmon_bitmask_t));
 
-	PFMON_CPUMASK_SET(mask, cpu);
+	pfmon_bitmask_set(mask, cpu);
 
 	return pfmon_set_affinity(gettid(), mask);
 }
@@ -919,38 +999,17 @@ pfmon_clone_sets(pfmon_event_set_t *list, pfmon_sdesc_t *sdesc)
 	sdesc->nsets = cnt;
 }
 
-int
-pfmon_validate_buf_size(size_t size)
+void
+pfmon_free_sets(pfmon_sdesc_t *sdesc)
 {
-	struct rlimit rlim_memlock;
-	int ret;
-
-	ret = getrlimit(RLIMIT_MEMLOCK, &rlim_memlock);
-	if (ret == -1) {
-		warning("cannot retrieve maximum memory useable by buffer\n");
-		return -1;
+	pfmon_event_set_t *set, *tmp;
+	for (set = sdesc->sets ; set; set = tmp) {
+		tmp = set->next;
+		free(set);
 	}
-
-	/*
-	 * that does not necessarily mean we will be ok because we may
-	 * have consumed some of that already
-	 */
-	if (size <= rlim_memlock.rlim_cur) return 0;
-
-	if (size > rlim_memlock.rlim_cur && size < rlim_memlock.rlim_max) {
-		warning("buffer size is too big for current resource limit on locked memory\n"
-			"you need to increase it to at least %lu\n", size);
-		return -1;
-	}
-
-	if (size > rlim_memlock.rlim_max) {
-		warning("buffer size %zu is too big, maximum size is %lu bytes\n",
-			size,
-			rlim_memlock.rlim_max);
-		return -1;
-	}
-	return 0;
+	sdesc->sets = NULL;
 }
+
 /*
  * on IA-64 with perfmon-2.0, we used to get the information from /proc/perfmon
  */
@@ -1037,7 +1096,7 @@ pfmon_cpu_virt_to_phys(int virt_cpu)
 	}
 
 	for(i=0, ncpus=0; i < PFMON_MAX_CPUS; i++) {
-		if (PFMON_CPUMASK_ISSET(options.phys_cpu_mask, i)) {
+		if (pfmon_bitmask_isset(options.phys_cpu_mask, i)) {
 			if (virt_cpu < ++ncpus) {
 				return i;
 			}
@@ -1051,26 +1110,78 @@ pfmon_collect_affinity(void)
 {
 	int ret;
 
-	ret = __pfmon_get_affinity(getpid(), sizeof(pfmon_cpumask_t), options.phys_cpu_mask);
+	ret = __pfmon_get_affinity(getpid(), sizeof(pfmon_bitmask_t), options.phys_cpu_mask);
 	if (ret == -1) {
 		warning("cannot get affinity: %s\n", strerror(errno));
 	}
 	return ret;
 }
 
+static size_t pfmon_get_perfmon_smpl_mem(void)
+{
+	int fd;
+	size_t max, used;
+	char number[32];
+	ssize_t ret;
+
+	if (options.opt_is22 == 0)
+		return (size_t)-1;
+
+	/*
+	 * extract amount of memory currently used
+	 */
+	fd = open("/sys/kernel/perfmon/smpl_buffer_mem_cur", O_RDONLY);
+	if (fd == -1)
+		return (size_t)-1;
+	memset(number, 0, sizeof(number));
+	ret = read(fd, number, sizeof(number));
+	close(fd);
+
+	if (ret != sizeof(number))
+		return (size_t)-1;
+
+	used = strtoul(number, NULL, 0);
+
+	/*
+	 * extract max amount of memory possible
+	 */
+	fd = open("/sys/kernel/perfmon/smpl_buffer_mem_max", O_RDONLY);
+	if (fd == -1)
+		return (size_t)-1;
+	memset(number, 0, sizeof(number));
+	ret = read(fd, number, sizeof(number));
+	close(fd);
+
+	if (ret != sizeof(number))
+		return (size_t)-1;
+
+	max = strtoul(number, NULL, 0);
+	if (max == -1)
+		max = sysconf(_SC_PHYS_PAGES)*getpagesize();
+
+	return max - used;
+}
+
 int
-pfmon_compute_smpl_entries(size_t hdr_sz, size_t entry_sz)
+pfmon_compute_smpl_entries(size_t hdr_sz, size_t entry_sz, size_t slack)
 {
 	int ret;
 	struct rlimit rlim_memlock;
-	unsigned int max_num_pmds;
-#define ENTRY_SIZE(npmd, ez)	((ez)+((npmd)*sizeof(uint64_t)))
-
-	pfm_get_num_pmds(&max_num_pmds);
+	unsigned long max_entries = 0, orig_smpl_entries = 0;
+	size_t pfm_avail, memlock_avail, mem_avail, pgsz;
 
 	/*
-	 * XXX: need to check against global perfmon buffer size limit
+	 * hack to work around a bug in older v2.0 kernel
+	 * implementations for IA-64
 	 */
+	if (options.opt_is22 == 0) {
+		if (options.smpl_entries == 0)
+			options.smpl_entries = 2048UL;
+		return 0;
+	}
+
+	pgsz = getpagesize();
+	pfm_avail = pfmon_get_perfmon_smpl_mem();
 
 	ret = getrlimit(RLIMIT_MEMLOCK, &rlim_memlock);
 	if (ret == -1) {
@@ -1078,16 +1189,60 @@ pfmon_compute_smpl_entries(size_t hdr_sz, size_t entry_sz)
 		return -1;
 	}
 
-	if (rlim_memlock.rlim_max != RLIM_INFINITY)
-		options.smpl_entries = (rlim_memlock.rlim_max - hdr_sz) / ENTRY_SIZE(max_num_pmds, entry_sz);
+	vbprintf("max=%zu cur=%lu\n", rlim_memlock.rlim_max, rlim_memlock.rlim_cur);
+	if (rlim_memlock.rlim_cur != RLIM_INFINITY)
+		memlock_avail = rlim_memlock.rlim_cur;
 	else
-		options.smpl_entries = PFMON_DFL_SMPL_ENTRIES;
+		memlock_avail = sysconf(_SC_PHYS_PAGES)*pgsz;
 
-	DPRINT(("memlock limit=%lu smpl_buffer_entries=%lu buffer_size=%zu max_pmds=%u\n",
-		rlim_memlock.rlim_max,
+	/* compute maximum memory available */
+	mem_avail = memlock_avail < pfm_avail ? memlock_avail : pfm_avail;
+
+	if (options.opt_syst_wide)
+		mem_avail /= options.selected_cpus;
+
+	/* round down to multiple of page size */
+	mem_avail /= pgsz;
+	mem_avail *= pgsz;
+
+	if (mem_avail < (hdr_sz + slack))
+		goto no_solution;
+
+	mem_avail -= hdr_sz + slack;
+
+	//max_entries = (mem_avail - hdr_sz - slack) / entry_sz;
+	max_entries = mem_avail / entry_sz;
+no_solution:
+	orig_smpl_entries = options.smpl_entries;
+
+	if (options.smpl_entries == 0) {
+		options.smpl_entries = max_entries < PFMON_DFL_SMPL_ENTRIES ? max_entries : PFMON_DFL_SMPL_ENTRIES;
+	} else if (options.smpl_entries > max_entries) {
+		options.smpl_entries = max_entries;
+	}
+
+		
+	vbprintf("locked_mem_avail=%zu pfm_mem_avail=%zu mem_avail=%zu ncpus=%lu max_entries=%lu smpl_entries=%lu orig_smpl_entries=%lu pgsz=%zu hdrsz=%zu entrysz=%zu slack=%zu\n",
+		memlock_avail,
+		pfm_avail,
+		mem_avail,
+		options.selected_cpus,
+		max_entries,
 		options.smpl_entries,
-		options.smpl_entries*ENTRY_SIZE(max_num_pmds, entry_sz), max_num_pmds));
-	return 0;
+		orig_smpl_entries,
+		pgsz,
+		hdr_sz,
+		entry_sz,
+		slack);
+
+	if ((options.smpl_entries < orig_smpl_entries && orig_smpl_entries) || !max_entries)
+		warning("Sampling buffer entries limited to %lu (from %u) due to lack of memory resource.\n"
+		 	"Check the locked memory limit with ulimit or limit.\n"
+		 	"Check /etc/limits.conf for memorylocked hard and soft limits.\n"
+			"Check the perfmon global buffer limit in /sys/kernel/perfmon/buffer_size_max.\n",
+			options.smpl_entries, orig_smpl_entries);
+	
+	return options.smpl_entries ? 0 : -1;
 }
 
 int
@@ -1095,7 +1250,7 @@ pfmon_detect_unavail_pmcs(pfmlib_regmask_t *r_pmcs)
 {
 	pfarg_ctx_t ctx;
 	pfmon_setinfo_t	setf;
-	int ret, i, j, fd;
+	int ret, i, j, fd, error;
 
 	memset(r_pmcs, 0, sizeof(*r_pmcs));
 
@@ -1108,13 +1263,11 @@ pfmon_detect_unavail_pmcs(pfmlib_regmask_t *r_pmcs)
 	/*
 	 * must use kernel interface directly
 	 */
-	ret = pfm_create_context(&ctx, NULL, 0);
-	if (ret)
+	fd = pfm_create_context(&ctx, NULL, NULL, 0);
+	if (fd == -1)
 		return 0;
 
-	fd = ctx.ctx_fd;
-
-	ret = pfmon_getinfo_evtsets(fd, &setf, 1);
+	ret = pfmon_getinfo_evtsets(fd, &setf, 1, &error);
 	if (ret == 0) {
 		for(i=0; i < PFM_PMC_BV; i++) {
 			for(j=0; j < 64; j++) {
