@@ -311,7 +311,7 @@ pfmon_arch_initialize(void)
 	 * XXX: temporary hack. pfmon allows both code and data
 	 * triggers to be set at the same time. Yet there is only
 	 * one set of DB registers. We should really use an allocator
-	 * but for nwo split registers into two sets and hack a
+	 * but for now split registers into two sets and hack a
 	 * base in the code
 	 */
 	options.nibrs = 2;
@@ -326,23 +326,13 @@ pfmon_enable_all_breakpoints(pid_t pid)
 	long r;
 
 	offset = offsetof(struct user, u_debugreg[7]);
-
-	dr7.val = ptrace(PTRACE_PEEKUSER, pid, offset, 0);
-	if (dr7.val == -1) return -1;
-
-	DPRINT(("dr7: peek=0x%lx\n", dr7.val));
-
-	/* set all lX bits */
-	dr7.reg.l0 = 1;
-	dr7.reg.l1 = 1;
-	dr7.reg.l2 = 1;
-	dr7.reg.l3 = 1;
+	dr7.val=0x355;
 
 	DPRINT(("dr7: poke=0x%lx offs=%ld\n", dr7.val, offset));
 
 	r = ptrace(PTRACE_POKEUSER, pid, offset, (void *)dr7.val);
 	if (r == -1) {
-		warning("cannot poke dr7\n");
+		DPRINT(("poke ptrace=%d pid=%d\n", errno, pid));
 		return -1;
 	}
 	return 0;
@@ -356,23 +346,13 @@ pfmon_disable_all_breakpoints(pid_t pid)
 	long r;
 
 	offset = offsetof(struct user, u_debugreg[7]);
+	dr7.val=0x300;
 
-	r = ptrace(PTRACE_PEEKUSER, pid, offset, &dr7);
-	if (r == -1) return -1;
-
-	DPRINT(("dr7=0x%lx\n", dr7.val));
-
-	/* set all lX bits */
-	dr7.reg.l0 = 0;
-	dr7.reg.l1 = 0;
-	dr7.reg.l2 = 0;
-	dr7.reg.l3 = 0;
-
-	DPRINT(("dr7: poke=0x%lx offs=%ld\n", dr7.val, offset));
+	DPRINT(("dr7: offfset=0x%lx poke=0x%lx\n", offset, dr7.val));
 
 	r = ptrace(PTRACE_POKEUSER, pid, offset, (void *)dr7.val);
 	if (r == -1) {
-		warning("cannot poke dr7\n");
+		DPRINT(("poke ptrace=%d pid=%d\n", errno, pid));
 		return -1;
 	}
 	return 0;
@@ -499,26 +479,17 @@ pfmon_get_return_pointer(pid_t pid, unsigned long *rp)
  * the kernel interface which has been stable for quite some time now.
  */
 int
-__pfmon_set_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask)
+__pfmon_set_affinity(pid_t pid, size_t size, pfmon_bitmask_t mask)
 {
 	return (int)syscall(__NR_sched_setaffinity, pid, size, mask);
 }
 
 int
-__pfmon_get_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask)
+__pfmon_get_affinity(pid_t pid, size_t size, pfmon_bitmask_t mask)
 {
 	return (int)syscall(__NR_sched_getaffinity, pid, size, mask);
 }
 
-int
-pfmon_get_timestamp(uint64_t *t)
-{
-	uint64_t tmp;
-     	__asm__ __volatile__("rdtsc" : "=A" (tmp));
-	*t = tmp;
-	return 0;
-}
-	
 void
 pfmon_print_simple_cpuinfo(FILE *fp, const char *msg)
 {
