@@ -40,7 +40,7 @@
  * PSR bits controlling debug breakpoints
  */
 #define PSR_DB_BIT	24	/* enable/disble all breakpoints */
-#define PSR_ID_BIT	37	/* instruction disblae debug fault for one instruction */
+#define PSR_ID_BIT	37	/* instruction disable debug fault for one instruction */
 #define PSR_DD_BIT	39	/* data disable debug fault for one instruction */
 
 /* 
@@ -786,29 +786,13 @@ pfmon_get_return_pointer(pid_t pid, uintptr_t *rp)
  * the kernel interface which has been stable for quite some time now.
  */
 int
-__pfmon_set_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask)
+__pfmon_set_affinity(pid_t pid, size_t size, pfmon_bitmask_t mask)
 {
 	return (int)syscall(__NR_sched_setaffinity, pid, size, mask);
 }
 
 int
-__pfmon_get_affinity(pid_t pid, size_t size, pfmon_cpumask_t mask)
+__pfmon_get_affinity(pid_t pid, size_t size, pfmon_bitmask_t mask)
 {
 	return (int)syscall(__NR_sched_getaffinity, pid, size, mask);
 }
-
-
-int
-pfmon_get_timestamp(uint64_t *t)
-{
-	uint64_t tmp;
-#if defined(PFMON_USING_INTEL_ECC_COMPILER)
-	tmp = __getIndReg(_IA64_REG_INDR_AR, _IA64_REG_AR_ITC);
-#elif defined(__GNUC__)
-	__asm__ __volatile__("mov %0=ar.itc" : "=r"(tmp) :: "memory");
-#else /* !GNUC nor INTEL_ECC */
-#error "need to define a set of compiler-specific macros"
-#endif
-	*t = tmp;
-	return 0;
-}
