@@ -1,6 +1,5 @@
 /*
- * compact_ia64_smpl.c - compact output sampling module for
- *                     legacy mode perfmon2 on all IA-64 PMUs
+ * compact_ia64_smpl.c - compact output sampling module for perfmon v2.0
  *
  * Copyright (c) 2002-2006 Hewlett-Packard Development Company, L.P.
  * Contributed by Stephane Eranian <eranian@hpl.hp.com>
@@ -127,15 +126,21 @@ compact_print_header(pfmon_sdesc_t *sdesc)
 	return 0;
 }
 
-pfmon_smpl_module_t compact_ia64_old_smpl_module ={
+pfmon_smpl_module_t compact_old_smpl_module;
+static void compact_initialize_mask(void)
+{
+	pfmon_bitmask_setall(compact_old_smpl_module.pmu_mask);
+}
+
+pfmon_smpl_module_t compact_old_smpl_module ={
 	.name		    = "compact",
-	.pmu_mask	    = PFMON_PMU_MASK(PFMLIB_GEN_IA64_PMU),
 	.description	    = "Column-style raw values (perfmon v2.0)",
 	.process_samples    = compact_process_samples,
 	.print_header       = compact_print_header,
 	.init_ctx_arg	    = default_smpl_init_ctx_arg,
 	.check_version	    = default_smpl_check_version,
 	.check_new_samples  = default_smpl_check_new_samples,
+	.initialize_mask    = compact_initialize_mask,
 	.flags              = PFMON_SMPL_MOD_FL_LEGACY,
 	.uuid		    = PFM_DEFAULT_SMPL_UUID,
 };
