@@ -128,11 +128,17 @@ compact_print_header(pfmon_sdesc_t *sdesc)
 	return 0;
 }
 
+pfmon_smpl_module_t compact_smpl_module;
+static void compact_initialize_mask(void)
+{
+	pfmon_bitmask_setall(compact_smpl_module.pmu_mask);
+}
+
 pfmon_smpl_module_t compact_smpl_module ={
 	.name		    = SMPL_MOD_NAME,
-	.pmu_mask	    = PFMON_ANY_PMU_MASK,
 	.description	    = "Column-style raw values",
+	.initialize_mask    = compact_initialize_mask,
 	.process_samples    = compact_process_samples,
 	.print_header       = compact_print_header,
-	.uuid		    = PFM_DFL_SMPL_UUID,
+	.fmt_name	    = PFM_DFL_SMPL_NAME
 };
