@@ -797,17 +797,26 @@ dear_hist_terminate_session(pfmon_sdesc_t *sdesc)
 			|PFMON_PMU_MASK(PFMLIB_ITANIUM2_PMU) \
 			|PFMON_PMU_MASK(PFMLIB_MONTECITO_PMU))
 
+pfmon_smpl_module_t dear_hist_ia64_smpl_module;
+static void
+dear_hist_initialize_mask(void)
+{
+	pfmon_bitmask_set(dear_hist_ia64_smpl_module.pmu_mask, PFMLIB_ITANIUM_PMU);
+	pfmon_bitmask_set(dear_hist_ia64_smpl_module.pmu_mask, PFMLIB_ITANIUM2_PMU);
+	pfmon_bitmask_set(dear_hist_ia64_smpl_module.pmu_mask, PFMLIB_MONTECITO_PMU);
+}
+
 pfmon_smpl_module_t dear_hist_ia64_smpl_module ={
 	.name		    = "dear-hist",
-	.pmu_mask	    = ALL_DEAR_PMUS,
 	.description	    = "Data EAR-based cache/tlb misses histograms",
 	.process_samples    = dear_hist_process_samples,
 	.show_options       = dear_hist_show_options,
 	.parse_options      = dear_hist_parse_options,
+	.initialize_mask    = dear_hist_initialize_mask,
 	.initialize_module  = dear_hist_initialize_module,
 	.initialize_session = dear_hist_initialize_session,
 	.terminate_session  = dear_hist_terminate_session,
 	.print_header       = dear_hist_print_header,
 	.validate_events    = dear_hist_validate_events,
-	.uuid		    = PFM_DFL_SMPL_UUID,
+	.fmt_name	    = PFM_DFL_SMPL_NAME
 };
