@@ -1,5 +1,5 @@
 /*
- * detailed_ia64_old_smpl.c - detailed sampling module for the all IA-64 PMU models
+ * detailed_old_smpl.c - detailed sampling module for all PMU models
  *                            using perfmon v2.0 interface
  *
  * Copyright (c) 2005-2006 Hewlett-Packard Development Company, L.P.
@@ -24,8 +24,11 @@
 #include "pfmon_smpl_ia64_old.h"
 #include <perfmon/perfmon_default_smpl.h> /* IA-64 compatibility ONLY! */
 
-static int has_btb; /* set to 1 if PMU has Branch Trace Buffer or Execution Trace Buffer */
+static int has_btb; /* set to 1 if IA-64 and has BTB/ETB */
 
+/*
+ * register helper functions
+ */
 extern int print_ita_reg(pfmon_event_set_t *set, pfmon_smpl_desc_t *csmpl, int rnum, unsigned long val);
 extern int print_ita2_reg(pfmon_event_set_t *set, pfmon_smpl_desc_t *csmpl, int rnum, unsigned long val);
 extern int print_mont_reg(pfmon_event_set_t *set, pfmon_smpl_desc_t *csmpl, int rnum, unsigned long val);
@@ -33,27 +36,26 @@ extern int print_mont_reg(pfmon_event_set_t *set, pfmon_smpl_desc_t *csmpl, int
 static int (*print_reg)(pfmon_event_set_t *set, pfmon_smpl_desc_t *csmpl, int rnum, unsigned long val);
 
 static int
-print_ia64_reg(pfmon_event_set_t *set, pfmon_smpl_desc_t *csmpl, int rnum, unsigned long val)
+dfl_print_reg(pfmon_event_set_t *set, pfmon_smpl_desc_t *csmpl, int rnum, unsigned long val)
 {
 	return fprintf(csmpl->smpl_fp, "\tPMD%-3d:0x%016lx\n", rnum, val);
 }
 
 static int
-detailed_ia64_process_samples(pfmon_sdesc_t *sdesc)
+detailed_process_samples(pfmon_sdesc_t *sdesc)
 {
 	pfm_default_smpl_hdr_t *hdr;
 	pfm_default_smpl_entry_t *ent;
 	pfmon_smpl_desc_t *csmpl;
 	pfmon_event_set_t *active_set;
 	FILE *fp;
-	unsigned long *msk;
-	size_t count, i;
-	uint64_t entry;
+	uint64_t *msk;
+	uint64_t count, i, entry;
 	void *hash_desc;
-	unsigned long *reg;
+	uint64_t *reg;
+	uint16_t ovfl_pmd;
 	unsigned int j, n;
 	int ret;
-	unsigned short ovfl_pmd; /* undefined PMD */
 
 	csmpl      = &sdesc->csmpl;
 	hdr        = csmpl->smpl_hdr;
@@ -80,8 +82,9 @@ detailed_ia64_process_samples(pfmon_sdesc_t *sdesc)
 			ent->set);
 
 		pfmon_print_address(fp, hash_desc, &options.primary_syms, PFMON_TEXT_SYMBOL, ent->ip);
+		fputc('\n', fp);
 
-		reg = (unsigned long *)(ent+1);
+		reg = (uint64_t *)(ent+1);
 
 		msk = active_set->rev_smpl_pmds[ovfl_pmd].smpl_pmds;
 		n   = active_set->rev_smpl_pmds[ovfl_pmd].num_smpl_pmds;
@@ -126,7 +129,7 @@ error:
  * Allocate space for the optional BTB trace buffer
  */
 static int
-detailed_ia64_initialize_session( pfmon_smpl_desc_t *csmpl)
+detailed_initialize_session( pfmon_smpl_desc_t *csmpl)
 {
 	unsigned int num_pmds;
 
@@ -142,7 +145,7 @@ detailed_ia64_initialize_session( pfmon_smpl_desc_t *csmpl)
 }
 
 static int
-detailed_ia64_terminate_session(pfmon_sdesc_t *sdesc)
+detailed_terminate_session(pfmon_sdesc_t *sdesc)
 {
 	pfmon_smpl_desc_t *csmpl = &sdesc->csmpl;
 	if (csmpl->data) free(csmpl->data);
@@ -150,7 +153,7 @@ detailed_ia64_terminate_session(pfmon_sdesc_t *sdesc)
 }
 
 static int
-detailed_ia64_initialize_module(void)
+detailed_initialize_module(void)
 {
 	switch(options.pmu_type) {
 		case PFMLIB_ITANIUM_PMU:
@@ -166,19 +169,25 @@ detailed_ia64_initialize_module(void)
 			print_reg = print_mont_reg;
 			break;
 		default:
-			print_reg = print_ia64_reg;
+			print_reg = dfl_print_reg;
 	}
 	return 0;
 }
 
-pfmon_smpl_module_t detailed_ia64_old_smpl_module = {
-	.name		    = "detailed-ia64",
-	.pmu_mask	    = PFMON_PMU_MASK(PFMLIB_GEN_IA64_PMU),
-	.description	    = "Itanium processors detailed sampling",
-	.process_samples    = detailed_ia64_process_samples,
-	.initialize_session = detailed_ia64_initialize_session,
-	.terminate_session  = detailed_ia64_terminate_session,
-	.initialize_module  = detailed_ia64_initialize_module,
+pfmon_smpl_module_t detailed_old_smpl_module;
+static void detailed_initialize_mask(void)
+{
+	pfmon_bitmask_setall(detailed_old_smpl_module.pmu_mask);
+}
+
+pfmon_smpl_module_t detailed_old_smpl_module = {
+	.name		    = "detailed",
+	.description	    = "decode register content",
+	.process_samples    = detailed_process_samples,
+	.initialize_mask    = detailed_initialize_mask,
+	.initialize_session = detailed_initialize_session,
+	.terminate_session  = detailed_terminate_session,
+	.initialize_module  = detailed_initialize_module,
 	.init_ctx_arg	    = default_smpl_init_ctx_arg,
 	.check_version	    = default_smpl_check_version,
 	.check_new_samples  = default_smpl_check_new_samples,
