@@ -320,6 +320,9 @@ inst_hist_show_results(pfmon_smpl_desc_t *smpl)
 	int need_resolve;
 	char buf[32];
 
+	if (hash_desc == NULL)
+		return -1;
+
 	pfmon_hash_num_entries(hash_desc, &num_entries);
 
 	tab = (hash_data_t **)malloc(sizeof(hash_data_t *)*num_entries);
@@ -400,13 +403,20 @@ inst_hist_terminate_session(pfmon_sdesc_t *sdesc)
 	return 0;
 }
 
-pfmon_smpl_module_t inst_hist_ia64_old_smpl_module ={
+pfmon_smpl_module_t inst_hist_old_smpl_module;
+static void
+inst_hist_initialize_mask(void)
+{
+	pfmon_bitmask_setall(inst_hist_old_smpl_module.pmu_mask);
+}
+
+pfmon_smpl_module_t inst_hist_old_smpl_module ={
 	.name		    = "inst-hist",
-	.pmu_mask	    = PFMON_PMU_MASK(PFMLIB_GEN_IA64_PMU),
 	.description	    = "IP-based histogram",
 	.process_samples    = inst_hist_process_samples,
 	.show_options       = inst_hist_show_options,
 	.parse_options      = inst_hist_parse_options,
+	.initialize_mask    = inst_hist_initialize_mask,
 	.initialize_module  = inst_hist_initialize_module,
 	.initialize_session = inst_hist_initialize_session,
 	.terminate_session  = inst_hist_terminate_session,
