@@ -28,8 +28,9 @@
 #define	SMPL_MOD_NAME "inst-hist"
 
 typedef struct {
-	unsigned long	show_top_num;
-	int		per_function;
+	unsigned long	show_top_num; /* show only to N functions */
+	int		per_function; /* collapse samples per function */
+	int		nopid;	      /* syswide, okay to ignore pid when PLM > 0 */
 } inst_hist_options_t;
 
 typedef struct {
@@ -46,19 +47,13 @@ typedef struct {
 
 static inst_hist_options_t inst_hist_options;
 
-/*
- * forward declaration
- */
-pfmon_smpl_module_t inst_hist_i386_p6_smpl_module;
-
 static int
 inst_hist_process_samples(pfmon_sdesc_t *sdesc)
 {
 	pfm_dfl_smpl_hdr_t *hdr;
 	pfm_dfl_smpl_entry_t *ent;
 	pfmon_smpl_desc_t *csmpl = &sdesc->csmpl;
-	unsigned long entry;
-	size_t count, i;
+	uint64_t entry, count;
 	void *hash_desc, *data;
 	hash_data_t *hash_entry;
 	pfmon_hash_key_t key;
@@ -69,11 +64,22 @@ inst_hist_process_samples(pfmon_sdesc_t *sdesc)
 	ent        = (pfm_dfl_smpl_entry_t *)(hdr+1);
 	entry      = options.opt_aggr ? *csmpl->aggr_count : csmpl->entry_count;
 	count      = hdr->hdr_count;
+	
+	/*
+	 * when aggregation is used, for are guaranteed sequential access to
+	 * this routine by higher level lock
+	 */
+	if (options.opt_aggr) {
+		*csmpl->aggr_count += count;
+	} else {
+		csmpl->entry_count += count;
+	}
+	csmpl->last_count = count;
+	csmpl->last_ovfl = hdr->hdr_overflows;
 
 	DPRINT(("hdr_count=%"PRIu64" hdr=%p csmpl=%p\n", count, hdr, csmpl));
 
-	for(i=0; i < count; i++) {
-
+	while(count--) {
 		DPRINT(("entry %"PRIu64" PID:%d CPU:%d STAMP:0x%"PRIx64" IIP: %llx\n",
 			entry,
 			ent->pid,
@@ -104,19 +110,6 @@ inst_hist_process_samples(pfmon_sdesc_t *sdesc)
 		ent++;
 		entry++;
 	}
-	
-	/*
-	 * when aggregation is used, for are guaranteed sequential access to
-	 * this routine by higher level lock
-	 */
-	if (options.opt_aggr) {
-		*csmpl->aggr_count += count;
-	} else {
-		csmpl->entry_count += count;
-	}
-	csmpl->last_count = count;
-	csmpl->last_ovfl = hdr->hdr_overflows;
-
 	return 0;
 }
 
@@ -124,6 +117,7 @@ inst_hist_process_samples(pfmon_sdesc_t *sdesc)
 static struct option inst_hist_cmd_options[]={
 	{ "smpl-show-top", 1, 0, IDX},
 	{ "smpl-show-function", 0, &inst_hist_options.per_function, 1},
+	{ "smpl-ignore-pids", 0, &inst_hist_options.nopid, 1},
 	{ NULL, 0, 0, 0}
 };
 #undef IDX
@@ -132,9 +126,11 @@ static void
 inst_hist_show_options(void)
 {
 	printf( "\t--smpl-show-top=n\t\tShow only the top n entries in the\n"
-		"\t\t\t\t\t histogram (default: all entries).\n"
+		"\t\t\t\t\thistogram (default: all entries).\n"
 		"\t--smpl-show-function\t\tShow per-function histograms (default\n"
-		"\t\t\t\t\t per address).\n"
+		"\t\t\t\t\tper address).\n"
+		"\t--smpl-ignore-pids\t\tIgnore pids when priv level is different from\n"
+		"\t\t\t\t\tk/0 (system wide)\n"
 	);
 }
 
@@ -175,6 +171,8 @@ inst_hist_initialize_module(void)
 	return pfmon_register_options(inst_hist_cmd_options, sizeof(inst_hist_cmd_options));
 }
 
+
+
 static int
 inst_hist_print_header(pfmon_sdesc_t *sdesc)
 {
@@ -204,11 +202,14 @@ inst_hist_validate_events(pfmon_event_set_t *set)
 	 * in system-wide mode, we currently only support kernel level ONLY monitoring
 	 * because we do not manage the process pid inside the hash table. This could
 	 * cause confusion and lead to interpretation mistakes for the final histogram.
+	 * To force, and if you know what you are doing,  use --smpl-nopid option
 	 */
-	if (options.opt_syst_wide) {
+	if (options.opt_syst_wide && inst_hist_options.nopid == 0) {
 		for(i=0; i < set->inp.pfp_event_count; i++) {
 			if (set->inp.pfp_events[i].plm != PFM_PLM0) {
-				warning("in system-wide mode, the sampling module only works when capturing kernel level ONLY events\n");
+				warning("In system-wide mode, the pid information is ignored. As a consequence,\n"
+					"this sampling format only works when capturing kernel level ONLY events.\n"
+					"Use --smpl-ignore-pid to override\n");
 				return -1;
 			}
 		}
@@ -317,6 +318,9 @@ inst_hist_show_results(pfmon_smpl_desc_t *smpl)
 	int need_resolve;
 	char buf[32];
 
+	if (hash_desc == NULL)
+		return -1;
+
 	pfmon_hash_num_entries(hash_desc, &num_entries);
 
 	tab = (hash_data_t **)malloc(sizeof(hash_data_t *)*num_entries);
@@ -379,6 +383,10 @@ inst_hist_show_results(pfmon_smpl_desc_t *smpl)
 
 	free(tab);
 
+	if (options.opt_verbose) {
+		vbprintf("total samples: %"PRIu64"\nsamples hash stats: ", smpl->entry_count);
+		pfmon_hash_stats(hash_desc, fp);
+	}
 	return 0;
 }
 
@@ -397,18 +405,25 @@ inst_hist_terminate_session(pfmon_sdesc_t *sdesc)
 	return 0;
 }
 
+pfmon_smpl_module_t inst_hist_smpl_module;
+static void
+inst_hist_initialize_mask(void)
+{
+	pfmon_bitmask_setall(inst_hist_smpl_module.pmu_mask);
+}
+
 pfmon_smpl_module_t inst_hist_smpl_module ={
 	.name		    = "inst-hist",
-	.pmu_mask	    = PFMON_ANY_PMU_MASK,
 	.description	    = "IP-based histogram",
 	.process_samples    = inst_hist_process_samples,
 	.show_options       = inst_hist_show_options,
 	.parse_options      = inst_hist_parse_options,
+	.initialize_mask    = inst_hist_initialize_mask,
 	.initialize_module  = inst_hist_initialize_module,
 	.initialize_session = inst_hist_initialize_session,
 	.terminate_session  = inst_hist_terminate_session,
 	.print_header       = inst_hist_print_header,
 	.validate_events    = inst_hist_validate_events,
 	.flags		    = PFMON_SMPL_MOD_FL_DEF_SYM,
-	.uuid		    = PFM_DFL_SMPL_UUID,
+	.fmt_name	    = PFM_DFL_SMPL_NAME
 };
